using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;

namespace Bustools1553
{
   namespace DataTypes
   {
      // a struct, not a class, because it is contained within other classes
      [StructLayout(LayoutKind.Sequential, Pack=2)]
      public struct BT_FIFO
      {
         public Int32 event_type;   // EVENT_ definitions below.
         public Int32 buffer_off;   // Byte offset of message buffer which caused event
         public Int32 rtaddress;    // Terminal address of message
         public Int32 transrec;     // Non-zero if transmit message, zero for receive
         public Int32 subaddress;   // Subaddress of message
         public Int32 wordcount;    // Word count of message; 0-31; 0 indicates 32 words
                                    // unless mode code (then indicates mode code number)
         public Int32 bufferID;     // Buffer ID number or message ID number.
         public UInt32 int_status;  // Contains the INT_STATUS.      

         public BT_FIFO(int dummy)
         {
            event_type = 0;   
            buffer_off = 0;   
            rtaddress = 0;    
            transrec = 0;     
            subaddress = 0;   
            wordcount = 0;    
            bufferID = 0;     
            int_status = 0;    
         }
      };

      // the delegate defines the signature of a callback function.  we specify the actual callback function
      // when we create a new instance of the delegate.
      public delegate Int32 API_INT_FIFO_Callback(UInt32 cardnum, IntPtr intFifoAddr);

      [StructLayout(LayoutKind.Sequential, Pack=2)]
      public class API_INT_FIFO
      {
         public API_INT_FIFO_Callback callbackFunc;   // Function should return API_SUCCESS if thread is to continue operation, any other value will cause the thread and event object to be distroyed.

         public Int32 iPriority = 0;      // THREAD_PRIORITY_* as defined for Windows functionSetThreadPriority() or zero for normal priority.
         public UInt32 dwMilliseconds = 0; //Thread time-out interval in milliseconds or INFINITE Mask to request startup and shutdown notification:
         public Int32 iNotification = 0;  // CALL_STARTUP if function to be called at creation
                                          // of thread, CALL_SHUTDOWN if function is to be
                                          // called upon distruction of thread.  "OR" both
                                          // together to enable notification on both events.
                                          // See "bForceShutdown" and "bForceStartup" below.
         [MarshalAs(UnmanagedType.ByValArray, SizeConst = 8)]
         public Int32[] nUser;            // Spare variables for use by the user
         [MarshalAs(UnmanagedType.ByValArray, SizeConst = 8)]
         public IntPtr[] pUser;            // Spare variables for use by the user

         public UInt32 FilterType = 0;    // One or more EVENT_ definitions, "or'ed" together.

         [MarshalAs(UnmanagedType.ByValArray, SizeConst = 2048)]
         private UInt32[] filterMask;

         public Int32 bForceShutdown  = 0;  // 1 - Thread is being shutdown, -1 complete    (RO) $
         public Int32 bForceStartup  = 0;  // 1 - Thread is being started, 0 complete      (RO) $
         public Int32 nPtrIndex  = 0;  // Index into API pointer table                 (RO) $
         public UInt32 cardnum  = 0;  // Card number associated with this thread.     (RO) $
         public UInt32 numEvents  = 0;  // Total number of events, including overflows  (RW) $
         public UInt32 queue_oflow  = 0;  // Count incremented by API when FIFO overflows (RW) $
         public IntPtr hThread = new IntPtr(0);  // Handle to thread                             (RO) $
         public IntPtr hEvent  = new IntPtr(0);  // Handle to event object                       (RO) $
         public IntPtr hkEvent = new IntPtr(0);  // Kernel mode handle to event object           (RO) $
         public UInt32 lThreadId  = 0;  // ID of new user interrupt thread.             (RO) $
         public Int32 head_index  = 0;  // Index of element being added to queue (0->63)(RO) $
         public Int32 tail_index  = 0;  // Index of element to be removed from queue    (RW) $
         public Int32 mask_index  = 0;  // Mask for wrapping head and tail pointers     (RO) $

         [MarshalAs(UnmanagedType.ByValArray, SizeConst = Constants.MAX_FIFO_LEN)]
         public BT_FIFO[] fifo;

         // SizeConst is 32 * 2 * 32
         [MarshalAs(UnmanagedType.ByValArray, SizeConst = 2048)]
         private UInt32[] eventMask;

         public UInt32 eventInit = 0;
         public IntPtr hMutex = new IntPtr(0);
         public UInt32 timeout = 0;

         private Int32 GetIndex(int rt, int tr, int sa)
         {
            return (rt * 64) + (tr * 32) + sa;
         }

         public UInt32 GetEventMask(int rt, int tr, int sa)
         {
            return eventMask[GetIndex(rt, tr, sa)];
         }

         public void SetEventMask(int rt, int tr, int sa, UInt32 value)
         {
            eventMask[GetIndex(rt, tr, sa)] = value;
         }

         public UInt32 GetFilterMask(int rt, int tr, int sa)
         {
            return filterMask[GetIndex(rt, tr, sa)];
         }

         public void SetFilterMask(int rt, int tr, int sa, UInt32 value)
         {
            filterMask[GetIndex(rt, tr, sa)] = value;
         }

         // constructor
         public API_INT_FIFO()
         {
            fifo = new BT_FIFO[Constants.MAX_FIFO_LEN];
            for (int i = 0; i < Constants.MAX_FIFO_LEN; i++) fifo[i] = new BT_FIFO();

            eventMask = new UInt32[32 * 2 * 32];
            filterMask = new UInt32[32 * 2 * 32];
            for (int i = 0; i < (32 * 2 * 32); i++)
            {
               eventMask[i] = 0;
               filterMask[i] = 0;
            }

            nUser = new Int32[8];
            pUser = new IntPtr[8];
            for (int i = 0; i < 8; i++)
			{
				nUser[i] = 0;
				pUser[i] = new IntPtr(0);	
			}
         }
      }

      // a struct, not a class, because it is contained within other classes
      [StructLayout(LayoutKind.Sequential, Pack=2)]
      public struct BT_1553_COMMAND_WORD
      {
         private UInt16 _value;

         // wcount:5 bits LSB, subaddr:5 bits, tran_rec:1 bit, rtaddr:5 bits, MSB
         public UInt16 wcount
         {
            set { _value &= 0xFFE0; _value |= (UInt16)(value & 0x1F); }
            get { return (UInt16)(_value & 0x1F); }
         }
         public UInt16 subaddr
         {
            set { _value &= 0xFC1F; _value |= (UInt16)((value & 0x1F) << 5); }
            get { return (UInt16)((_value & 0x3E0) >> 5); }
         }
         public UInt16 tran_rec
         {
            set { _value &= 0xFBFF; _value |= (UInt16)((value & 0x1) << 10); }
            get { return (UInt16)((_value & 0x400) >> 10); }
         }
         public UInt16 rtaddr
         {
            set { _value &= 0x7FF; _value |= (UInt16)((value & 0x1F) << 11); }
            get { return (UInt16)((_value & 0xF800) >> 11); }
         }
         public UInt16 cmdWord
         {
            set { _value = value; }
            get { return _value; }
         }
         public BT_1553_COMMAND_WORD(int dummy)
         {
            _value = 0;
         }
      };
      
      // a struct, not a class, because it is contained within other classes
      [StructLayout(LayoutKind.Sequential, Pack=2)]
      public struct BT_1553_STATUS_WORD
      {
         private UInt16 _value;

         // wcount:5 bits LSB, subaddr:5 bits, tran_rec:1 bit, rtaddr:5 bits, MSB
         public UInt16 tf
         {
            set { _value &= 0xFFFE; _value |= (UInt16)(value & 0x1); }
            get { return (UInt16)(_value & 0x1); }
         }
         public UInt16 dba
         {
            set { _value &= 0xFFFD; _value |= (UInt16)((value & 0x1) << 1); }
            get { return (UInt16)((_value & 0x2) >> 1); }
         }
         public UInt16 sf
         {
            set { _value &= 0xFFFB; _value |= (UInt16)((value & 0x1) << 2); }
            get { return (UInt16)((_value & 0x4) >> 2); }
         }
         public UInt16 busy
         {
            set { _value &= 0xFFF7; _value |= (UInt16)((value & 0x1) << 3); }
            get { return (UInt16)((_value & 0x8) >> 3); }
         }
         public UInt16 bcr
         {
            set { _value &= 0xFFEF; _value |= (UInt16)((value & 0x1) << 4); }
            get { return (UInt16)((_value & 0x10) >> 4); }
         }
         public UInt16 res
         {
            set { _value &= 0xFF1F; _value |= (UInt16)((value & 0x7) << 5); }
            get { return (UInt16)((_value & 0xE0) >> 5); }
         }
         public UInt16 sr
         {
            set { _value &= 0xFEFF; _value |= (UInt16)((value & 0x1) << 8); }
            get { return (UInt16)((_value & 0x100) >> 8); }
         }
         public UInt16 inst
         {
            set { _value &= 0xFDFF; _value |= (UInt16)((value & 0x1) << 9); }
            get { return (UInt16)((_value & 0x200) >> 9); }
         }
         public UInt16 me
         {
            set { _value &= 0xFBFF; _value |= (UInt16)((value & 0x1) << 10); }
            get { return (UInt16)((_value & 0x400) >> 10); }
         }
         public UInt16 rtaddr
         {
            set { _value &= 0x7FF; _value |= (UInt16)((value & 0x7) << 11); }
            get { return (UInt16)((_value & 0xF800) >> 11); }
         }
         public UInt16 statusWord
         {
            set { _value = value; }
            get { return _value; }
         }
         public BT_1553_STATUS_WORD(int dummy)
         {
            _value = 0;
         }
      }

      // a struct, not a class, because it is contained within other classes
      [StructLayout(LayoutKind.Sequential, Pack=2)]
      public struct BT1553_TIME
      {
         private UInt32 microseconds_lo;
         private UInt32 microseconds_hi;

         //public UInt32 microseconds
         //{
         //   get
         //   {
         //      UInt32 retval = (UInt32)microseconds_lo_0;
         //      retval |= ((UInt32)microseconds_lo_1) << 16;
         //      return retval;
         //   }
         //   set
         //   {
         //      microseconds_lo_0 = (UInt16) (value & 0x0000FFFF);
         //      microseconds_lo_1 = (UInt16)((value & 0xFFFF0000) >> 16);
         //   }
         //}

         public BT1553_TIME(UInt32 init)
         {
            microseconds_lo = init;
            microseconds_hi = init;
         }
      }

      ///////////////////////////////////////////////////////////////////////////
      // Start API_RT_MBUF (read and write, class and struct)
#region API_RT_MBUF
      [StructLayout(LayoutKind.Sequential, Pack=2)]
      public class API_RT_MBUF_WRITE
      {
         public UInt32 enable = 0;
         public UInt32 error_inj_id = 0;

         [MarshalAs(UnmanagedType.ByValArray, SizeConst = 32)]
         public UInt16[] mess_data;

         public API_RT_MBUF_WRITE()
         {
            mess_data = new UInt16[32];
            for (int i = 0; i < 32; i++) mess_data[i] = 0;
         }
      }

      [StructLayout(LayoutKind.Sequential, Pack=2)]
      public class API_RT_MBUF_READ
      {
         public UInt32 status = 0;                     // interrupt status
         public UInt32 ttreserved = 0;                 // was the hardware time tag
         public BT1553_TIME time = new BT1553_TIME(0);                  // BusTools 48-bit time tag LSW
         public BT_1553_COMMAND_WORD mess_command = new BT_1553_COMMAND_WORD(0); // 1553 command word
         public BT_1553_STATUS_WORD mess_status = new BT_1553_STATUS_WORD(0);
         [MarshalAs(UnmanagedType.ByValArray, SizeConst = 32)]
         public UInt16[] mess_data;                // data words
         public UInt32 spare = 0;                      // spare data word for word count errors

         public API_RT_MBUF_READ()
         {
            mess_data = new UInt16[32];
            for (int i = 0; i < 32; i++) mess_data[i] = 0;
         }
      }

      // This is a struct version of API_RT_MBUF_READ.  We need a struct version (a value type) so we can pass 
      // arrays of them to RT_ReadLastMessageBlock.  In all the other calls involving MBUFS it is more convenient to 
      // have a class version (i.e. a reference type) so we make both versions.
      [StructLayout(LayoutKind.Sequential, Pack=2)]
      public struct API_RT_MBUF_READ_STRUCT
      {
         public UInt32 status;                     // interrupt status
         public UInt32 ttreserved;                 // was the hardware time tag
         public BT1553_TIME time;                  // BusTools 48-bit time tag LSW
         public BT_1553_COMMAND_WORD mess_command; // 1553 command word
         public BT_1553_STATUS_WORD mess_status;

         [MarshalAs(UnmanagedType.ByValArray, SizeConst = 32)]
         public UInt16[] mess_data;                // data words
         
         public UInt32 spare;                      // spare data word for word count errors

         public API_RT_MBUF_READ_STRUCT(int dummy)
         {
            mess_data = new UInt16[32];
            for (int i = 0; i < 32; i++) mess_data[i] = 0;
            status = 0;
            ttreserved = 0;
            time = new BT1553_TIME(0);
            mess_command = new BT_1553_COMMAND_WORD(0);
            mess_status = new BT_1553_STATUS_WORD(0);
            spare = 0;
         }
      }

#endregion
      // End API_RT_MBUF
      ///////////////////////////////////////////////////////////////////////////

      ///////////////////////////////////////////////////////////////////////////
      // Start API_BC_MBUF (class and struct)
#region API_BC_MBUF
      [StructLayout(LayoutKind.Sequential, Pack=2)]
      public class API_BC_MBUF
      {
         public UInt16 messno = 0;        // Message number (0-based) //FFFF// indicates end of aperiodic list
         public UInt16 control = 0;       // Bus Controller Control Word.  Defines this
                                          // message to be either a 1553 data transfer
                                          // msg (BC_CONTROL_MESSAGE) or a list management
                                          // msg (conditional branch or noop).
         public UInt16 messno_next = 0;   // Next message number
         public UInt16 messno_prev = 0;   // Previous message number (Not used by API)
                                          // This group is for standard bc messages (not conditional messages)
                                          // Any given message may NOT be both a 1553 message and a conditional message
                                          // at the same time.  Data in this section is ignored for conditional msgs,
                                          // and data in the following section is ignored for 1553 messages.
         public BT_1553_COMMAND_WORD mess_command1 = new BT_1553_COMMAND_WORD(0); // 1553 command word (Receive for RT-RT msgs)
         public BT_1553_COMMAND_WORD mess_command2 = new BT_1553_COMMAND_WORD(0); // 1553 cmd word #2 (Transmit, RT-RT msgs)
         public UInt16 errorid = 0;       // error injection buffer id number
         public UInt16 gap_time = 0;      // time to delay after end of current msg
                                          // before next message (PC-1553)  OR
                                          // time from end of previous msg to start
                                          // of this msg (IP-1553/PCI-1553)
         public BT_1553_STATUS_WORD mess_status1 = new BT_1553_STATUS_WORD(0); // 1553 status wor
         public BT_1553_STATUS_WORD mess_status2 = new BT_1553_STATUS_WORD(0); // 1553 status word #2 (for RT-RT msgs)
         public UInt32 status = 0; // message error status from hardware       

         [MarshalAs(UnmanagedType.ByValArray, SizeConst = 2 * Constants.BT1553_BUFCOUNT)]
         private UInt16[] data;
         
         // This group is for BC_CONTROL_CONDITION, #2 and #3 list management messages
         
         public UInt32 data_value = 0;    // Data value to compare
         public UInt32 data_mask = 0;     // Bit mask for compare
         public UInt16 address = 0;       // Word number of the previous or specified msg
                                          // to compare for BC_CONTROL_CONDITION or
                                          // BC_CONTROL_CONDITION3 Conditional branch msgs.
         public UInt16 messno_branch = 0; // If //==// branch to this message
         public UInt16 messno_compare = 0; // Msg # containing data word in "address"
         
         // These variables are only used by the extended BC message function calls.
         public UInt16 start_frame = 0; 
         public UInt32 test_address = 0;   // Byte address of word to be tested by
                                       // BC_CONTROL_CONDITION2 conditional branch msg.
         public UInt32 cond_count_val = 0; // Conditional counter reload value
         public UInt32 cond_counter = 0;   // Conditional counter initial value
         public UInt16 data_buf_num1 = 0;  // Buffer number of the first BC data buffer
         public UInt16 data_buf_num2 = 0;  // Buffer number of the second BC data buffer
         public BT1553_TIME time_tag; // = new BT1553_TIME(24);

         public UInt32 long_gap;
         public UInt32 next_branch_addr;
         public UInt16 spare;
         public UInt16 rep_rate;

         public UInt16[] ReceiveData
         {
            get
            {
               UInt16[] retVal = new UInt16[32];
               for (int i = 0; i < 32; i++)
               {
                  retVal[i] = data[GetIndex(Constants.RECEIVE, i)];
               }
               return retVal;
            }
         }

         public UInt16[] TransmitData
         {
            get
            {
               UInt16[] retVal = new UInt16[32];
               for (int i = 0; i < 32; i++)
               {
                  retVal[i] = data[GetIndex(Constants.TRANSMIT, i)];
               }
               return retVal;
            }
         }

         private Int32 GetIndex(int tr, int buffIndex)
         {
            return tr * Constants.BT1553_BUFCOUNT + buffIndex;
         }
         public UInt16 GetData(int tr, int buffIndex)
         {
            return data[GetIndex(tr, buffIndex)];
         }
         public void SetData(int tr, int buffIndex, UInt16 value)
         {
            data[GetIndex(tr, buffIndex)] = value;
         }

         // constructor
         public API_BC_MBUF()
         {
            data = new UInt16[2 * Constants.BT1553_BUFCOUNT];
            for (int i = 0; i < 2 * Constants.BT1553_BUFCOUNT; i++)
            {
               data[i] = 0;
            }
         }
      }

      // This is a struct version of API_BC_MBUF.  We need a struct version (a value type) so we can pass 
      // arrays of them to BM_ReadLastMessageBlock.  In all the other calls involving MBUFS it is more convenient 
      // to have a class version (i.e. a reference type) so we make both versions.
      public struct API_BC_MBUF_STRUCT
      {
         public UInt16 messno;                        // Message number (0-based) //FFFF// indicates end of aperiodic list
         public UInt16 control;                       // Bus Controller Control Word.  Defines this
                                                      // message to be either a 1553 data transfer
                                                      // msg (BC_CONTROL_MESSAGE) or a list management
                                                      // msg (conditional branch or noop).
         public UInt16 messno_next;                   // Next message number
         public UInt16 messno_prev;                   // Previous message number (Not used by API)
                                                      // This group is for standard bc messages (not conditional messages)
                                                      // Any given message may NOT be both a 1553 message and a conditional message
                                                      // at the same time.  Data in this section is ignored for conditional msgs,
                                                      // and data in the following section is ignored for 1553 messages.
         public BT_1553_COMMAND_WORD mess_command1;   // 1553 command word (Receive for RT-RT msgs)
         public BT_1553_COMMAND_WORD mess_command2;   // 1553 cmd word #2 (Transmit, RT-RT msgs)
         public UInt16 errorid;                       // error injection buffer id number
         public UInt16 gap_time;                      // time to delay after end of current msg
                                                      // before next message (PC-1553)  OR
                                                      // time from end of previous msg to start
                                                      // of this msg (IP-1553/PCI-1553)
         public BT_1553_STATUS_WORD mess_status1;     // 1553 status wor
         public BT_1553_STATUS_WORD mess_status2;     // 1553 status word #2 (for RT-RT msgs)
         public UInt32 status;                        // message error status from hardware       

         [MarshalAs(UnmanagedType.ByValArray, SizeConst = 2 * Constants.BT1553_BUFCOUNT)]
         private UInt16[] data;

         // This group is for BC_CONTROL_CONDITION, #2 and #3 list management messages

         public UInt32 data_value;                    // Data value to compare
         public UInt32 data_mask;                     // Bit mask for compare
         public UInt16 address;                       // Word number of the previous or specified msg
                                                      // to compare for BC_CONTROL_CONDITION or
                                                      // BC_CONTROL_CONDITION3 Conditional branch msgs.
         public UInt16 messno_branch;                 // If //==// branch to this message
         public UInt16 messno_compare;                // Msg # containing data word in "address"

         // These variables are only used by the extended BC message function calls.
         public UInt16 start_frame;
         public UInt32 test_address;   // Byte address of word to be tested by BC_CONTROL_CONDITION2 conditional branch msg.
         public UInt32 cond_count_val; // Conditional counter reload value
         public UInt32 cond_counter;   // Conditional counter initial value
         public UInt16 data_buf_num1;  // Buffer number of the first BC data buffer
         public UInt16 data_buf_num2;  // Buffer number of the second BC data buffer
         public BT1553_TIME time_tag;  // = new BT1553_TIME(24);

         public UInt32 long_gap;
         public UInt32 next_branch_addr;
         public UInt16 spare;
         public UInt16 rep_rate;

         public UInt16[] ReceiveData
         {
            get
            {
               UInt16[] retVal = new UInt16[32];
               for (int i = 0; i < 32; i++)
               {
                  retVal[i] = data[GetIndex(Constants.RECEIVE, i)];
               }
               return retVal;
            }
            set
            {
               for (int i = 0; i < 32; i++)
               {
                  data[GetIndex(Constants.RECEIVE, i)] = value[i];
               }
            }
         }

         public UInt16[] TransmitData
         {
            get
            {
               UInt16[] retVal = new UInt16[32];
               for (int i = 0; i < 32; i++)
               {
                  retVal[i] = data[GetIndex(Constants.TRANSMIT, i)];
               }
               return retVal;
            }
            set
            {
               for (int i = 0; i < 32; i++)
               {
                  data[GetIndex(Constants.TRANSMIT, i)] = value[i];
               }
            }
         }

         private Int32 GetIndex(int tr, int buffIndex)
         {
            return tr * Constants.BT1553_BUFCOUNT + buffIndex;
         }
         public UInt16 GetData(int tr, int buffIndex)
         {
            return data[GetIndex(tr, buffIndex)];
         }

         public void SetData(int tr, int buffIndex, UInt16 value)
         {
            data[GetIndex(tr, buffIndex)] = value;
         }

         // constructor
         public API_BC_MBUF_STRUCT(int dummy)
         {
            messno = 0;
            control = 0;
            messno_next = 0;
            messno_prev = 0;
            mess_command1 = new BT_1553_COMMAND_WORD(0);
            mess_command2 = new BT_1553_COMMAND_WORD(0);
            errorid = 0;
            gap_time = 0;
            mess_status1 = new BT_1553_STATUS_WORD(0);
            mess_status2 = new BT_1553_STATUS_WORD(0);
            status = 0;
            data_value = 0;
            data_mask = 0;
            address = 0;
            messno_branch = 0;
            messno_compare = 0;
            start_frame = 0;
            test_address = 0;
            cond_count_val = 0;
            cond_counter = 0;
            data_buf_num1 = 0;
            data_buf_num2 = 0;
            time_tag = new BT1553_TIME(24);
            rep_rate = 0;
            spare = 0;
            long_gap = 0;
            next_branch_addr = 0;

            data = new UInt16[2 * Constants.BT1553_BUFCOUNT];
            for (int i = 0; i < 2 * Constants.BT1553_BUFCOUNT; i++)
            {
               data[i] = 0;
            }
         }
      }
#endregion
      // End API_BC_MBUF
      ///////////////////////////////////////////////////////////////////////////

      ///////////////////////////////////////////////////////////////////////////
      // Start API_BM_MBUF (class and struct)
#region API_BM_MBUF
      [StructLayout(LayoutKind.Sequential, Pack=2)]
      public class API_BM_MBUF
      {
         public UInt32 messno = 0;                  // Message number (generated by API, 1-based)
         public UInt32 int_status = 0;              // Interrupt status from board
         public BT1553_TIME time = new BT1553_TIME(0);
         public BT_1553_COMMAND_WORD command1 = new BT_1553_COMMAND_WORD(0);  // 1553 command word #1
         public UInt16 status_c1 = 0;               // 1553 command word #1 error status
         public BT_1553_COMMAND_WORD command2 = new BT_1553_COMMAND_WORD(0);  // 1553 command word #2
         public UInt16 status_c2 = 0;               // 1553 command word #2 error status
         public Byte response1 = 0;                 // 1553 response time #1 (byte)
         public Byte response2 = 0;                 // 1553 response time #1 (byte)
         public BT_1553_STATUS_WORD status1;        // 1553 status word #1
         public UInt16 status_s1 = 0;               // 1553 status word #1 error status
         public BT_1553_STATUS_WORD status2;        // 1553 status word #2
         public UInt16 status_s2 = 0;               // 1553 status word #2 error status
         
         [MarshalAs(UnmanagedType.ByValArray, SizeConst = Constants.BT1553_MBUFCOUNT)]
         public UInt16[] value;

         [MarshalAs(UnmanagedType.ByValArray, SizeConst = Constants.BT1553_MBUFCOUNT)]
         public UInt16[] status;

#if _BM_USER_DATA
         public UInt16 user1 = 0; 
         public UInt16 user2 = 0; 
#endif
         // constructor
         public API_BM_MBUF()
         {
            value = new UInt16[Constants.BT1553_BUFCOUNT];
            status = new UInt16[Constants.BT1553_BUFCOUNT];
            for (int i = 0; i < Constants.BT1553_MBUFCOUNT; i++)
            {
               value[i] = status[i] = 0;
            }
         }
      }

      // This is a struct version of API_BM_MBUF.  We need a struct version (a value type) so we can pass 
      // arrays of them to BM_ReadLastMessageBlock.  In all the other calls involving MBUFS it is more convenient 
      // to have a class version (i.e. a reference type) so we make both versions.
      [StructLayout(LayoutKind.Sequential, Pack=2)]
      public struct API_BM_MBUF_STRUCT
      {
         public UInt32 messno;                  // Message number (generated by API, 1-based)
         public UInt32 int_status;              // Interrupt status from board
         
         public BT1553_TIME time;
         public BT_1553_COMMAND_WORD command1;      // 1553 command word #1
         public UInt16 status_c1;               // 1553 command word #1 error status
         public BT_1553_COMMAND_WORD command2;      // 1553 command word #2
         
         public UInt16 status_c2;               // 1553 command word #2 error status
         
         public Byte response1;                 // 1553 response time #1 (byte)
         public Byte response2;                 // 1553 response time #1 (byte)
         public BT_1553_STATUS_WORD status1;    // 1553 status word #1
         public UInt16 status_s1;               // 1553 status word #1 error status
         public BT_1553_STATUS_WORD status2;    // 1553 status word #2
         public UInt16 status_s2;               // 1553 status word #2 error status

         [MarshalAs(UnmanagedType.ByValArray, SizeConst = Constants.BT1553_MBUFCOUNT)]
         public UInt16[] value;

         [MarshalAs(UnmanagedType.ByValArray, SizeConst = Constants.BT1553_MBUFCOUNT)]
         public UInt16[] status;

#if _BM_USER_DATA
         public UInt16 user1 = 0; 
         public UInt16 user2 = 0; 
#endif
         // constructor
         public API_BM_MBUF_STRUCT(int dummy)
         {
            messno = 0;
            int_status = 0;
            time = new BT1553_TIME(0);
            command1 = new BT_1553_COMMAND_WORD(0);
            status_c1 = 0;
            command2 = new BT_1553_COMMAND_WORD(0);
            status_c2 = 0;
            response1 = 0;
            response2 = 0;
            status1 = new BT_1553_STATUS_WORD(0);
            status_s1 = 0;
            status2 = new BT_1553_STATUS_WORD(0);
            status_s2 = 0;

            value = new UInt16[Constants.BT1553_BUFCOUNT];
            status = new UInt16[Constants.BT1553_BUFCOUNT];
            for (int i = 0; i < Constants.BT1553_MBUFCOUNT; i++)
            {
               value[i] = status[i] = 0;
            }
         }
      }


#endregion
      // End API_BM_MBUF
      ///////////////////////////////////////////////////////////////////////////

      [StructLayout(LayoutKind.Sequential, Pack=2)]
      public class API_PLAYBACK_STATUS
      {
         public UInt32 tailPointer = 0;
         public UInt16 playbackStatus = 0;
         public UInt32 playbackError = 0;
         public UInt32 recordsProcessed = 0;
      }

      [StructLayout(LayoutKind.Sequential, Pack=2)]
      public class API_PLAYBACK
      {
         public BT1553_TIME timeStart = new BT1553_TIME(0);       // Start Tag Time
         public BT1553_TIME timeStop = new BT1553_TIME(0);        // Stop Tag Time
         public Int32 messageStart = 0;          // Starting message number
         public Int32 messageStop = 0;           // Ending message number
         public Int32 filterFlag = 0;            // 0 no filter, 1 = time filter, 2 = message filter
         public Int32 activeRT = 0;              // Active RTs 0 = do not playback = 0; 1 = use in playback
         public Int32 Subadress31Flag = 0;       // 0 = SA 31 is not a mode code, 1 = SA31 is mode code
         public Int32 Rt31Flag = 0;              // 0 = RT 31 is not broadcast, 1 = RT 31 is broadcast
         public IntPtr status = new IntPtr(0);   // 
         public string fileName = null;
      }

      [StructLayout(LayoutKind.Sequential, Pack=2)]
      public class API_RT_ABUF
      {
         public UInt16 enable_a = 0; // enable channel a = 0; 1=enable
         public UInt16 enable_b = 0; // enable channel b = 0; 1=enable
         public UInt16 inhibit_term_flag = 0; // inhibit terminal flag
         public UInt16 status = 0; // latest RT status word
         public UInt16 command1 = 0; // latest RT command word (read only)
         public UInt16 bit_word = 0; // latest built in test (BIT) word
      }

      [StructLayout(LayoutKind.Sequential, Pack=2)]
      public class API_RT_CBUFBROAD
      {
         [MarshalAs(UnmanagedType.ByValArray, SizeConst = 31)]
         public UInt32[] legal_wordcount;
         public UInt32 mbuf_count;

         public API_RT_CBUFBROAD(int dummy)
         {
            mbuf_count = 0; 
            legal_wordcount = new UInt32[31];
            for (int i = 0; i < 31; i++) legal_wordcount[i] = 0;
         }
      }

      [StructLayout(LayoutKind.Sequential, Pack=2)]
      public class API_RT_CBUF
      {
         public UInt32 legal_wordcount = 0;
      }

      [StructLayout(LayoutKind.Explicit)]
      public class API_BM_CBUF
      {
         [System.Runtime.InteropServices.FieldOffset(0)]
         public Int32 wcount = 0;
         [System.Runtime.InteropServices.FieldOffset(0)]
         public Int32 modecode = 0;

         [System.Runtime.InteropServices.FieldOffset(4)]
         public UInt16 pass_count = 0;
      }

      // a struct, not a class, because it is contained within other classes
      [StructLayout(LayoutKind.Sequential, Pack=2)]
      public struct API_BM_TBUF_CAPTURE_STOP_STRUCT
      {
         public UInt16 type;
         public UInt16 mask;
         public UInt16 value;
         public UInt16 word;
         public UInt16 count;
         public API_BM_TBUF_CAPTURE_STOP_STRUCT(int dummy)
         {
            type = 0;
            mask = 0;
            value = 0;
            word = 0;
            count = 0;
         }
      }

      [StructLayout(LayoutKind.Sequential, Pack=2)]
      public class API_BM_TBUF
      {
         public UInt16 trig_ext = 0;         // 1 -> external trigger input enabled
                                             // 2 -> external output on every BM interrupt
                                             // (rest of structure is ignored unless zero)
         public UInt16 trig_err = 0;         // 1 -> trigger on errors
         public UInt16 trig_ext_output = 0;  // 1 -> external output on trigger
                                             // 2 -> repetitive external output on trigger
         public UInt16 control1 = 0;         // control of trigger start
         public UInt16 control2 = 0;         // control of trigger stop
         
         [MarshalAs(UnmanagedType.ByValArray, SizeConst = 4)]
         public API_BM_TBUF_CAPTURE_STOP_STRUCT[] capture;
         
         [MarshalAs(UnmanagedType.ByValArray, SizeConst = 4)]
         public API_BM_TBUF_CAPTURE_STOP_STRUCT[] stop;
         
         public API_BM_TBUF()
         {
            capture = new API_BM_TBUF_CAPTURE_STOP_STRUCT[4];
            stop = new API_BM_TBUF_CAPTURE_STOP_STRUCT[4];
            for (int i = 0; i < 4; i++)
            {
               capture[i] = new API_BM_TBUF_CAPTURE_STOP_STRUCT(0);
               stop[i] = new API_BM_TBUF_CAPTURE_STOP_STRUCT(0);
            }
         }
      }

      // a struct, not a class, because it is contained within other classes
      [StructLayout(LayoutKind.Sequential, Pack=2)]
      public struct API_EIBUF_ERROR
      {
         public Byte etype;
         public Byte edata;
         public API_EIBUF_ERROR(int dummy)
         {
            etype = 0;
            edata = 0;
         }
      }

      [StructLayout(LayoutKind.Sequential, Pack=2)]
      public class API_EIBUF
      {
         public UInt16 bufftype = 0;

         [MarshalAs(UnmanagedType.ByValArray, SizeConst = Constants.EI_COUNT)]
         public API_EIBUF_ERROR[] error;

         public API_EIBUF()
         {
            error = new API_EIBUF_ERROR[Constants.EI_COUNT];
            for (int i = 0; i < Constants.EI_COUNT; i++)
            {
               error[i] = new API_EIBUF_ERROR(0);
            }
         }
      }

      [StructLayout(LayoutKind.Sequential, Pack=2)]
      public class DEVICE_LIST
      {
         public Int32 num_devices = 0;
         
         [MarshalAs(UnmanagedType.ByValArray, SizeConst = Constants.MAX_BTA)]
         public UInt32[] device_name;

         [MarshalAs(UnmanagedType.ByValArray, SizeConst = Constants.MAX_BTA)]
         public Int32[] device_num;

         [MarshalAs(UnmanagedType.ByValArray, SizeConst = Constants.MAX_BTA * 256)]
         private Byte[,] _name;
         
         // routine to get a string from 2-dimensional byte array
         public String name(int nameIndex)
         {
            char[] nameBytes = new char[256];
            for (int i = 0; i < 256; i++)
            {
               nameBytes[i] = (char)_name[nameIndex, i];
            }

            return new String(nameBytes);
         }
         
         public DEVICE_LIST(int dummy) // call constructor with dummy = null
         {
            _name = new Byte[Constants.MAX_BTA, 256];
            for (int i = 0; i < Constants.MAX_BTA; i++)
            {
               for (int j = 0; j < 256; j++)
               {
                  _name[i, j] = System.Convert.ToByte(0);
               }
            }
            
            device_name = new UInt32[Constants.MAX_BTA];
            device_num = new Int32[Constants.MAX_BTA];
            for (int i = 0; i < Constants.MAX_BTA; i++)
            {
               device_name[i] = 0;
               device_num[i] = 0;
            }
         }
      }

      [StructLayout(LayoutKind.Sequential, Pack=2)]
      public class DEVICE_INFO
      {
         public Int32 busType = 0;
         public Int32 nchan = 0;
         public Int32 irig = 0;
         public Int32 mode = 0;
         public Int32 memSections = 0;
         public UInt32 vendorID = 0;
         public UInt32 deviceID = 0;
      }

   }
}
