using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;

/*
   This demo project shows various interation between a managed client (represented by this C# program),
   and an unmanaged server (represented by the project "CDll").
   
   A static wrapper class is required, which is represented by the "API" class in this file.  The API class maps
   managed function calls to unmanaged functions in the server.
*/

namespace BustoolsInterop
{
   public partial class Form1 : Form
   {
      // Delegates can be used to implement callback pointers.  This declaration defines that "MyDelegate" is 
      // bound to the function signature int x(int x, int y).  It doesn't create a callback pointer by itself.
      public delegate Int32 MyDelegate(int x, int y);

      // This creates a .NET delegate.  A Delegate is a managed type that can contain a function pointer.
      // You can call the delegate just like it's a function, e.g. 
      //    globalDelegate(2, 3);
      // or you can pass it as a parameter.
      MyDelegate globalDelegate = new MyDelegate(MyCallBackFunc);
      IntPtr memPtr;

      // this function is called back by the unmanaged code
      public static Int32 MyCallBackFunc(int x, int y)
      {
         return x + y;
      }

      // This class encapsulates a byte array and is used to demonstrate marshaling data from server to client
      [StructLayout(LayoutKind.Sequential)]
      public class ByteContainer
      {
         [MarshalAs(UnmanagedType.ByValArray, SizeConst = 32)]
         Byte[] bytes;
         public ByteContainer()
         {
            bytes = new Byte[32];
         }
      }

      public Form1()
      {
         InitializeComponent();

         ////////////////////////////////////////////////////////////////////////////////////////////////////
         // Example 1
         // This is an example of passing an array to the server, where it is modified and returned.
         // The array must be of blittable value types (i.e. structs).

         // allocate the array and set some values in it
         API.MySuperStruct[] a = new API.MySuperStruct[3];
         a[0] = new API.MySuperStruct(0);
         a[0].number = 10;
         a[1].number = 11;
         a[2].number = 12;

         // This call to unmanaged code will modify the array.  We will be able to see the modification
         // because the managed signature has the [In, Out] attributes.
         API.ArrayTest1(a);

         // check the modification
         uint newNumber = a[0].number;

         // now do the same thing with an array of shorts
         UInt16[] shorts = new UInt16[3];
         shorts[0] = 23;
         shorts[1] = 24;
         shorts[2] = 25;
         API.ArrayTest2(shorts);
         // check the modification
         UInt16 newShort = shorts[2];


         ////////////////////////////////////////////////////////////////////////////////////////////////////
         // Example 2
         // This is an example of using an un-managed funtion where a ** is passed as a parameter.
         // The server modifies our pointer to point to an unmanaged buffer it allocated.

         IntPtr pUnmanagedBytes;
         ByteContainer byteContainer = new ByteContainer();
         API.ReturnArray(out pUnmanagedBytes);

         // pUnmanagedBytes was modified by the server, so now we must copy the unmanaged array to a managed structure
         Marshal.PtrToStructure(pUnmanagedBytes, byteContainer);
         

         ////////////////////////////////////////////////////////////////////////////////////////////////////
         // Example 3
         // We pass myClass to the API where it is modified.  We will be able to see the modification
         // because the managed signature has the [In, Out] attributes.
         // This paradigm is used when passing a pointer to a structure in C, except it must be a Class.
         
         API.MyClass myClass = new API.MyClass();
         int t = API.Test(myClass); // you can see changes to myClass after this call
         

         ////////////////////////////////////////////////////////////////////////////////////////////////////
         // Example 4
         // We call the API CreateStruct() function, which creates an unmanaged structure for us and passes back a
         // pointer to it.  We pass CreateStruct() a callback pointer that will be assigned to the structure member.  
         // Before it returns, the server calls the managed callback which is MyCallBackFunc()

         memPtr = API.CreateStruct(Marshal.GetFunctionPointerForDelegate(globalDelegate));
         if (memPtr == IntPtr.Zero)
         {
            // error
         }

         // Now that we have a pointer to the unmanaged structure, marshall the pointer to a managed class.  The 
         // managed class is NOT the same memory as the structure the API allocated.
         API.MyClass managedClass = (API.MyClass)Marshal.PtrToStructure(memPtr, typeof(API.MyClass));
         
         // modify our new managed class
         managedClass.number = 999;

         // Send the API the pointer it originally sent to us (which still points to the original structure in
         // unmanaged memory), and the modified class.  The API can update the original unmanaged structure using the pointers
         // to the original structure and the new one.
         API.UpdateStructure(memPtr, managedClass);


         ////////////////////////////////////////////////////////////////////////////////////////////////////
         // Example 5
         // Simple example of passing a pointer to a value type.  The unmanaged code modifies the argument.
         UInt32 refInt = 1;
         API.PassByReference(ref refInt);
      }
   }


   // This class represents an unmanaged API.  It wraps the unmanaged functions in the CDll project.
   public class API
   {
      [DllImport("CDll.dll", EntryPoint = "ArrayTest1")]
      public static extern int ArrayTest1
      (
         // we need [In, Out] so we can see modifications to the class made by the called function
         // the CLR mashalls the parameters upon entry and exit
         [In, Out] MySuperStruct[] myClassStruct
      );

      [DllImport("CDll.dll", EntryPoint = "ArrayTest2")]
      public static extern int ArrayTest2
      (
         // we need [In, Out] so we can see modifications to the class made by the called function
         // the CLR mashalls the parameters upon entry and exit.
         
         // LPArray is the default for arrays that are not embedded in a class or struct,
         // so we really don't need this attribute.
         [MarshalAs(UnmanagedType.LPArray)]
         [In, Out] UInt16[] shortArray
      );

      [DllImport("CDll.dll", EntryPoint = "Test")]
      public static extern int Test
      (
         // we need [In, Out] so we can see modifications to the class made by the called function
         // the CLR mashalls the parameters upon entry and exit
         [In, Out] MyClass myClass
      );

      [DllImport("CDll.dll", EntryPoint = "UpdateStructure")]
      public static extern int UpdateStructure
      (
         [In] IntPtr pVoid1,
         [In] MyClass myClass
      );

      [DllImport("CDll.dll", EntryPoint = "CreateStruct")]
      public static extern IntPtr CreateStruct
      (
         IntPtr pointer
      );

      [DllImport("CDll.dll", EntryPoint = "PassByReference")]
      public static extern int PassByReference
      (
         ref UInt32 pInt
      );

      [DllImport("CDll.dll", EntryPoint = "ReturnArray")]
      public static extern int ReturnArray
      (
         out IntPtr pBytes
      );

      // "MyClass" and "MySuperStruct" is passed between managed and unmanaged code using interop.  
      // The only difference between them is that one is a class and one is a struct.  

      // Their structure mirrors MYSTRUCT in the unmanaged code.  Arrays of the struct, but not 
      // the class, can be passed to the unmanaged code.

      // They contains a function pointer, an integer, an array of structures, and a 2-dimesional array of 
      // integers.  For a class to work with interop it must contain only "blittable" types, e.g. no members 
      // that are reference types.  Arrays are considered reference types but they are converted to C style arrays
      // during the Marshaling.

      [StructLayout(LayoutKind.Sequential)]
      public class MyClass
      {
         // this creates a callback pointer that unmanaged code can use
         public Form1.MyDelegate m = new Form1.MyDelegate(Form1.MyCallBackFunc);

         // an integer
         public UInt32 number = 43;

         // An embedded array of structures.  SizeConst = 3 means we have 3 structs, not total size of "embedded"
         [MarshalAs(UnmanagedType.ByValArray, SizeConst = 3)]
         public MyStruct[] structArray;

         // The unmanaged structure has 2-dimensional array of integers of size [2][3].  
         // Marshalling arrays by value must be done only with single dimension arrays, so we have to marshall it 
         // as a ByValArray array with SizeConst = 6, and provide get/set accessors to access it with array indicies.

         // ByValArray is required for embedded arrays
         [MarshalAs(UnmanagedType.ByValArray, SizeConst = 6)]
         private UInt32[] _anArray = new UInt32[6];

         private Int32 GetIndex(int x, int y)
         {
            return x * y + y;
         }
         
         public UInt32 GetArrayData(int x, int y)
         {
            return _anArray[GetIndex(x, y)];
         }
         
         public void SetArrayData(int x, int y, UInt32 value)
         {
            _anArray[GetIndex(x, y)] = value;
         }

         // class constructor.  for classes, you can override the default (no argument) construcor.  You can also
         // initialize in the member declarations as shown above.
         public MyClass()
         {
            structArray = new MyStruct[3];
            for (int i = 0; i < 3; i++)
            {
               structArray[i] = new MyStruct(i);
            }
            for (int i = 0; i < 6; i++)
            {
               _anArray[i] = (UInt32)i;
            }
         }
      }

      [StructLayout(LayoutKind.Sequential)]
      public struct MySuperStruct
      {
         // the callback pointer
         public Form1.MyDelegate m;

         // an integer
         public UInt32 number;

         // An embedded array of structures.  SizeConst = 3 means we have 3 structs, not total size of "embedded"
         [MarshalAs(UnmanagedType.ByValArray, SizeConst = 3)]
         public MyStruct[] structArray;

         // The unmanaged structure has 2-dimensional array of integers of size [2][3].  
         // Marshalling arrays by value must be done only with single dimension arrays, so we have to marshall it 
         // as a ByValArray array with SizeConst = 6, and provide get/set accessors to access it with array indicies.
         [MarshalAs(UnmanagedType.ByValArray, SizeConst=6)]
         public UInt32[] _anArray;

         private Int32 GetIndex(int x, int y)
         {
            return x * y + y;
         }

         public UInt32 GetArrayData(int x, int y)
         {
            return _anArray[GetIndex(x, y)];
         }

         public void SetArrayData(int x, int y, UInt32 value)
         {
            _anArray[GetIndex(x, y)] = value;
         }

         // struct constructor.  for structs, you can't override the default (no argument) construcor so you may have
         // to have a dummy parameter.  You can only initialize the members in the constructor, not in the declarations 
         // like in a class.
         public MySuperStruct(int dummy)
         {
            this.number = 43;
            // this creates a callback pointer that unmanaged code can use
            this.m = new Form1.MyDelegate(Form1.MyCallBackFunc);

            structArray = new MyStruct[3];
            for (int i = 0; i < 3; i++)
            {
               structArray[i] = new MyStruct(i);
            }

            _anArray = new UInt32[6];
            for (int i = 0; i < 6; i++)
            {
               _anArray[i] = (UInt32)i;
            }
         }
      }

      // structure used as a member of MyClass and MySuperStruct
      [StructLayout(LayoutKind.Sequential)]
      public struct MyStruct
      {
         public Int32 x;
         public Int32 y;

         // class constructor.  for structures, you can NOT override the default (no argument) construcor,
         // so your constructor must have an argument.

         // You can NOT initialize in the member declarations like in classes.

         // If you create the structure with the default (no argument) constructor, the members will be uninitialized.
         public MyStruct(int initVal)
         {
            x = initVal;
            y = initVal + 1;
         }
      }

   }
}