/*===========================================================================*
 * FILE:             E X A M P L E _ B C _ C O N D _ C O U N T . C
 *===========================================================================*
 *
 *      COPYRIGHT (C) 1998-2016 BY ABACO SYSTEMS, INC. 
 *      ALL RIGHTS RESERVED.
 *
 *      THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND
 *      COPIED ONLY IN ACCORDANCE WITH THE TERMS OF SUCH LICENSE AND WITH
 *      THE INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY
 *      OTHER COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE
 *      AVAILABLE TO ANY OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF THE
 *      SOFTWARE IS HEREBY TRANSFERRED.
 *
 *      THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT
 *      NOTICE AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY ABACO SYSTEMS.
 *
 *============================================================================*
 *
 * FUNCTION:    Demonstration of BusTools/1553-API routines. This console 
 *              application shows how setup a BC messages and use conditional 
 *              branch on count to run a specific number of frames.  There is
 *              also a branch on data in a previous message.  
 *
 *              The BC sets up message buffer 1 as a Condtional Branch 3.  That
 *              branch looks at the data in the third data word of the previous 
 *              message.  If the third data word equals 0xA002, the branch
 *              skips the next message.  In this example that condition is always
 *              TRUE and the next message in the bus list, RT3 SA3 WC3 TX is 
 *              never transmitted.  You can alter the data and see how that
 *              effects the branch.
 *        
 *              Then message buffer 5 is setup as a Conditional Branch 2 that
 *              is set to branch on the 10th occurrence. Because of the way the 
 *              firmware decrements the branch count, it is set to n-1. In this
 *              case count is set to 9 (10-1). The count is decremented each time the 
 *              branch executes. When it hits to zero the branch occurs. In the 
 *              example the non-branching false condition is a nop message setting
 *              the end-of-frame with the next-message pointing to the frame start
 *              at message buffer 0.  When the branch does executes the message
 *              buffer contains a BC LAST command that stops the Bus Controller.
 *              This setup executes the frame 10 times then halts the BC.  You
 *              change the two count values and see how it alters the execution.
 *
 *===========================================================================*/

/* $Revision:  8.12 Release $
   Date        Revision
  ----------   ---------------------------------------------------------------
  11/18/2010    Initial release
  08/20/2013    Changed to ensure V6 compatibility 

 *---------------------------------------------------------------------------*
 *                     INCLUDES, DEFINES and GLOBALS
 *---------------------------------------------------------------------------*/
#include <stdio.h>
#include "busapi.h"

int BusController_Init(int,int);
void setupThe_BC_IntFIFO(int);

API_INT_FIFO  sIntFIFO1;     // Thread FIFO structure for BC

static unsigned bm_tcnt=0,bc_tcnt=0,rt_tcnt=0;

void main(int argc, char **argv)                     
{
   int status,cardnum;
   unsigned mode, device;

   mode = API_SW_INTERRUPT | API_B_MODE;

   if(argc==2)
      device = atoi(argv[1]); // Command-line entry
   else
      device = 0;             // Default device
   
   printf("Initializing Test\n");

   // Initial the channel running the BC
   status = BusTools_API_OpenChannel(&cardnum,mode,device,CHANNEL_1);
   printf("BusTools_API_OpenChannel BC status = %d\n",status);

   // Set the Bus to External
   status = BusTools_SetInternalBus(cardnum,EXTERNAL_BUS);  // Set up External bus 0 on Internal 1
   printf("BusTools_SetInternalBus BC status = %d\n",status);
   
   // Set up the BC
   status = BusController_Init(cardnum,0);

   // Setting up a callback function to show the BC data
   setupThe_BC_IntFIFO(cardnum); //Set up BC interrupts using BusTools_RegisterFunction

   status = BusTools_BC_StartStop(cardnum, BC_START);
   printf("BusTools_BC_StartStop BC status = %d\n",status);

   // Wait until the Bus Controller is no longer running
   while((status = BusTools_BC_IsRunning2(cardnum)) == API_BC_IS_RUNNING);

   status = BusTools_RegisterFunction(cardnum, &sIntFIFO1, UNREGISTER_FUNCTION);

   BusTools_API_Close(cardnum);
}

/****************************************************************************
*
*  Function:  BusController_Init
*
*  Description:
*     This routine configures the Bus Controller message into a single frame
*     there are two conditional branch messages that execute.   One skip the
*     following message and the other halts the BC after 10 times through the
*     frame.
****************************************************************************/
int BusController_Init(int cardnum,int sa_index)
{
   int status, messno,i;
   API_BC_MBUF bcmessage;;

   /* Initialize the BC.  wint interrupt on end-of-message        */
   status = BusTools_BC_Init(cardnum,0,BT1553_INT_END_OF_MESS,0,16,14,1000000,1);
   printf("BusTools_BC_Init status = %d\n",status);

   // Allocate enough BC Buffer to hold the maximum number of meesage you are using
   status = BusTools_BC_MessageAlloc(cardnum,10);
   printf("BusTools_BC_MessageAlloc status = %d\n",status);

   // Now create each message.
   messno = 0;
   memset((char*)&bcmessage,0,sizeof(bcmessage));    
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Create a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_INTERRUPT;
   bcmessage.control |= BC_CONTROL_MFRAME_BEG;
   bcmessage.mess_command1.rtaddr   = 2;
   bcmessage.mess_command1.subaddr  = 2;
   bcmessage.mess_command1.wcount   = 4;
   bcmessage.mess_command1.tran_rec = RECEIVE;
   bcmessage.errorid = 0;                         // Default error injection buffer (no errors)
   bcmessage.gap_time = 20;                       // 20 microsecond inter-message gap.
   for(i=0;i<4;i++)
      bcmessage.data[0][i] = 0xA000+i;

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   //CONDITION3. 
   messno++;
   memset((char*)&bcmessage,0,sizeof(bcmessage));    
   bcmessage.control = BC_CONTROL_CONDITION3;
   bcmessage.control |= BC_CONTROL_INTERRUPT;
   bcmessage.messno_compare = 0;		   // Condition looks at message 1
   bcmessage.address        = API_BC_BRANCH_DATA+2; // Compare the data in the third data worf	
   bcmessage.data_mask      = 0xffff;      // Mask value (look at entire data word)	
   bcmessage.data_value     = 0xa002;      // Data value. Branch (TRUE) if data word 3 is equal
   bcmessage.messno_next    = messno+1;    // If condition is FALSE, go here to continue frame
   bcmessage.messno_branch  = messno+2;    // If condition is TRUE,  go here to stop the BC

   status = BusTools_BC_MessageWrite(cardnum, messno, &bcmessage);
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   //Now create each message.
   messno++;
   memset((char*)&bcmessage,0,sizeof(bcmessage));    
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Create a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_INTERRUPT;
   bcmessage.mess_command1.rtaddr   = 3;
   bcmessage.mess_command1.subaddr  = 3;
   bcmessage.mess_command1.wcount   = 3;
   bcmessage.mess_command1.tran_rec = TRANSMIT;
   bcmessage.errorid = 0;                         // Default error injection buffer (no errors)
   bcmessage.long_gap = 20;                       // 20 microsecond inter-message gap.
 
   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);


   //Now create each message.
   messno++;
   memset((char*)&bcmessage,0,sizeof(bcmessage));    
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Create a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_INTERRUPT;
   bcmessage.mess_command1.rtaddr   = 4;
   bcmessage.mess_command1.subaddr  = 4;
   bcmessage.mess_command1.wcount   = 4;
   bcmessage.mess_command1.tran_rec = TRANSMIT;
   bcmessage.errorid = 0;                        // Default error injection buffer (no errors)
   bcmessage.long_gap = 20;                      // 20 microsecond inter-message gap.
 
   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   //Now create each message.
   messno++;
   memset((char*)&bcmessage,0,sizeof(bcmessage));    
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Create a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_RETRY;
   bcmessage.control |= BC_CONTROL_INTERRUPT;
   bcmessage.mess_command1.rtaddr   = 5;
   bcmessage.mess_command1.subaddr  = 5;
   bcmessage.mess_command1.wcount   = 5;
   bcmessage.mess_command1.tran_rec = TRANSMIT;
   bcmessage.errorid = 0;                        // Default error injection buffer (no errors)
   bcmessage.long_gap = 20;                      // 20 microsecond inter-message gap.
 
   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   //BRANCH 2 - CONDITION COUNTER. 
   messno++;
   memset((char*)&bcmessage,0,sizeof(bcmessage));    
   bcmessage.control = BC_CONTROL_CONDITION2;
   bcmessage.cond_counter   = 9;                 // N-1 counter branch on 10th occurence	
   bcmessage.cond_count_val = 9;                 // Starting count	
   bcmessage.messno_next    = messno+1;          // If condition is FALSE, go here to continue frame
   bcmessage.messno_branch  = messno+2;          // If condition is TRUE,  go here to stop the BC

   status = BusTools_BC_MessageWrite(cardnum, messno, &bcmessage);
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   //This loops back to message 0 to restart the frame. 
   messno++;
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(0);
   bcmessage.control = BC_CONTROL_NOP; 
   bcmessage.control |= BC_CONTROL_MFRAME_END;

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

  //This stops the BC at the end of the frame. 
   messno++;
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.control = BC_CONTROL_HALT;        // Stop the Bus Controller

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   return status; 
}

/***************************************************************************
*
*  Function:  bc_intFunction
*
*  Description:
*     This routine is the user callback function invoke when the Board detects
*     a end-of-message condition. This function read the data in the BC message
*     and prints message data to the screen.
*
****************************************************************************/
BT_INT  _stdcall bc_intFunction(BT_UINT cardnum, struct api_int_fifo *sIntFIFO)
{   
   API_BC_MBUF bcmessage;
   int       bufnum;
   static int sa_index;
   int       i,j;
   int       status;
   int       wcount;
   BT_INT tail, rtrt,mc;
   BT_UINT messno;

   tail = sIntFIFO->tail_index;
   while(tail != sIntFIFO->head_index)
   {
      messno = sIntFIFO->fifo[tail].bufferID;
               
      status = BusTools_BC_MessageRead(cardnum,messno,&bcmessage);
	  printf("Word Count - %d  \n",bcmessage.mess_command1.wcount);

      if (status)
         return status;
      rtrt=0;
      mc=0;
      if(bcmessage.status & BT1553_INT_RT_RT_FORMAT)
      {
         printf("RT<->RT (%d) \n",messno);
         rtrt=1;
      }
      else if(bcmessage.status & BT1553_INT_MODE_CODE)
      {
         printf("Mode Code --  (%d) \n", messno);
         mc=1;
      }
      else if (bcmessage.mess_command1.tran_rec)
         printf("BC**RT->BC (%d)  \n", messno);
      else
         printf("BC->RT (%d)  \n", messno);
 
      printf("BC**RT-%d  \n",bcmessage.mess_command1.rtaddr);
      printf("BC**SA-%d  \n",bcmessage.mess_command1.subaddr);
      printf("BC**WC-%d  \n",bcmessage.mess_command1.wcount);
      if(rtrt)
      {   
         printf("BC**RT-%d  \n",bcmessage.mess_command2.rtaddr);
         printf("BC**SA-%d  \n",bcmessage.mess_command2.subaddr);
         printf("BC**WC-%d  \n",bcmessage.mess_command2.wcount);
      }
        
      printf("BC**status-1 0x%04x\n",*((WORD*)(&bcmessage.mess_status1)));
      if(rtrt)
         printf("BC**status-2 0x%04x\n",*((WORD*)(&bcmessage.mess_status2)));
         
      printf("BC**int_stat-0x%08lx\n",*((DWORD*)(&bcmessage.status)));

      wcount = bcmessage.mess_command1.wcount;

      if(mc)
      {
        if(wcount >=16)
           wcount = 1;
        else
           wcount = 0;
      }
      else
      { 
         if (wcount == 0)
            wcount = 32;        // Zero word count means 32 words.
      }

      j = wcount;               // See if this msg shorted than previous
    
      if (bcmessage.control & BC_CONTROL_BUFFERA)
         bufnum = 0;            // Display data from first BC buffer
      else
         bufnum = 1;            // Display data from second BC buffer

      for ( i = 0; i < j; i++ ) // Display all of the data words
      {
         // If this is an RT->BC message, and there is a Message Error
         //  or No Response, than xxxx the data (since the RT did not
         //    send any data words).
         if ( i >= wcount )
         {
            printf("    ");     // Data does not apply to this msg
         }
         else if ( bcmessage.mess_command1.tran_rec &&
           (bcmessage.mess_status1.me || (bcmessage.status&BT1553_INT_NO_RESP) ) )
         {
            printf("xxxx ");    // No data was transfered because of error
         }
         else
         {
            printf("BC**%04x ",bcmessage.data[bufnum][i]);
         }
      }
      printf("\n\n\n");
           
      tail++;
      tail &= sIntFIFO->mask_index;
      sIntFIFO->tail_index = tail;
   }
   fflush(stdout);
   return 0;
}

/****************************************************************************
*
*  Function:  setupThe_BC_IntFIFO
*
*  Description:
*     This routine configures the interrupt FIFO with the values needed for a 
*     interrupts on BC messages.
****************************************************************************/
void setupThe_BC_IntFIFO(int cardnum)
{
   int rt,tr,sa,status;
   // Setup the FIFO structure for this board.
   memset(&sIntFIFO1, 0, sizeof(sIntFIFO1));
   sIntFIFO1.function     = bc_intFunction;
   sIntFIFO1.iPriority    = THREAD_PRIORITY_ABOVE_NORMAL;
   sIntFIFO1.dwMilliseconds = INFINITE;
   sIntFIFO1.iNotification  = 0;
   sIntFIFO1.FilterType     = EVENT_BC_MESSAGE;

   for ( rt=0; rt < 32; rt++ )
      for (tr = 0; tr < 2; tr++ )
         for (sa = 0; sa < 32; sa++ )
            sIntFIFO1.FilterMask[rt][tr][sa] =  0xffffffff;  // Enable all Word Counts on all RT/SA/TR combos

   // Call the register function to register and start the thread.
   status = BusTools_RegisterFunction(cardnum, &sIntFIFO1, REGISTER_FUNCTION);
   printf("BusTools_RegisterFunction status = %d\n",status);
}


