/*============================================================================*
 * FILE:         E X A M P L E _ B C _ D A T A _ T R A N S F E R . C
 *============================================================================*
 *
 *      COPYRIGHT (C) 1998-2016 BY ABACO SYSTEMS, INC. 
 *      ALL RIGHTS RESERVED.
 *
 *      THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND
 *      COPIED ONLY IN ACCORDANCE WITH THE TERMS OF SUCH LICENSE AND WITH
 *      THE INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY
 *      OTHER COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE
 *      AVAILABLE TO ANY OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF THE
 *      SOFTWARE IS HEREBY TRANSFERRED.
 *
 *      THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT
 *      NOTICE AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY ABACO SYSTEMS.
 *
 *===========================================================================*
 *
 * FUNCTION:    EXAMPLE PROGRAM
 *              This example program sets up a Bus Controller to transfer
 *              a block of data to an RT.  This approach could be used to
 *				transfer any amount of data, in this case 1024 words are
 *              transfered.
 *
 *				We use an interrupt event to signal when the BC message
 *              completes so we can write the next buffer of data to the
 *				BC message buffer.  A Conditional Branch on count is used to
 *              transfer 32 messages of 32 words (32 x 32 = 1024).  When the
 *              transfer is complete the Bus Controller halts and the example
 *              exits.  This program also uses BusTools_BC_MessageUpdate to 
 *              update only the data portion of the Bus Controller message 
 *              buffer.
 *
 *===========================================================================*/

/* $Revision:  8.12 Release $
   Date        Revision
  ----------   ---------------------------------------------------------------
	05/30/01	Initial version.  RSW
	08/13/01	Minor cleanup, removed unnecessary BM init code.  RSW.
	03/15/02	Removed conio.h and getc calls.  RSW.
    09/20/04    Added option to initialize with BusTools_API_LoadChannel.  RSW
*/

#include "busapi.h"
#include <stdio.h>

// Constants for our program.
#define RT			1		// RT address 1
#define TR			0		// Receive
#define SA			1		// Subaddress 1
#define WC			32		// Word count
#define FRAME_TIME	50000	// 50ms minor frame
#define DATA_SIZE	1024	// Size (in 16-bit words) of data to transfer.

// Global variables
BT_U16BIT    load_data[DATA_SIZE];  // Array of data to transfer.
BT_U32BIT    data_index = 0;		// Index into our load_data array.
BT_INT       transfer_complete = 0; // Flag to indicate completion of xfer.
API_INT_FIFO sIntFIFO1;             // Interrupt FIFO

// Prototype for event handler function
void setupTheBusController(BT_INT ch_id);
BT_INT _stdcall bcIntFunction(BT_UINT cardnum, struct api_int_fifo *pFIFO);
void setupTheBcInt(BT_INT ch_id);

// Main program
int main(int argc, char **argv)
{
   BT_INT status, ch_id;
   BT_INT i;
   API_INT_FIFO sIntFIFO1;	    // For interrupt handling
   BT_U32BIT loop_count = 0;    // For loop timeout.

   int dev_num, mode;

   if(argc == 2)
     dev_num = atoi(argv[1]);
   else
     dev_num = 0;

   printf("Initializing API with BusTools_API_OpenChannel . . . ");

   // Open the device and get the channel id.
   mode = API_B_MODE | API_SW_INTERRUPT;  // 1553B protocol, use SW interrupts.
   status = BusTools_API_OpenChannel( &ch_id, mode, dev_num, CHANNEL_1);
   printf("BusTools_API_OpenChannel status = %d\n",status);

   // Define our array of data to send from the BC to the RT, load_data[].
   // Note that you could do things like read from a file here.
   for (i=0; i<DATA_SIZE; i++)
      load_data[i] = i;

   // Select External Bus.
   status = BusTools_SetInternalBus(ch_id, EXTERNAL_BUS);
   printf("BusTools_SetInternalBus status = %d\n",status);		

   setupTheBusController(ch_id);
   setupTheBcInt(ch_id);

   // Start the Bus Controller
   status = BusTools_BC_StartStop(ch_id, BC_START);
   printf("BusTools_BC_StartStop status = %d\n",status);

   do
   {
      MSDELAY(100);
   }
   while(BusTools_BC_IsRunning2(ch_id) == API_BC_IS_RUNNING);  // What for the Bus Controller to stop

   // Un-Register our interrupt event handling functions
   status = BusTools_RegisterFunction(ch_id, &sIntFIFO1, UNREGISTER_FUNCTION);
   printf("BusTools_RegisterFunction status =%d\n",status);
		
   // Close API and board
   status = BusTools_API_Close(ch_id);
   printf("BusTools_API_Close status = %d\n");
} // End of main


/****************************************************************************
*
*  Function:  bcIntFunction
*
*  Description:  This callback function updates the data portion of the Bus
*                Controller message buffer that transacted with the next set
*                Data used by the next message.  Data is only updated if the 
*                transacted message was error free.
*
****************************************************************************/
BT_INT _stdcall bcIntFunction(BT_UINT cardnum, struct api_int_fifo *sIntFIFO)
{
   BT_INT				tail,messno;	           // FIFO Tail index
   BT_INT				status;
   API_BC_MBUF          bcmessage;

   tail = sIntFIFO->tail_index;
   while ( tail != sIntFIFO->head_index )
   {
      messno = sIntFIFO->fifo[tail].bufferID;
      status = BusTools_BC_MessageRead(cardnum, messno, &bcmessage);
   
      if(bcmessage.status & BT1553_INT_ERROR_BITS == 0) //Check for transaction errors
      {      
        data_index+=32;
        status = BusTools_BC_MessageUpdate(cardnum, messno, &load_data[data_index]);
      }
      tail++;                         // Next entry
      tail &= sIntFIFO->mask_index;   // Wrap the index
      sIntFIFO->tail_index = tail;    // Save the index
   }
   return( API_SUCCESS );
}

void setupTheBcInt(BT_INT ch_id)
{
   BT_INT status;

   // Setup for our interrupt event handling functions
   memset(&sIntFIFO1, 0, sizeof(API_INT_FIFO));
   sIntFIFO1.function       = bcIntFunction;
   sIntFIFO1.iPriority      = THREAD_PRIORITY_ABOVE_NORMAL;
   sIntFIFO1.dwMilliseconds = INFINITE;
   sIntFIFO1.iNotification  = 0;
   sIntFIFO1.FilterType     = EVENT_BC_MESSAGE;
   sIntFIFO1.FilterMask[RT][TR][SA] = 0xFFFFFFFF;  // Enable all messages for RT/TR/SA

   // Call the register function to register and start the thread.
   status = BusTools_RegisterFunction(ch_id, &sIntFIFO1, REGISTER_FUNCTION);
   printf("BusTools_RegisterFunction status =%d\n",status);
}

void setupTheBusController(BT_INT cardnum)
{
   API_BC_MBUF bcmessage;  // Structure to define BC message
   BT_INT status, messno, i;  // returned status and the BC message number

   // Initialize BC for interrupts, no retries, no resp at 14us, late resp at 12us,
   // minor frame time of FRAME_TIME, 1 data buffer per message.
   status = BusTools_BC_Init(cardnum, 0, BT1553_INT_END_OF_MESS, NO_BC_RETRY,14, 12, FRAME_TIME, 1);
   printf("BusTools_BC_Init status = %d\n",status);

   // Allocate memory on the board for one message block.
   status = BusTools_BC_MessageAlloc(cardnum, 5);
   printf("BusTools_BC_MessageAlloc status = %d\n",status);
		
   // Define data transfer message and write to the allocated BC buffer.
   messno = 0;
   memset((char*)&bcmessage,0,sizeof(API_BC_MBUF));
   bcmessage.messno   = messno;		 // Message number
   bcmessage.control  = BC_CONTROL_MESSAGE;        // This is a standard BC message.
   bcmessage.control |= BC_CONTROL_BUSA;           // Send message on bus A (primary).
   bcmessage.control |= BC_CONTROL_BUFFERA;        // Using buffer A
   bcmessage.control |= BC_CONTROL_MFRAME_BEG;     // Beginning of minor frame.
   bcmessage.control |= BC_CONTROL_INTERRUPT;      // ENABLE INTERRUPT ON THIS MESSAGE.
   bcmessage.messno_next			 = messno + 1; // Next message number, go to msg #1
   bcmessage.mess_command1.rtaddr	 = RT;         // Command word 1, RT address
   bcmessage.mess_command1.subaddr   = SA;         // Command word 1, Subaddress
   bcmessage.mess_command1.tran_rec  = TR;         // Command word 1, transmit (1) or receive (0)
   bcmessage.mess_command1.wcount	 = WC;         // Command word 1, word count, 0-31, 0=32 words
   bcmessage.errorid				 = 0;          // Error injection buffer ID, no err inj
   bcmessage.gap_time				 = 10;         // Intermessage gap time in microseconds

   // Initialize data buffer
   for (i=0; i<WC; i++)
   {								// write appropriate data
      bcmessage.data[0][i] = load_data[i];
   }

   status = BusTools_BC_MessageWrite(cardnum, messno, &bcmessage);
   printf("BusTools_BC_MessageWrite status = %d\n",status);  

   //BRANCH 2 - CONDITION COUNTER. 
   messno++;
   memset((char*)&bcmessage,0,sizeof(bcmessage));    
   bcmessage.control = BC_CONTROL_CONDITION2;
   bcmessage.cond_counter   = (DATA_SIZE/32) - 1; // N-1 counter branch on 10th occurence	
   bcmessage.cond_count_val = (DATA_SIZE/32) - 1; // Starting count	
   bcmessage.messno_next    = messno+1;           // If condition is FALSE, go here to continue frame
   bcmessage.messno_branch  = messno+2;           // If condition is TRUE,  go here to stop the BC

   status = BusTools_BC_MessageWrite(cardnum, messno, &bcmessage);
   printf("BusTools_BC_MessageWrite status = %d\n",status);  

   //This loops back to message 0 to restart the frame. 
   messno++;
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(0);
   bcmessage.control = BC_CONTROL_NOP; 
   bcmessage.control |= BC_CONTROL_MFRAME_END;

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

  //This stops the BC at the end of the frame. 
   messno++;
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.control = BC_CONTROL_HALT;        // Stop the Bus Controller 

   // Write the message to board memory
   status = BusTools_BC_MessageWrite(cardnum, messno, &bcmessage);
   printf("BusTools_BC_MessageWrite status = %d\n",status);

}

