/*===========================================================================*
 * FILE:         E X A M P L E _ B C _ W A T C H D O G _ T I M E R. C
 *===========================================================================*
 *
 *      COPYRIGHT (C) 1998-2016 BY ABACO SYSTEMS, INC. 
 *      ALL RIGHTS RESERVED.
 *
 *      THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND
 *      COPIED ONLY IN ACCORDANCE WITH THE TERMS OF SUCH LICENSE AND WITH
 *      THE INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY
 *      OTHER COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE
 *      AVAILABLE TO ANY OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF THE
 *      SOFTWARE IS HEREBY TRANSFERRED.
 *
 *      THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT
 *      NOTICE AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY ABACO SYSTEMS.
 *
 *============================================================================*
 *
 * FUNCTION:    Demonstration of BusTools/1553-API routines. This example shows 
 * how to configure a watchdog timer to stop the Bus Controller if the host does 
 * not reset the timer.
 * 
 * This example uses a Condition Branch on count (CONDITION2) as the timer. 
 * The Conditional Branch is setup to branch when the count decrements to 0.  
 * The count value is decremented each time the branch executes.  The branch 
 * command is a BC_CONTROL_HALT that stops the Bus Controller. Otherwise the frame 
 * keeps running. To keep the branch from ever executing the halt, the host must 
 * reset the counter value to prevent it from reaching 0. To do this, the example 
 * gets the address of the BC buffer containing the branch command and resets the 
 * count value by writing the initial count value back into the count register 
 * thus restarting the down-count.  
 * 
 * This example runs until the user enters the kill command from the console.  
 * Then the count value is no longer reset and the Bus controller halts.  Two 
 * callback functions are used.  One for Bus Controller messages prints the message 
 * data to the console.  The other is for the conditional branch. This callback 
 * resets the counter value.  See how altering the count value affects the timer.
 *
 *===========================================================================*/

/* $Revision:  8.12 Release $
   Date        Revision
  ----------   ---------------------------------------------------------------
  6/11/2001    initial release
*/

/*---------------------------------------------------------------------------*
 *                     INCLUDES, DEFINES and GLOBALS
 *---------------------------------------------------------------------------*/

#include <stdio.h>
#include "busapi.h"

//forward declarations
BT_INT BusController_Init(int,int);
void setupThe_BC_IntFIFO(int);
void setupTheTrigger(BT_INT cardnum);
void setupThe_Watchdog_Timer_IntFIFO(int cardnum);

API_INT_FIFO  sIntFIFO1;          // Thread FIFO structure for BC
API_INT_FIFO  sIntFIFO2;          // Thread FIFO for branch
BT_INT KILL_THE_BC = 0;           // activate deadman timer
BT_U32BIT m_msgAddr, bc_dataAddr;

//******************************************************************************
//*                                                                            *
//******************************************************************************
int main(int argc, char **argv)                     
{
   BT_UINT status, cardnum;
   BT_UINT mode=3;
   BT_INT device = 0;                     // Default device ID
   char ichar;

   if(argc == 2)
      device = atoi(argv[1]);             // Read command line device input

   mode = API_B_MODE | API_SW_INTERRUPT;  // 1553B protocol, use SW interrupts.
   status = BusTools_API_OpenChannel( &cardnum, mode, device, CHANNEL_1);
   printf("BusTools_API_OpenChannel status=%d\n", status, BusTools_StatusGetString(status));

   // Set the Bus to External for BC
   status = BusTools_SetInternalBus(cardnum,EXTERNAL_BUS);  // Set up External bus
   printf("BusTools_SetInternalBus BC status = %d (%s)\n",status, BusTools_StatusGetString(status));

   // Set the time mode for both channels
   status = BusTools_TimeTagMode(cardnum, API_TTD_IRIG_NS, API_TTI_IRIG, API_TTM_FREE, NULL, 0, 0, 0 );
   printf("BusTools_TimeTagMode BC status = %d (%s)\n",status, BusTools_StatusGetString(status));

   // Set up the BC for Trigger-User mode
   status = BusController_Init(cardnum,0);

   // Setting up a callback function to show the BC data
   setupThe_BC_IntFIFO(cardnum);             // Sets up BC message interrupt
   setupThe_Watchdog_Timer_IntFIFO(cardnum); // Sets the callback to reset the timer so BC does not stop

   printf("Hit Enter to start BC.\nHit k kill the BC\nHit d to dump board's memory\nHit q to stop\n");
   getchar();
   fflush(stdin);

   // Start the BC.  BC wait until trigger input to start each user-defined set of frames.
   status = BusTools_BC_StartStop(cardnum,BC_START);
   printf("BusTools_BC_StartStop status = %d\n",status);

   do
   {
      ichar = getchar();
      switch(ichar)
      {
         case 'k':
         case 'K':
            KILL_THE_BC = 1;
            break;
         case 'd':
         case 'D':
            BusTools_DumpMemory(cardnum,DUMP_ALL,"bc_trigger_dump.txt","exmaple_bc_trigger-user Dumping Memory");
            break;
      }
   }while (ichar != 'q' && ichar != 'Q');

   printf("main: closing API...\n");

   // Stop the bus controller
   status = BusTools_BC_StartStop(cardnum,BC_STOP); 
   printf("BusTools_BC_StartStop status = %d\n",status);

   // Unregister BC Message callback function
   status = BusTools_RegisterFunction(cardnum,&sIntFIFO1,UNREGISTER_FUNCTION);
   printf("unregistering via BusTools_RegisterFunction status = %d\n",status);

   // Unregister BC Control callback function
   status = BusTools_RegisterFunction(cardnum,&sIntFIFO2,UNREGISTER_FUNCTION);
   printf("unregistering via BusTools_RegisterFunction status = %d\n",status);

   // Close the channel
   status = BusTools_API_Close(cardnum);
   printf("BusTools_API_Close BC status = %d\n",status);
}


/****************************************************************************
*
*  Function:  BusController_Init
*
*  Description:
*     This function initialize the Bus Controller and creates the messages in
*     the frame.  The last message in the frame is a conditional branch on count.
*     When the count has not reached zero (0), the next message is the start of
*     the frame.  If the count reaches zero (0), the branch command is a
*     BC_CONTROL_HALT.
*
****************************************************************************/
int BusController_Init(int cardnum,int sa_index)
{
   BT_INT status, messno;
   API_BC_MBUF bcmessage;

   // Initialize the BC.  
   status = BusTools_BC_Init( cardnum, REL_GAP, BT1553_INT_END_OF_MESS, NO_BC_RETRY, 14, 12, 1000000, 1 );
   printf("BusTools_BC_Init status = %d\n",status);

   // Allocate enough BC Buffer
   status = BusTools_BC_MessageAlloc(cardnum,50);
   printf("BusTools_BC_MessageAlloc status = %d\n",status);

   // Create messages for The Minor Frame
   messno = 0; // Msg 0
   memset((char*)&bcmessage,0,sizeof(bcmessage));    
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Create a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // Use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_INTERRUPT;
   bcmessage.control |= BC_CONTROL_MFRAME_BEG;
   bcmessage.mess_command1.rtaddr   = 1; 
   bcmessage.mess_command1.subaddr  = 1;
   bcmessage.mess_command1.wcount   = 1;
   bcmessage.mess_command1.tran_rec = RECEIVE;
   bcmessage.errorid = 0;                         // Default error injection buffer (no errors)
   bcmessage.gap_time = 100;                      // inter-message gap in microseconds.
   bcmessage.data[0][0] = 1;
   bcmessage.data[0][1] = 1;

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage);
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   messno++;  // Msg 1
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Create a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // Use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_INTERRUPT;
   bcmessage.mess_command1.rtaddr   = 2;
   bcmessage.mess_command1.subaddr  = 2;
   bcmessage.mess_command1.wcount   = 2;
   bcmessage.mess_command1.tran_rec = TRANSMIT;
   bcmessage.errorid = 0;                         // Default error injection buffer (no errors)
   bcmessage.gap_time = 100;                      // inter-message gap in microseconds.

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage);
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   messno++;  // Msg 2
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Create a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // Use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_INTERRUPT;
   bcmessage.mess_command1.rtaddr   = 10;
   bcmessage.mess_command1.subaddr  = 10;
   bcmessage.mess_command1.wcount   = 10;
   bcmessage.mess_command1.tran_rec = RECEIVE;
   bcmessage.errorid = 0;                         // Default error injection buffer (no errors)
   bcmessage.gap_time = 100;                      // inter-message gap in microseconds.

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   messno++;  // Msg 3
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Create a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // Use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_INTERRUPT;
   bcmessage.mess_command1.rtaddr   = 11;
   bcmessage.mess_command1.tran_rec = 1;
   bcmessage.mess_command1.subaddr  = 11;
   bcmessage.mess_command1.wcount   = 11;
   bcmessage.errorid = 0;                         // Default error injection buffer (no errors)
   bcmessage.gap_time = 100;                      // inter-message gap in microseconds.

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   messno++;  // 4
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Create a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_INTERRUPT;
   bcmessage.mess_command1.rtaddr   = 20;
   bcmessage.mess_command1.subaddr  = 20;
   bcmessage.mess_command1.wcount   = 20;
   bcmessage.mess_command1.tran_rec = TRANSMIT;
   bcmessage.errorid = 0;                        // Default error injection buffer (no errors)
   bcmessage.gap_time = 100;                     // inter-message gap in microseconds.

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   messno++;  // 5
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Create a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // Use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_INTERRUPT;
   bcmessage.mess_command1.rtaddr   = 21;
   bcmessage.mess_command1.subaddr  = 21;
   bcmessage.mess_command1.wcount   = 21;
   bcmessage.mess_command1.tran_rec = TRANSMIT;
   bcmessage.errorid = 0;                         // Default error injection buffer (no errors)
   bcmessage.gap_time = 100;                      // inter-message gap in microseconds.

   status = BusTools_BC_MessageWrite(cardnum, bcmessage.messno, &bcmessage);
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   // Setup deadman message branch to halt after 1 time if counter value is not reset.
   messno++;  // 6
   memset( (char*)&bcmessage, 0, sizeof(bcmessage) );  // Clear the bcmessage structure
   bcmessage.messno         = messno;
   bcmessage.messno_next    = (BT_U16BIT)(messno + 1); // Next message end-of-frame
   bcmessage.messno_branch  = (BT_U16BIT)(messno + 2); // Last message ID BC Halt
   bcmessage.control        = BC_CONTROL_CONDITION2;   // Branch command
   bcmessage.control       |= BC_CONTROL_INTERRUPT;    // Set interrupt on branch;
   bcmessage.cond_counter   = 1;                       // Counter initial value
   bcmessage.cond_count_val = 1;                       // Reload value

   status = BusTools_BC_MessageWrite(cardnum, bcmessage.messno, &bcmessage);
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   // Get the address of the BC Conditional Branch message buffer
   status = BusTools_BC_MessageGetaddr(cardnum, bcmessage.messno, &m_msgAddr);
   if(BusTools_BoardIsV6(cardnum))  // If using V6 firmware get address of the data buffer
   {
      BusTools_MemoryRead2(cardnum,RAM,m_msgAddr+BC_MSG_DATA_OFFSET,4,&bc_dataAddr);
      bc_dataAddr = BusTools_RelAddr(cardnum, bc_dataAddr); // This a absolute adddress.  Convert to relative
      printf("Data Buffer Address 0x%08x\n", bc_dataAddr);  // Print the address of the data buffer
   }

   //==========================================================================
   // This is the bus list END-FRAME
   messno++; //7
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(0);      // Go back to start of the frame
   bcmessage.control =  BC_CONTROL_MFRAME_END;  // End-frame marker
   bcmessage.control |= BC_CONTROL_NOP;

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   //==========================================================================
   // This Halts the BC. 
   messno++; //8
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(0); // next message is a don't care
   bcmessage.control |= BC_CONTROL_HALT;   // stop the BC and clears the BC_ENABLE_EXTERNAL_SYNC

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   return status;
} 

/****************************************************************************
*
*  Function:  bc_intFunction
*
*  Description:
*     This routine is the user callback function invoke when the Board detects
*     a end-of-message condition. This function read the data in the BC message
*     and print message data to the screen.
*
****************************************************************************/
BT_INT  _stdcall bc_intFunction(BT_UINT cardnum, struct api_int_fifo *sIntFIFO)
{   
   API_BC_MBUF bcmessage;
   static int sa_index;
   int       i,j;
   int       status;
   int       wcount;
   BT_INT tail;
   BT_UINT messno;
   static int nFlag = 1;

   char outbuf[80];

   tail = sIntFIFO->tail_index;
   while(tail != sIntFIFO->head_index)
   {
      messno = sIntFIFO->fifo[tail].bufferID;
      status = BusTools_BC_MessageRead(cardnum,messno,&bcmessage);    
      if (status)
         return status;

      printf("%d - %x\n",messno,bcmessage.status);

	  if(BusTools_BoardIsV6(cardnum))
         BusTools_TimeGetFmtString(API_TTD_IRIG_NS,&bcmessage.time_tag, outbuf);
	  else
         BusTools_TimeGetFmtString(API_TTD_IRIG,&bcmessage.time_tag, outbuf);

      printf("tag time = %s\n",outbuf);
       
      if (bcmessage.mess_command1.tran_rec)
         printf("BC**RT->BC (%d)  ", messno);
      else
         printf("BC->RT (%d)  ", messno);
 
      printf("BC**RT-%d  \n",bcmessage.mess_command1.rtaddr);
      printf("BC**SA-%d  \n",bcmessage.mess_command1.subaddr);
      printf("BC**WC-%d  \n",bcmessage.mess_command1.wcount);
      printf("BC**status-1 0x%04x\n",*((WORD*)(&bcmessage.mess_status1)));
      printf("BC**int_stat-0x%08lx\n",*((DWORD*)(&bcmessage.status)));
      wcount = bcmessage.mess_command1.wcount;
      if (wcount == 0)
         wcount = 32;           // Zero word count means 32 words.

      j = wcount;               // See if this msg shorted than previous
    
      for ( i = 0; i < j; i++ ) // Display all of the data words
      {
         if ( i >= wcount )
         {
            printf("    ");     // Data does not apply to this msg
         }
         printf("BC**%04x ",bcmessage.data[0][i]);
      }
      printf("\n");
               
      tail++;
      tail &= sIntFIFO->mask_index;
      sIntFIFO->tail_index = tail;
   }

   return 0;
} 

/****************************************************************************
*
*  Function:  setupThe_BC_IntFIFO
*
*  Description:
*     This routine setup the interrupt FIFO with the values needed for a 
*     interrupt on a BC messages.  The interrupt is set for the end-of message.  
*
****************************************************************************/
void setupThe_BC_IntFIFO(int cardnum)
{
   int rt,tr,sa,status;
   // Setup the FIFO structure for this board.
   memset(&sIntFIFO1, 0, sizeof(sIntFIFO1));
   sIntFIFO1.iNotification = 0;
   sIntFIFO1.function     = bc_intFunction;
   sIntFIFO1.iPriority    = THREAD_PRIORITY_ABOVE_NORMAL;
   sIntFIFO1.dwMilliseconds = INFINITE;
   sIntFIFO1.bForceStartup  = 0;
   sIntFIFO1.FilterType     =  EVENT_BC_MESSAGE;

   for ( rt=0; rt < 32; rt++ )
      for (tr = 0; tr < 2; tr++ )
         for (sa = 0; sa < 32; sa++ )
            sIntFIFO1.FilterMask[rt][tr][sa] =  0xffffffff;  // Enable all Word Counts on all RT/SA/TR combos

   // Call the register function to register and start the thread.
   status = BusTools_RegisterFunction(cardnum, &sIntFIFO1, REGISTER_FUNCTION);
   printf("BusTools_RegisterFunction status = %d\n",status);
}

/****************************************************************************
*
*  Function:  deadman_timer_reset
*
*  Description:
*     This routine is the user callback function invoked when the Board detects
*     a BC control message. In this example it is the Conditional Branch. If the
*     "KILL_THE_BC" is not set the callback resets the Conditional Branch counter
*      value to prevent it from decrementing to 0.
*
****************************************************************************/
BT_INT  _stdcall watchdog_timer_reset(BT_UINT cardnum, struct api_int_fifo *sIntFIFO)
{   
   BT_INT status, tail;
   BT_U32BIT reset32=1;  // For V6 32-bit value required
   BT_U16BIT reset16=1;  // For V4/5 16 bit value required

   printf("********watchdog_timer_reset s\n");
   tail = sIntFIFO->tail_index;
   while (tail != sIntFIFO->head_index)
   {
      if(!KILL_THE_BC)  // If the user has not ask to stop the BC reset the counter value 
                        // to keep the branch on count from activating. 
      { 
         if(BusTools_BoardIsV6(cardnum)) // Use BusTools_MemoryWrite2 for V6 firmware.
            status = BusTools_MemoryWrite2(cardnum, RAM32, bc_dataAddr + BC_CTRLDATA_COND_CTR_OFFSET, 4, &reset32);
         else
            status = BusTools_MemoryWrite(cardnum, (m_msgAddr+(6*2)), 2, &reset16);         
      }

      tail++;
      tail &= sIntFIFO->mask_index;
      sIntFIFO->tail_index = tail;
   }
   return API_SUCCESS;
}


/****************************************************************************
*
*  Function:  etupThe_Watchdog_Timer_IntFIFO
*
*  Description:
*     This routine setup the interrupt FIFO with the value needed for a 
*     interrupt on a BC message.  The interrupt is set for only the end-of
*     message.  
*
*     Any errors detected are reported to the user.  The user
*     has the option of terminating the test by striking any key.
*
****************************************************************************/
void setupThe_Watchdog_Timer_IntFIFO(int cardnum)
{
    int status;
    // Setup the FIFO structure for this board.
    memset(&sIntFIFO2, 0, sizeof(API_INT_FIFO));  // Zero out the entire structure
    sIntFIFO2.function       = watchdog_timer_reset;
    sIntFIFO2.iPriority      = THREAD_PRIORITY_NORMAL;
    sIntFIFO2.dwMilliseconds = INFINITE;
    sIntFIFO2.iNotification  = 0;
    sIntFIFO2.FilterType     = EVENT_BC_CONTROL;

    // Call the register function to register and start the thread.
    status = BusTools_RegisterFunction(cardnum, &sIntFIFO2, REGISTER_FUNCTION);
    printf("BusTools_RegisterFunction status = %d\n",status);
}
