/*============================================================================*
 * FILE:                 E X A M P L E _ B C _ E R R O R _ I N J . C
 *============================================================================*
 *
 *      COPYRIGHT (C) 1998-2016 BY ABACO SYSTEMS, INC. 
 *      ALL RIGHTS RESERVED.
 *
 *      THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND
 *      COPIED ONLY IN ACCORDANCE WITH THE TERMS OF SUCH LICENSE AND WITH
 *      THE INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY
 *      OTHER COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE
 *      AVAILABLE TO ANY OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF THE
 *      SOFTWARE IS HEREBY TRANSFERRED.
 *
 *      THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT
 *      NOTICE AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY ABACO SYSTEMS.
 *
 *===========================================================================*
 *
 * FUNCTION:    EXAMPLE PROGRAM
 *    This example program sets up a Bus Controller message list containing two  
 *    messages, 1-R-1-32 and 2-T-2-32, in a 20ms minor frame.  This BC list runs  
 *    until stopped by user input.  Error are inject into the command or data  
 *    words of the two messages.
 *   
 *   The example injects a PARITY error on the command word either for 1-R-1-32 
 *   or 2-T-2-32.  It also inject a parity error into the first data of the receive 
 *   command.  Notice the effect of the error by monitoring with an external Bus 
 *   Analyzer like BusTools/1553.  When a parity error is injected on a command 
 *   word it becomes an invalid command the Analyzer (monitor) ignores the message. 
 *   If error is injected on the data word then the command is valid and the monitor 
 *   will show errors like invalid word and parity error.
 *===========================================================================*/

/* $Revision:  8.12 Release $
   Date        Revision
  ----------   ---------------------------------------------------------------
	03/14/02	Initial version.  RSW
    09/20/04    Added option to initialize with BusTools_API_LoadChannel.  RSW
*/

#include "busapi.h"
#include <stdio.h>

void bc_init(BT_INT ch_id);
void define_errors(BT_INT ch_id);

API_ENH_EIBUF error_buffer;  // Error injection buffer

// Main program
int main(int argc, char** argv) 
{
   BT_INT        status, ch_id;
   char c;
   BT_UINT device, mode;

   if(argc == 2)
      device = atoi(argv[1]);   // Command-line device input
   else
      device = 0;               // Default device ID

   // Open the device and get the channel id.
   mode = API_B_MODE | API_SW_INTERRUPT;  // 1553B protocol, use SW interrupts.
   status = BusTools_API_OpenChannel( &ch_id, mode, device, CHANNEL_1);
   printf("BusTools_API_OpenChannel status = %d\n",status);

   // Select External Bus.
   status = BusTools_SetInternalBus(ch_id, EXTERNAL_BUS);
   printf("BusTools_SetInternalBus status = %d\n",status);

   // Define the error injectin buffer that the BC will use
   define_errors(ch_id)	;	

   // Initialize the Bus Controller and setup the two messages
   bc_init(ch_id);

   // Start the bus controller, will start list at msg #0.
   status = BusTools_BC_StartStop(ch_id, BC_START);
		
   // BC is up and running, wait for user to tell us to stop or read data.
   do
   {
      printf("\nInput Q to stop BC and exit, R to inj err on 1-R-1-32, \n");
      printf("T to inj err on 2-T-2-32, C to clear errors.\n");
      scanf("%c",&c);
      fflush(stdin);

      switch (c) 
      {
      case 'R':  // Inj error on 1-R-1-32 Command word
      case 'r':
         error_buffer.buftype = EI_BC_REC;
         error_buffer.error[0].etype = EI_PARITY;
         status = BusTools_EI_EbufWriteENH(ch_id, 1, &error_buffer);
         printf("BusTools_EI_EbufWriteENH Status = %d\n",status);
         break;
      case 'T': // Inj error on 2-T-2-32 Command word
      case 't':
         error_buffer.buftype = EI_BC_TRANS;
         error_buffer.error[0].etype = EI_PARITY;
         status = BusTools_EI_EbufWriteENH(ch_id, 2, &error_buffer);
         printf("BusTools_EI_EbufWriteENH Status = %d\n",status);
         break;
      case 'D':  // Inj error on 1-R-1-32 data word 1
      case 'd':
         error_buffer.buftype = EI_BC_REC;
         error_buffer.error[1].etype = EI_PARITY;
         status = BusTools_EI_EbufWriteENH(ch_id, 1, &error_buffer);
         printf("BusTools_EI_EbufWriteENH Status = %d\n",status);
         break;
      case 'C':  // Clear errors
      case 'c':
         define_errors(ch_id);
         break;
      } // End of switch (c)
   } while (c != 'Q' && c != 'q'); // End of do-while

   // Stop the BC.
   status = BusTools_BC_StartStop(ch_id, BC_STOP);
   printf("BusTools_BC_StartStop Status = %d\n",status);

   // Close API and board
   status = BusTools_API_Close(ch_id);
   printf("BusTools_API_Close Status = %d\n",status);		

} // End of main

void define_errors(BT_INT ch_id)
{
   BT_INT i, status;
   // Define two error injection buffers, no errors initially.
   // RECEIVE Error buffer 1 for Message 0
   error_buffer.buftype = EI_BC_REC;
   for (i=0; i<EI_COUNT; i++)
   {
      error_buffer.error[i].etype = EI_NONE;
      error_buffer.error[i].edata = 0;
   }
   status = BusTools_EI_EbufWriteENH(ch_id, 1, &error_buffer);
   printf("BusTools_EI_EbufWrite status = %d\n",status);
		
   // TRANSMIT Error buffer 2 for Message 1
   error_buffer.buftype = EI_BC_TRANS;
   status = BusTools_EI_EbufWriteENH(ch_id, 2, &error_buffer);
   printf("BusTools_EI_EbufWrite status = %d\n",status);
}

void bc_init(BT_INT ch_id)
{
   API_BC_MBUF bc_msg;	    // Structures to define BC messages
   BT_U16BIT   messno;
   BT_INT status, i;

   // Set up the BC.
   // Initialize BC for no interrupts, no retries, no resp at 16us, late resp at 14us,
   // minor frame time of 20000us, 1 data buffer per message.
		
   status = BusTools_BC_Init(ch_id, REL_GAP, NO_INTERUPT, NO_BC_RETRY, 16, 14, 20000, 1);
   printf("BusTools_BC_Init status = %d\n",status);	

   // Allocate memory on the board for two message blocks.
   status = BusTools_BC_MessageAlloc(ch_id, 10);
   printf("BusTools_BC_MessageAlloc status = %d\n",status);	
		
   // Define the messages and write them to the allocated board memory.

   messno = 0;
   memset((char*)&bc_msg,0,sizeof(API_BC_MBUF));
   bc_msg.messno = messno;                       // Message number
   bc_msg.messno_next = (messno + 1);            // Next message number, go to msg #1
   bc_msg.control = BC_CONTROL_MESSAGE;          // This is a BC message.
   bc_msg.control |= BC_CONTROL_BUSA;            // Send message on bus A (primary).
   bc_msg.control |= BC_CONTROL_BUFFERA;         // Only using one buffer, buffer A
   bc_msg.control |= BC_CONTROL_MFRAME_BEG;	     // Beginning of minor frame.
   bc_msg.mess_command1.rtaddr	 = 1;            // Command word 1, RT address
   bc_msg.mess_command1.subaddr  = 1;            // Command word 1, Subaddress
   bc_msg.mess_command1.tran_rec = RECEIVE;      // Command word 1, receive (0)
   bc_msg.mess_command1.wcount	 = 0;            // Command word 1, word count, 0-31, 0=32 words
   bc_msg.errorid				 = 1;            // Error injection buffer ID
   bc_msg.gap_time				 = 10;           // Intermessage gap time in microseconds
   // Fill data buffer
   for (i=0; i<32; i++) bc_msg.data[0][i] = 0xAB00 + i;
   // Write the message to board memory
   status = BusTools_BC_MessageWrite(ch_id, 0, &bc_msg);
   printf("BusTools_BC_MessageWrite status = %d\n",status);	

   messno++;
   memset((char*)&bc_msg,0,sizeof(API_BC_MBUF));
   bc_msg.messno = messno;                       // Message number
   bc_msg.messno_next = 0;                       // Next message number, go to msg #0.
   bc_msg.control = BC_CONTROL_MESSAGE;          // This is a BC message.
   bc_msg.control |= BC_CONTROL_BUSA;            // Send message on bus A (primary).
   bc_msg.control |= BC_CONTROL_BUFFERA;         // Only using one buffer, buffer A.
   bc_msg.control |= BC_CONTROL_MFRAME_END;      // End of minor frame.
   bc_msg.mess_command1.rtaddr	 = 2;            // Command word 2, RT address
   bc_msg.mess_command1.subaddr  = 2;            // Command word 2, Subaddress
   bc_msg.mess_command1.tran_rec = TRANSMIT;     // Command word 1, transmit
   bc_msg.mess_command1.wcount	 = 0;            // Command word 1, 0=32 words
   bc_msg.errorid				 = 2;            // Error injection buffer ID
   bc_msg.gap_time				 = 10;           // Intermessage gap time in microseconds

   // Write the message to board memory
   status = BusTools_BC_MessageWrite(ch_id, 1, &bc_msg);
   printf("BusTools_BC_MessageWrite status = %d\n",status);
}
