/*============================================================================*
 * FILE:        E X A M P L E _ B C _ M S G _ B L K _ R E A D. C
 *============================================================================*
 *
 *      COPYRIGHT (C) 1998-2016 BY ABACO SYSTEMS, INC. 
 *      ALL RIGHTS RESERVED.
 *
 *      THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND
 *      COPIED ONLY IN ACCORDANCE WITH THE TERMS OF SUCH LICENSE AND WITH
 *      THE INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY
 *      OTHER COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE
 *      AVAILABLE TO ANY OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF THE
 *      SOFTWARE IS HEREBY TRANSFERRED.
 *
 *      THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT
 *      NOTICE AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY ABACO SYSTEMS.
 *
 *===========================================================================*
 *
 * FUNCTION:   This example program shows how to program the Bus Controller
 *             to run a simple three message minor-frame. The example
 *             initializes the channel the Bus Controller is running on. 
 *             Then sets the channel for external bus.  Following 
 *             those initialization steps it initializes the Bus Controller and
 *             builds a simple 3-message frame.  The message in the frame are
 *             linked together and the frame will run continually until stopped
 *             by the user.
 *
 *             This example also shows how to process the messages using the 
 *             function BusTools_BC_ReadLastMessageBlock.  This function parses
 *             the interrupt queue to determine the BC messages that have
 *             transacted since the last call.  In order for a BC message to 
 *             record it must have the BC_CONTROL_INTERRUPT set and the BC
 *             initialization must define an interrupt condition like 
 *             BT1553_INT_END_OF_MESS.  This processing differs from BusTools_
 *             RegisterFunction in that your application must provide a timing
 *             loop to call BusTools_BC_ReadLastMessageBlock periodically.  The
 *             timing must set so the BC messages (and other messages that may be
 *             recorded in the interrupt queue do not overwrite unprocessed 
 *             entries.  This examples uses a 50 millisecond delay.  
 *
 *             This application use the Windows function kbhit to break out of 
 *             timing loop.  If using this example on non-Windows systems, you 
 *             will need to provide a kbhit function.
 *             
 *===========================================================================*/

/* $Revision:  8.12 Release $
   Date        Revision
  ----------   ---------------------------------------------------------------
  10/31/2013   Initial version.  
*/

#include "busapi.h"
#include <stdio.h>
#ifdef WIN32
#include <conio.h>
#endif //WIN32

void bc_init(int cardnum);
BT_INT  bc_readFunction(BT_UINT cardnum);

// Main program
int main(int argc, char **argv) 
{
   BT_INT status,cardnum,noop=0;
   char c;
   BT_INT dev_num, mode;

   printf("Initializing API with BusTools_API_OpenChannel . . . ");
 
   //Set the device ID 
   if(argc == 2)
      dev_num = atoi(argv[1]);
   else
      dev_num = 0;

   // Open the device and get the channel handle.
   mode = API_B_MODE | API_SW_INTERRUPT;                      // 1553B protocol using SW interrupts.
   status = BusTools_API_OpenChannel(&cardnum, mode, dev_num, CHANNEL_1);
   printf("BusTools_API_OpenChannel status = %d\n",status);

   // Select External Bus.
   status = BusTools_SetInternalBus(cardnum, EXTERNAL_BUS);
   printf("BusTools_SetInternalBus status = %d\n",status);
   
   // Initialize the time tag to current system clock time.
   BusTools_TimeTagMode(cardnum,API_TTD_DATE,API_TTI_IRIG,API_TTM_FREE,0,0,0,0);
        
   bc_init(cardnum);

   printf("\nHit Enter to start the BC\n");            
   printf("type Q/q to Quit\n");
   c = getc(stdin);
   fflush(stdin);

   //Start the bus controller.
   status = BusTools_BC_StartStop(cardnum, BC_START);
   printf("BusTools_BC_StartStop status = %d\n",status);

   // BC is running.  Process the data
   do
   {
      MSDELAY(50);                         // Delay for 50 Milliseconds
	  status = bc_readFunction(cardnum);   // Read data from the interrupt queue using BusTools_ReadLastMessageBlock

      if(status != API_SUCCESS && status != API_BC_READ_NODATA)
         printf("bc_readFunction status = %d\n",status);
  
   }while(!kbhit());                       // Exit loop using Windows kbhit.  Must replace for non-Windows O/S


   // Stop the BC.
   status = BusTools_BC_StartStop(cardnum, BC_STOP);
   printf("BusTools_BC_StartStop status = %d\n",status);

   // Close API and board
   status = BusTools_API_Close(cardnum);
   printf("BusTools_API_Close status = %d\n",status);

   printf("FINISHED.\n");
} // End of main

void bc_init(int cardnum)
{
   int status, messno,i;
   API_BC_MBUF bcmessage;

   // Initialize BC for End-of-Message interrupts, no retries, no resp at 16us, late resp at 14us,
   // minor frame time of 500000us, 1 data buffer per message.
   status = BusTools_BC_Init(cardnum, REL_GAP, BT1553_INT_END_OF_MESS, NO_BC_RETRY, 16, 14, 500000, 1);
   printf("BusTools_BC_Init status = %d\n",status);
   
   // Allocate memory on the board for 10 message buffers.
   // Only three are used in this example.
   status = BusTools_BC_MessageAlloc(cardnum, 10);
   printf("BusTools_BC_MessageAlloc status = %d\n",status);
		
   // Write the Bus Controllere messages to the allocated message buffers.

   //RT2 SA2 WC2 RX  
   messno = 0;                                     // Set the message buffer index to the first buffer
   memset((char*)&bcmessage,0,sizeof(bcmessage));  // Clear out the message buffer data       
   bcmessage.messno = messno;                      // Current buffer index
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);// Link to next message buffer
   bcmessage.control = BC_CONTROL_MESSAGE;         // Set as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;        // use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;           // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_INTERRUPT;      // Enable interrupt on this message
   bcmessage.control |= BC_CONTROL_MFRAME_BEG;     // Start of the frame
   bcmessage.mess_command1.rtaddr   = 2;           // RT Address 2
   bcmessage.mess_command1.subaddr  = 2;           // Subassress 2
   bcmessage.mess_command1.wcount   = 2;           // Word count 2
   bcmessage.mess_command1.tran_rec = 0;           // receive command.
   bcmessage.errorid = 0;                          // Default error injection buffer (no errors)
   bcmessage.gap_time = 15;                        // 15 microsecond inter-message gap_time.
   // Fill in the data for the receive command
   for(i=0;i<2;i++)
       bcmessage.data[0][i] = 0xA000+i;            // Add data for the specified number of data elements
   // Write the message buffer data to the buffer
   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage);
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   //RT3 SA3 WC3 TX  
   messno++;                                       // Index to the next message buffer                                             
   memset((char*)&bcmessage,0,sizeof(bcmessage));  // Clear out the message buffer data
   bcmessage.messno = messno;                      // Current buffer index
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);// Link to next message buffer
   bcmessage.control = BC_CONTROL_MESSAGE;         // Set as a NOP message
   bcmessage.control |= BC_CONTROL_BUFFERA;        // Use buffer A
   bcmessage.control |= BC_CONTROL_BUSB;           // Transmit on Bus B
   bcmessage.control |= BC_CONTROL_INTERRUPT;      // Enable interrupt on this message
   bcmessage.mess_command1.rtaddr   = 3;           // RT Address 3
   bcmessage.mess_command1.subaddr  = 3;           // Subaddress 3
   bcmessage.mess_command1.wcount   = 3;           // Word count 3
   bcmessage.mess_command1.tran_rec = 1;           // Transmit command 
   bcmessage.errorid = 0;                          // Default error injection buffer (no errors)
   bcmessage.gap_time = 15;                        // 15 microsecond inter-message gap_time
   // Write the message buffer data to the buffer 
   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage);
   printf("BusTools_BC_MessageWrite status = %d\n",status);


   //RT4 SA4 WC4 TX 
   messno++;                                       // Index to the next message buffer                                             
   memset((char*)&bcmessage,0,sizeof(bcmessage));  // Clear out the message buffer data
   bcmessage.messno = messno;                      // Current buffer index
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);// Link to next message buffer
   bcmessage.control = BC_CONTROL_MESSAGE;         // show as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;        // use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;           // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_INTERRUPT;      // Enable interrupt on this message
   bcmessage.mess_command1.rtaddr   = 4;           // RT address 4
   bcmessage.mess_command1.subaddr  = 4;           // Subaddress 4
   bcmessage.mess_command1.wcount   = 4;           // Word count 4
   bcmessage.mess_command1.tran_rec = 1;           // Transmit command
   bcmessage.errorid = 0;                          // Default error injection buffer (no errors)
   bcmessage.gap_time = 15;                        // 15 microsecond inter-message gap_time.
   // Write the message buffer data to the buffer
   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   messno++;                                       // Index to the next message buffer                                             
   memset((char*)&bcmessage,0,sizeof(bcmessage));  // Clear out the message buffer data
   bcmessage.messno = messno;                      // Current buffer index
   bcmessage.messno_next = (BT_U16BIT)0;           // link back to the first message buffer
   bcmessage.control = BC_CONTROL_NOP;             // Noop buffer
   bcmessage.control |= BC_CONTROL_MFRAME_END;     // End of frame
   // Write the message buffer data to the buffer
   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

} 

/****************************************************************************
*
*  Function:  bc_readFunction
*
*  Description:
*     This function calls BusTools_BC_ReadLastMessageBlock to read the BC data 
*     stored in the interrupt queue.  It must be called at a periodic rate that
*     ensure the unprocessed data in the interrupt queue is not overwritten.  
*     In this example the array bcmessage is set to 100.  You can modify that
*     based on the maximum expected messages.  The most message that can be
*     processed is the size of the queue (296 for V4/5 firmware and 512 for
*     V6 firmware).
*
*****************************************************************************/
BT_INT bc_readFunction(BT_UINT cardnum)
{   
   API_BC_MBUF bcmessage[100];
   BT_INT i, j, status, wcount, bufnum;
   BT_UINT msg_index, mcount;
   char outbuf[80];
   BT_INT RT_MASK=-1, SA_MASK=-1, TR=-1;

   status = BusTools_BC_ReadLastMessageBlock(cardnum,RT_MASK,SA_MASK,TR,&mcount,bcmessage);
   if(status)
      return status;

   for(msg_index=0; msg_index<mcount; msg_index++)
   {
      if(BusTools_BoardIsV6(cardnum))  //check for the F/w version to display time correctly
         BusTools_TimeGetFmtString(API_TTD_IRIG_NS, &bcmessage[msg_index].time_tag, outbuf);  // MSB = nanoseconds
      else
         BusTools_TimeGetFmtString(API_TTD_IRIG, &bcmessage[msg_index].time_tag, outbuf);     // MSB = microesconds 
      printf("BC**Message Time - %s\n",outbuf);        

      printf("%d - %x\n",bcmessage[msg_index].messno,bcmessage[msg_index].status);

      if (bcmessage[msg_index].mess_command1.tran_rec)
         printf("BC**RT->BC (%d)  ", bcmessage[msg_index].messno);
      else
         printf("BC->RT (%d)  ", bcmessage[msg_index].messno);
 
      printf("BC**RT-%d  \n",bcmessage[msg_index].mess_command1.rtaddr);
      printf("BC**SA-%d  \n",bcmessage[msg_index].mess_command1.subaddr);
      printf("BC**WC-%d  \n",bcmessage[msg_index].mess_command1.wcount);
      printf("BC**status-1 0x%04x\n",*((WORD*)(&bcmessage[msg_index].mess_status1)));         
      printf("BC**int_stat-0x%08lx\n",*((DWORD*)(&bcmessage[msg_index].status)));

      wcount = bcmessage[msg_index].mess_command1.wcount;
      if (wcount == 0)
         wcount = 32;           // Zero word count means 32 words.

      if (bcmessage[msg_index].control & BC_CONTROL_BUFFERA)
         bufnum = 0;            // Display data from first BC buffer
      else
         bufnum = 1;            // Display data from second BC buffer

      j = wcount;               // See if this msg shorted than previous

      for ( i = 0; i < j; i++ ) // Display all of the data words
      {
         // If this is an RT->BC message, and there is a Message Error
         //  or No Response, than xxxx the data (since the RT did not
         //    send any data words).
         if ( i >= wcount )
         {
            printf("    ");     // Data does not apply to this msg
         }
         else if (bcmessage[msg_index].mess_command1.tran_rec &&
           (bcmessage[msg_index].mess_status1.me || (bcmessage[msg_index].status&BT1553_INT_NO_RESP)))
         {
            printf("xxxx ");      // No data was transfered because of error
         }
         else
         {
            printf("BC**%04x ",bcmessage[msg_index].data[bufnum][i]);
         }
      }
      printf("\n_____________________________________________________________\n");
   }

   return API_SUCCESS;
}
