/*============================================================================*
 * FILE:                 E X A M P L E _ B C _ O P T I O N S . C
 *============================================================================*
 *
 *      COPYRIGHT (C) 1998-2016 BY ABACO SYSTEMS, INC. 
 *      ALL RIGHTS RESERVED.
 *
 *      THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND
 *      COPIED ONLY IN ACCORDANCE WITH THE TERMS OF SUCH LICENSE AND WITH
 *      THE INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY
 *      OTHER COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE
 *      AVAILABLE TO ANY OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF THE
 *      SOFTWARE IS HEREBY TRANSFERRED.
 *
 *      THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT
 *      NOTICE AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY ABACO SYSTEMS.
 *
 *===========================================================================*
 *
 * FUNCTION:    EXAMPLE PROGRAM
 * This is an example program that uses several Bus Controller options to 
 * configure a bus list.  The options used are Frame-start timing, Message 
 * Scheduling, retries and Interrupts. There are two callback functions used.  
 * One is for the specific BC message in the bus list excluding the sync mode 
 * code.  The other callback process the sync mode code.  Retries on no-response 
 * or busy are enable on several of the messages.  Frame start timing use the 
 * gap time as the message delay time from the start of the frame.  Message 
 * scheduling is used to setup the message in the different frames.  Some 
 * messages transact at 20Hz, some at 10Hz some at 2 Hz and the sync mode code 
 * runs at 1Hz.
 *===========================================================================*/

#include "busapi.h"
#include <stdio.h>

void setupTheBusController(int cardnum);
void setupThe_BC_IntFIFO(cardnum);
void setupThe_BC_syncFIFO(int cardnum);

API_INT_FIFO sIntFIFO1;
API_INT_FIFO sIntFIFO2;

void main(int argc, char **argv)
{
   BT_INT status;
   BT_INT cardnum;
   BT_UINT pwFlag, device = 0; // default to device 0
   char ichar;

   pwFlag = API_B_MODE | API_SW_INTERRUPT;

   if(argc == 2)
      device = atoi(argv[1]);
   
   /* Initializing Channel */
   status = BusTools_API_OpenChannel(&cardnum,pwFlag,device,CHANNEL_1);
   printf("BusTools_API_OpenChannel status = %d\n",status);;

   {
      float wcs,lpu;
      BT_INT build;

      status = BusTools_GetFWRevision(cardnum,&wcs,&lpu,&build);
      printf("WCS = %f LPU = %f BUILD = %i \n",wcs,lpu,build);
   }

   status = BusTools_SetInternalBus(cardnum, EXTERNAL_BUS); // Run externally for this example */
   printf("BusTools_SetInternalBus status = %d\n",status);

   // Initialize the time tag counter to current system time and use IRIG format to display time tags
   BusTools_TimeTagMode(cardnum, API_TTD_IRIG,API_TTI_IRIG,API_TTM_FREE,NULL,0,0,0); 
   printf("BusTools_TimeTagMode status = %d\n",status);

   setupTheBusController(cardnum);
   setupThe_BC_syncFIFO(cardnum);  // Setup the interrupt callback for the sync mode code

   printf("Hit enter to start the test\n");
   getc(stdin);

   status = BusTools_BC_StartStop(cardnum,BC_START);
   printf("BusTools_BC_StartStop status = %d\n",status);
 
   do{
       ichar = getchar();
       if(ichar=='d')
          BusTools_DumpMemory(cardnum,DUMP_ALL,"retry_dump.txt","dump on retry");
   }while(ichar !='q');

   status = BusTools_BC_StartStop(cardnum,BC_STOP);
   printf("BusTools_BC_StartStop status = %d\n",status);

   status = BusTools_RegisterFunction(cardnum, &sIntFIFO1, UNREGISTER_FUNCTION);
   printf("BusTools_RegisterFunction UNREGISTER status = %d\n",status);

   status = BusTools_RegisterFunction(cardnum, &sIntFIFO2, UNREGISTER_FUNCTION);
   printf("BusTools_RegisterFunction UNREGISTER status = %d\n",status);

   BusTools_API_Close(cardnum);
   printf("BusTools_API_Close status = %d\n",status);
}

void setupTheBusController(int cardnum)
{
   BT_INT status;
   BT_UINT messno;
   API_BC_MBUF bcmessage;
   BT_U16BIT multiple_retry[9];

   // Init BC with retry on NO-REPSONSE or BUSY.  Interrupt on a end-of-message
   // Use message schedulong and frame start timing.   
   status = BusTools_BC_Init(cardnum,MSG_SCHD | FRAME_START_TIMING, BT1553_INT_END_OF_MESS, 0,14,12,1000000,1); //BC_RETRY_NRSP | BC_RETRY_BUSY
   printf("BusTools_BC_Init status = %d\n",status);

   // Allocate ten Bus Controller message buffers (allocating more than needed is OK)
   status = BusTools_BC_MessageAlloc(cardnum,10);
   printf("BusTools_BC_MessageAlloc status = %d\n",status);

   messno=0;                                              
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Create as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // Use Buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_INTERRUPT;
   bcmessage.control |= BC_CONTROL_MFRAME_BEG;
   bcmessage.mess_command1.rtaddr   = 4;
   bcmessage.mess_command1.subaddr  = 3;
   bcmessage.mess_command1.wcount   = 5;
   bcmessage.mess_command1.tran_rec = 1;
   bcmessage.long_gap    = 0;     // 0 microsecond from frame start
   bcmessage.start_frame = 1;     // start in the first frame
   bcmessage.rep_rate    = 1;     // repeat every frame (20Hz)
 
   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage);
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   messno++;                                              
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Create as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // Use Buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_INTERRUPT;
   bcmessage.mess_command1.rtaddr   = 5;
   bcmessage.mess_command1.subaddr  = 22;
   bcmessage.mess_command1.wcount   = 8;
   bcmessage.mess_command1.tran_rec = 0;
   bcmessage.errorid     = 0;      // Default error injection buffer (no errors)
   bcmessage.long_gap    = 1000;   // 1000 microseconds (1 millisecond) from frame start.
   bcmessage.start_frame = 1;      // start in the first frame
   bcmessage.rep_rate    = 2;      // repeat every second frame (10Hz)

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   messno++;                                              
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Create as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // Use Buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on Bus B
   bcmessage.control |= BC_CONTROL_INTERRUPT;
   bcmessage.mess_command1.rtaddr   = 6;
   bcmessage.mess_command1.subaddr  = 23;
   bcmessage.mess_command1.wcount   = 6;
   bcmessage.mess_command1.tran_rec = 1;
   bcmessage.errorid     = 0;     // Default error injection buffer (no errors)
   bcmessage.long_gap    = 1000;  // 1000 microseconds (1 millisecond) from frame start
   bcmessage.start_frame = 2;     // Start in the second frame
   bcmessage.rep_rate    = 2;     // repeat every second frame (10Hz)

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   messno++;                                              
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Create as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // Use Buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on Bus B
   bcmessage.control |= BC_CONTROL_INTERRUPT;
   //bcmessage.control |= BC_CONTROL_RETRY;
   bcmessage.mess_command1.rtaddr   = 7;
   bcmessage.mess_command1.subaddr  = 7;
   bcmessage.mess_command1.wcount   = 7;
   bcmessage.mess_command1.tran_rec = 0;
   bcmessage.errorid     = 0;     // Default error injection buffer (no errors)
   bcmessage.long_gap   = 4000;  // 4000 microseconds (4 milliseconds) from frame start
   bcmessage.start_frame = 1;     // Start in the second frame
   bcmessage.rep_rate    = 10;    // repeat every 10th frame (2Hz)

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   messno++;                                              
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Create as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // Use Buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_INTERRUPT;
   //bcmessage.control |= BC_CONTROL_RETRY;
   bcmessage.mess_command1.rtaddr   = 8;
   bcmessage.mess_command1.subaddr  = 0;
   bcmessage.mess_command1.wcount   = 1;
   bcmessage.mess_command1.tran_rec = 1;
   bcmessage.errorid     = 0;        // Default error injection buffer (no errors)
   bcmessage.long_gap    = 5000;     // 5000 microsecond (5 millisceconds) from frame start
   bcmessage.start_frame = 1;        // Start in the first frame
   bcmessage.rep_rate    = 20;       // repeat 20th frame (1Hz)

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   messno++;                                              
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(0);     // Restart the frame   
   bcmessage.control = BC_CONTROL_NOP;         // Noop
   bcmessage.control |= BC_CONTROL_MFRAME_END; // end of frame marker.

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   /* This array sets three retries, switching between same bus and alternate bus. 
   multiple_retry[0] = RETRY_SAME_BUS;
   multiple_retry[1] = RETRY_ALTERNATE_BUS;
   multiple_retry[2] = RETRY_ALTERNATE_BUS;
   multiple_retry[3] = RETRY_END;

   status = BusTools_BC_RetryInit(cardnum, multiple_retry);
   printf("BusTools_BC_RetryInit status = %d\n",status);*/
}

/****************************************************************************
*
*  Function:  bc_intFunction
*
*  Description:
*     This routine is the user callback function invoke when the Board detects
*     a end-of-message condition. This function read the data in the RT message
*     and print message data to the screen.
*
****************************************************************************/
BT_INT  _stdcall bc_intFunction(BT_UINT cardnum, struct api_int_fifo *sIntFIFO)
{   
   API_BC_MBUF bcmessage;
   int       bufnum;
   static int sa_index;
   int       i,j;
   int       status;
   int       wcount;
   BT_INT tail;
   BT_UINT messno;
   char outbuf[80];

   tail = sIntFIFO->tail_index;
   while(tail != sIntFIFO->head_index)
   {
      // Get the message number for message in the interrupt queue
      messno = sIntFIFO->fifo[tail].bufferID;
	  // Read the message data
      status = BusTools_BC_MessageRead(cardnum,messno,&bcmessage);     
      if (status)
         return status;
      // Convert the time tag base on time base
      if(BusTools_BoardIsV6(cardnum)) // Nanosecond time base for V6 firmware
         BusTools_TimeGetFmtString(API_TTD_IRIG_NS, &bcmessage.time_tag, outbuf);
      else                            // Microseond time base for all other firmware
         BusTools_TimeGetFmtString(API_TTD_IRIG, &bcmessage.time_tag, outbuf);
      printf("\ntime tag = %s\n",outbuf);

	  // Check if this is a retry message
	  if(bcmessage.status & BT1553_INT_RETRY)
	     printf("Last Retry on Bus ");

	  // Determine the transaction Bus
	  if(bcmessage.status & BT1553_INT_BUSB)
         printf("B\n");
	  else
		 printf("A\n");

	  // Print out the message information
      if (bcmessage.mess_command1.tran_rec)
         printf("BC**RT->BC (%d)  \n", messno);
      else
         printf("BC->RT (%d)  \n", messno);
 
      printf("BC**RT-%d  \n",bcmessage.mess_command1.rtaddr);
      printf("BC**SA-%d  \n",bcmessage.mess_command1.subaddr);
      printf("BC**WC-%d  \n",bcmessage.mess_command1.wcount);
        
      printf("BC**status-1 0x%04x\n",*((WORD*)(&bcmessage.mess_status1)));
          
      printf("BC**int_stat-0x%08lx\n",*((DWORD*)(&bcmessage.status)));

      wcount = bcmessage.mess_command1.wcount;
      if (wcount == 0)
         wcount = 32;           // Zero word count means 32 words.

      j = wcount;               // See if this msg shorted than previous
    
      if (bcmessage.control & BC_CONTROL_BUFFERA)
         bufnum = 0;            // Display data from first BC buffer
      else
         bufnum = 1;            // Display data from second BC buffer

      for ( i = 0; i < j; i++ ) // Display all of the data words
      {
         // If this is an RT->BC message, and there is a Message Error
         //  or No Response, than xxxx the data (since the RT did not
         //    send any data words).
         if ( i >= wcount )
         {
            printf("    ");     // Data does not apply to this msg
         }
         else if ( bcmessage.mess_command1.tran_rec &&
           (bcmessage.mess_status1.me || (bcmessage.status&BT1553_INT_NO_RESP) ) )
         {
            printf("xxxx ");      // No data was transfered because of error
         }
         else
         {
            printf("BC**%04x ",bcmessage.data[bufnum][i]);
         }
      }
      printf("\n");

	  tail++;
      tail &= sIntFIFO->mask_index;
      sIntFIFO->tail_index = tail;
  }
            
   return 0;
}

/****************************************************************************
*
*  Function:  setupThe_BC_IntFIFO
*
*  Description:
*     This routine setup the interrupt FIFO with the value needed for a 
*     interrupt on a BC message.  The interrupt is set for only the end-of
*     message.  
*
*     Any errors detected are reported to the user.  The user
*     has the option of terminating the test by striking any key.
*
****************************************************************************/
void setupThe_BC_IntFIFO(int cardnum)
{
   int status;
   // Setup the FIFO structure for this board.
   memset(&sIntFIFO1, 0, sizeof(sIntFIFO1));
   sIntFIFO1.function       = bc_intFunction;
   sIntFIFO1.iPriority      = THREAD_PRIORITY_ABOVE_NORMAL;
   sIntFIFO1.dwMilliseconds = INFINITE;
   sIntFIFO1.FilterType     = EVENT_BC_MESSAGE;

   sIntFIFO1.FilterMask[4][1][3]  =  0xffffffff;  // Enable all Word Counts
   sIntFIFO1.FilterMask[5][0][22] =  0xffffffff;  // Enable all Word Counts
   sIntFIFO1.FilterMask[6][1][23] =  0xffffffff;  // Enable all Word Counts
   sIntFIFO1.FilterMask[7][0][7]  =  0xffffffff;  // Enable all Word Counts

   // Call the register function to register and start the thread.
   status = BusTools_RegisterFunction(cardnum, &sIntFIFO1, REGISTER_FUNCTION);
   printf("BusTools_RegisterFunction REGISTER status = %d\n",status);
}

/****************************************************************************
*
*  Function:  bc_intFunction
*
*  Description:
*     This routine is the user callback function invoke when the Board detects
*     a end-of-message condition. This function read the data in the RT message
*     and print message data to the screen.
*
****************************************************************************/
BT_INT  _stdcall sync_intFunction(BT_UINT cardnum, struct api_int_fifo *sIntFIFO)
{   
   API_BC_MBUF bcmessage;
   static int sa_index;
   int       status;
   BT_INT tail;
   BT_UINT messno;
   char outbuf[80];

   tail = sIntFIFO->tail_index;
   while(tail != sIntFIFO->head_index)
   {
      messno = sIntFIFO->fifo[tail].bufferID;

      printf("INT-STATUS = %x\n",sIntFIFO->fifo[tail].int_status);

      status = BusTools_BC_MessageRead(cardnum,messno,&bcmessage);     
      if (status)
         return status;

      if(BusTools_BoardIsV6(cardnum))
         BusTools_TimeGetFmtString(API_TTD_IRIG_NS, &bcmessage.time_tag, outbuf);
      else
         BusTools_TimeGetFmtString(API_TTD_IRIG, &bcmessage.time_tag, outbuf);

      printf("\ntime tag = %s\n",outbuf);

	  if(bcmessage.status & BT1553_INT_RETRY)
	     printf("Last Retry on Bus ");

	  if(bcmessage.status & BT1553_INT_BUSB)
         printf("B\n");
	  else
		 printf("A\n");

      printf("SYNCHRONIZE MODE CODE !!!!!!!!\n");

	  tail++;
      tail &= sIntFIFO->mask_index;
      sIntFIFO->tail_index = tail;
  }
            
   return 0;
}

/****************************************************************************
*
*  Function:  setupThe_BC_syncFIFO
*
*  Description:
*     This routine setup the interrupt FIFO with the value needed for a 
*     interrupt on a BC message.  The interrupt is set for only the end-of
*     message.  
*
*     Any errors detected are reported to the user.  The user
*     has the option of terminating the test by striking any key.
*
****************************************************************************/
void setupThe_BC_syncFIFO(int cardnum)
{
   int status;
   // Setup the FIFO structure for this board.
   memset(&sIntFIFO2, 0, sizeof(API_INT_FIFO));
   sIntFIFO2.function       = sync_intFunction;
   sIntFIFO2.iPriority      = THREAD_PRIORITY_ABOVE_NORMAL;
   sIntFIFO2.dwMilliseconds = INFINITE;
   sIntFIFO2.FilterType     =  EVENT_BC_MESSAGE;

   sIntFIFO2.FilterMask[8][1][0]  =  0x2;  // Enable sync mode code Bit 1 (0x2)

   // Call the register function to register and start the thread.
   status = BusTools_RegisterFunction(cardnum, &sIntFIFO2, REGISTER_FUNCTION);
   printf("BusTools_RegisterFunction REGISTER status = %d\n",status);
}
