/*============================================================================*
 * FILE:                 E X A M P L E _ B C _ R E T R Y . C
 *============================================================================*
 *
 *      COPYRIGHT (C) 1998-2016 BY ABACO SYSTEMS, INC. 
 *      ALL RIGHTS RESERVED.
 *
 *      THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND
 *      COPIED ONLY IN ACCORDANCE WITH THE TERMS OF SUCH LICENSE AND WITH
 *      THE INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY
 *      OTHER COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE
 *      AVAILABLE TO ANY OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF THE
 *      SOFTWARE IS HEREBY TRANSFERRED.
 *
 *      THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT
 *      NOTICE AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY ABACO SYSTEMS.
 *
 *===========================================================================*
 *
 * FUNCTION:    EXAMPLE PROGRAM
 * This example demonstrates enabling retries on Bus Controller messages.
 * Enabling retries lets the hardware automatically resend a message if the 
 * retry condition occurs. In the example the Bus Controller automatically 
 * resends the message if a no-response is detected on a retry enabled message 
 * or if the RT responds with Busy or Message Error.
 *
 * In order to enable retries you need to configure the Bus Controller to retry 
 * by setting the retry condition(s) in BusTools_BC_Init.  Once a condition is 
 * programmed, each Bus Controller message can then be set to retry. In this 
 * example retries are set for message 1 and 2 (0-based) in the bus list. 
 * Furthermore, up to eight retries can be programmed by calling 
 * BusTools_BC_RetryInit. That function allows programming up to eight retries 
 * on either the same or alternate bus. The same and alternate bus designations 
 * are from the initial bus setting of the message.  For example, if the message 
 * transacts Bus A the alternate bus is Bus B.  In addition to setting up retries 
 * the example also sets up a callback function on a retry.
 *
 *===========================================================================*/
#include <stdio.h>
#include "busapi.h"

void setupTheBusController(int cardnum);
void setupTheBusController2(int cardnum);
void initializeBCRetries(BT_INT cardnum);
void setupThe_BC_IntFIFO(cardnum);

API_INT_FIFO sIntFIFO1;

void main(int argc, char **argv)
{
   BT_INT status;
   BT_INT cardnum;
   BT_UINT pwFlag, device = 0; // default to device 0
   char ichar;

   pwFlag = API_B_MODE | API_SW_INTERRUPT;

   if(argc == 2)
   {
      device = atoi(argv[1]);
   }
   
   /* Initializing Channel */
   status = BusTools_API_OpenChannel(&cardnum,pwFlag,device,CHANNEL_1);
   printf("BusTools_API_OpenChannel status = %d\n",status);

   {
      float wcs,lpu;
      BT_INT build;

      status = BusTools_GetFWRevision(cardnum,&wcs,&lpu,&build);
      printf("WCS = %f LPU = %f BUILD = %i \n",wcs,lpu,build);
   }

   status = BusTools_SetInternalBus(cardnum, EXTERNAL_BUS); // External for this example
   printf("BusTools_SetInternalBus status = %d\n",status);

   setupTheBusController2(cardnum);
   initializeBCRetries(cardnum);
   setupThe_BC_IntFIFO(cardnum);  // Setup the interrupt callback for BC messages

   printf("Hit enter to start the test\n");
   getc(stdin);

   status = BusTools_BC_StartStop(cardnum,BC_START);
   printf("BusTools_BC_StartStop status = %d\n",status);
 
   do{
       ichar = getchar();
       if( ichar == 'd' || ichar == 'D')   // Dump option for debugging
          BusTools_DumpMemory(cardnum,DUMP_ALL,"retry_dump.txt","dump on retry");
   }while(ichar !='q');

   status = BusTools_BC_StartStop(cardnum,BC_STOP);
   printf("BusTools_BC_StartStop status = %d\n",status);

   status = BusTools_RegisterFunction(cardnum, &sIntFIFO1, UNREGISTER_FUNCTION);
   printf("BusTools_RegisterFunction UNREGISTER status = %d\n",status);

   BusTools_API_Close(cardnum);
   printf("BusTools_API_Close status = %d\n",status);
}

void initializeBCRetries(BT_INT cardnum)
{
   BT_U16BIT multiple_retry[9];
   BT_INT status;

   // This array sets three retries, switching between alternate and same bus.
   // This array allow you to program up to eight retries on the same or alternate bus
   multiple_retry[0] = RETRY_SAME_BUS;
   multiple_retry[1] = RETRY_ALTERNATE_BUS;
   multiple_retry[2] = RETRY_ALTERNATE_BUS;
   multiple_retry[3] = RETRY_END;

   status = BusTools_BC_RetryInit(cardnum, multiple_retry);
   printf("BusTools_BC_RetryInit status = %d\n",status);
}

void setupTheBusController(int cardnum)
{
   BT_INT status;
   BT_UINT messno, retry_conditions;
   API_BC_MBUF bcmessage;

   // Init BC with retry on NO-REPSONSE.  Interrupt on a retry
   retry_conditions = BC_RETRY_NRSP | BC_RETRY_ME;
   status = BusTools_BC_Init(cardnum,MSG_SCHD,BT1553_INT_RETRY,retry_conditions,14,12,1000000,1);
   printf("BusTools_BC_Init status = %d\n",status);

   status = BusTools_BC_MessageAlloc(cardnum,10);
   printf("BusTools_BC_MessageAlloc status = %d\n",status);

   messno=0;                                              
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Create a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // Use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_MFRAME_BEG;
   bcmessage.mess_command1.rtaddr   = 4;
   bcmessage.mess_command1.subaddr  = 3;
   bcmessage.mess_command1.wcount   = 5;
   bcmessage.mess_command1.tran_rec = 1;
   bcmessage.errorid     = 0;     // Default error injection buffer (no errors)
   bcmessage.gap_time    = 10;    // 10 microsecond
   bcmessage.start_frame = 1;     // start in the first frame
   bcmessage.rep_rate    = 1;     // repeat every frame
 
   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage);
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   messno++;                                              
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Create a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // Use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_INTERRUPT;
   bcmessage.control |= BC_CONTROL_RETRY;
   bcmessage.mess_command1.rtaddr   = 5;
   bcmessage.mess_command1.subaddr  = 22;
   bcmessage.mess_command1.wcount   = 8;
   bcmessage.mess_command1.tran_rec = 0;
   bcmessage.errorid     = 0;      // Default error injection buffer (no errors)
   bcmessage.gap_time    = 10;     // 10 microseconds.
   bcmessage.start_frame = 1;      // start in the first frame
   bcmessage.rep_rate    = 2;      // repeat every second (other) frame

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   messno++;                                              
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Create a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // Use buffer A
   bcmessage.control |= BC_CONTROL_BUSB;          // Transmit on Channel B
   bcmessage.control |= BC_CONTROL_INTERRUPT;
   bcmessage.control |= BC_CONTROL_RETRY;
   bcmessage.mess_command1.rtaddr   = 6;
   bcmessage.mess_command1.subaddr  = 23;
   bcmessage.mess_command1.wcount   = 6;
   bcmessage.mess_command1.tran_rec = 1;
   bcmessage.errorid     = 0;     // Default error injection buffer (no errors)
   bcmessage.gap_time    = 10;    // 10 microseconds
   bcmessage.start_frame = 2;     // Start in the second frame
   bcmessage.rep_rate    = 2;     // repeat every second (other) frame 

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   messno++;                                              
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(0);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Create a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // Use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_MFRAME_END;
   bcmessage.mess_command1.rtaddr   = 16;
   bcmessage.mess_command1.subaddr  = 1;
   bcmessage.mess_command1.wcount   = 8;
   bcmessage.mess_command1.tran_rec = 1;
   bcmessage.errorid     = 0;        // Default error injection buffer (no errors)
   bcmessage.gap_time    = 20;       // 20 microsecond from frame start
   bcmessage.start_frame = 1;        // Start in the first frame
   bcmessage.rep_rate    = 1;        // Repeat every frame

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);
}

void setupTheBusController2(int cardnum)
{
   BT_INT status;
   BT_UINT messno, retry_conditions;
   API_BC_MBUF bcmessage;

   // Init BC with retry on NO-REPSONSE.  Interrupt on a retry
   retry_conditions = BC_RETRY_NRSP | BC_RETRY_ME;
   status = BusTools_BC_Init(cardnum,REL_GAP,BT1553_INT_END_OF_MESS,retry_conditions,14,12,1000000,1);//BT1553_INT_END_OF_MESS
   printf("BusTools_BC_Init status = %d\n",status);

   status = BusTools_BC_MessageAlloc(cardnum,10);
   printf("BusTools_BC_MessageAlloc status = %d\n",status);

   messno=0;                                              
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Create a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // Use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_MFRAME_BEG;
   bcmessage.mess_command1.rtaddr   = 4;
   bcmessage.mess_command1.subaddr  = 3;
   bcmessage.mess_command1.wcount   = 5;
   bcmessage.mess_command1.tran_rec = 1;
   bcmessage.errorid     = 0;     // Default error injection buffer (no errors)
   bcmessage.gap_time    = 10;    // 10 microsecond
 
   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage);
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   messno++;                                              
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Create a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // Use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_INTERRUPT;
   bcmessage.control |= BC_CONTROL_RETRY;
   bcmessage.mess_command1.rtaddr   = 5;
   bcmessage.mess_command1.subaddr  = 22;
   bcmessage.mess_command1.wcount   = 8;
   bcmessage.mess_command1.tran_rec = 0;
   bcmessage.errorid     = 0;      // Default error injection buffer (no errors)
   bcmessage.gap_time    = 10;     // 10 microseconds.

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   messno++;                                              
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Create a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // Use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_MFRAME_END;
   bcmessage.mess_command1.rtaddr   = 16;
   bcmessage.mess_command1.subaddr  = 1;
   bcmessage.mess_command1.wcount   = 8;
   bcmessage.mess_command1.tran_rec = 1;
   bcmessage.errorid     = 0;        // Default error injection buffer (no errors)
   bcmessage.gap_time    = 20;       // 20 microsecond from frame start

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   //Second frame
   messno++;                                            
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Create a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // Use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_MFRAME_BEG;
   bcmessage.mess_command1.rtaddr   = 4;
   bcmessage.mess_command1.subaddr  = 3;
   bcmessage.mess_command1.wcount   = 5;
   bcmessage.mess_command1.tran_rec = 1;
   bcmessage.errorid     = 0;     // Default error injection buffer (no errors)
   bcmessage.gap_time    = 10;    // 10 microsecond

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage);
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   messno++;                                              
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Create a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // Use buffer A
   bcmessage.control |= BC_CONTROL_BUSB;          // Transmit on Channel B
   bcmessage.control |= BC_CONTROL_INTERRUPT;
   bcmessage.control |= BC_CONTROL_RETRY;
   bcmessage.mess_command1.rtaddr   = 6;
   bcmessage.mess_command1.subaddr  = 23;
   bcmessage.mess_command1.wcount   = 6;
   bcmessage.mess_command1.tran_rec = 1;
   bcmessage.errorid     = 0;     // Default error injection buffer (no errors)
   bcmessage.gap_time    = 10;    // 10 microseconds

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   messno++;                                              
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(0);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Create a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // Use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_MFRAME_END;
   bcmessage.mess_command1.rtaddr   = 16;
   bcmessage.mess_command1.subaddr  = 1;
   bcmessage.mess_command1.wcount   = 8;
   bcmessage.mess_command1.tran_rec = 1;
   bcmessage.errorid     = 0;        // Default error injection buffer (no errors)
   bcmessage.gap_time    = 20;       // 20 microsecond from frame start

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

}

/****************************************************************************
*
*  Function:  bc_intFunction
*
*  Description:
*     This routine is the user callback function invoke when the Board detects
*     a end-of-message condition. This function read the data in the RT message
*     and print message data to the screen.
*
****************************************************************************/
BT_INT  _stdcall bc_intFunction(BT_UINT cardnum, struct api_int_fifo *sIntFIFO)
{   
   API_BC_MBUF bcmessage;
   int       bufnum;
   static int sa_index;
   int       i,j;
   int       status;
   int       wcount;
   BT_INT tail;
   BT_UINT messno;
   char outbuf[80];

   tail = sIntFIFO->tail_index;
   while(tail != sIntFIFO->head_index)
   {
      messno = sIntFIFO->fifo[tail].bufferID;
      status = BusTools_BC_MessageRead(cardnum,messno,&bcmessage);     
      if (status)
         return status;
      
      if(BusTools_BoardIsV6(cardnum))
         BusTools_TimeGetFmtString(API_TTD_IRIG_NS, &bcmessage.time_tag, outbuf);
      else
         BusTools_TimeGetFmtString(API_TTD_IRIG, &bcmessage.time_tag, outbuf);

      printf("\ntime tag = %s\n",outbuf);

	  if(bcmessage.status & BT1553_INT_RETRY)
	     printf("Last Retry on Bus ");

	  if(bcmessage.status & BT1553_INT_CHANNEL)
         printf("B\n");
	  else
		 printf("A\n");

      if (bcmessage.mess_command1.tran_rec)
         printf("BC**RT->BC (%d)  \n", messno);
      else
         printf("BC->RT (%d)  \n", messno);
 
      printf("BC**RT-%d  \n",bcmessage.mess_command1.rtaddr);
      printf("BC**SA-%d  \n",bcmessage.mess_command1.subaddr);
      printf("BC**WC-%d  \n",bcmessage.mess_command1.wcount);
        
      printf("BC**status-1 0x%04x\n",*((WORD*)(&bcmessage.mess_status1)));
          
      printf("BC**int_stat-0x%08lx\n",*((DWORD*)(&bcmessage.status)));

      wcount = bcmessage.mess_command1.wcount;
      if (wcount == 0)
         wcount = 32;           // Zero word count means 32 words.

      j = wcount;               // See if this msg shorted than previous
    
      if (bcmessage.control & BC_CONTROL_BUFFERA)
         bufnum = 0;            // Display data from first BC buffer
      else
         bufnum = 1;            // Display data from second BC buffer

      for ( i = 0; i < j; i++ ) // Display all of the data words
      {
         // If this is an RT->BC message, and there is a Message Error
         //  or No Response, than xxxx the data (since the RT did not
         //    send any data words).
         if ( i >= wcount )
         {
            printf("    ");     // Data does not apply to this msg
         }
         else if ( bcmessage.mess_command1.tran_rec &&
           (bcmessage.mess_status1.me || (bcmessage.status&BT1553_INT_NO_RESP) ) )
         {
            printf("xxxx ");      // No data was transfered because of error
         }
         else
         {
            printf("BC**%04x ",bcmessage.data[bufnum][i]);
         }
      }
      printf("\n");

	  tail++;
      tail &= sIntFIFO->mask_index;
      sIntFIFO->tail_index = tail;
  }
            
   return 0;
}

/****************************************************************************
*
*  Function:  setupThe_BC_IntFIFO
*
*  Description:
*     This routine setup the interrupt FIFO with the value needed for a 
*     interrupt on a BC message.  The interrupt is set for only the end-of
*     message.  
*
*     Any errors detected are reported to the user.  The user
*     has the option of terminating the test by striking any key.
*
****************************************************************************/
void setupThe_BC_IntFIFO(int cardnum)
{
   int rt,tr,sa,status;
   // Setup the FIFO structure for this board.
   memset(&sIntFIFO1, 0, sizeof(sIntFIFO1));
   sIntFIFO1.iNotification = 0;
   sIntFIFO1.function     = bc_intFunction;
   sIntFIFO1.iPriority    = THREAD_PRIORITY_ABOVE_NORMAL;
   sIntFIFO1.dwMilliseconds = INFINITE;
   sIntFIFO1.bForceStartup  = 0;
   sIntFIFO1.FilterType     =  EVENT_BC_MESSAGE;

   for ( rt=0; rt < 32; rt++ )
      for (tr = 0; tr < 2; tr++ )
         for (sa = 0; sa < 32; sa++ )
            sIntFIFO1.FilterMask[rt][tr][sa] =  0xffffffff;  // Enable all Word Counts on all RT/SA/TR combos

   // Call the register function to register and start the thread.
   status = BusTools_RegisterFunction(cardnum, &sIntFIFO1, REGISTER_FUNCTION);
   printf("BusTools_RegisterFunction REGISTER status = %d\n",status);
}
