/*===========================================================================*
 * FILE:         E X A M P L E _ B C _ T R I G G E R _ U S E R. C
 *===========================================================================*
 *
 *      COPYRIGHT (C) 1998-2016 BY ABACO SYSTEMS, INC. 
 *      ALL RIGHTS RESERVED.
 *
 *      THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND
 *      COPIED ONLY IN ACCORDANCE WITH THE TERMS OF SUCH LICENSE AND WITH
 *      THE INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY
 *      OTHER COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE
 *      AVAILABLE TO ANY OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF THE
 *      SOFTWARE IS HEREBY TRANSFERRED.
 *
 *      THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT
 *      NOTICE AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY ABACO SYSTEMS.
 *
 *============================================================================*
 *
 * FUNCTION: 
 * This example application shows BC Triggering using the BC_TRIGGER_USER option. 
 * In this option the user controls how the BC frames run by adding BC_CONTROL_LAST 
 * to stop the Bus Controller.  Each time you stop the Bus Controller using 
 * BC_CONTROL_LAST a trigger input is needed to restart.  You can have one or more 
 * frames transacting off each trigger. This example creates six frames.  The first 
 *  four frames each start on a trigger input.  The last two frames combined.  The 
 * fifth frame starts on a trigger. The sixth frame runs after the normal frame 
 * delay, then stop when the BC_CONTROL_LAST buffer transacts.  The list loops back 
 * to the first frame requiring a trigger to start.
 * 
 * The application provides an option for internally generated triggers or an 
 * external user-supplied trigger.  If you select the internal trigger, you will 
 * need to connect discrete7 and 8 together.  Not all boards have these discrete 
 * channels available so check the board's configuration.
 *   
 *===========================================================================*/

/* $Revision:  8.12 Release $
   Date        Revision
  ----------   ---------------------------------------------------------------
  6/11/2001    initial release
*/

/*---------------------------------------------------------------------------*
 *                     INCLUDES, DEFINES and GLOBALS
 *---------------------------------------------------------------------------*/

#include <stdio.h>
#include "busapi.h"

//forward declarations
int BusController_Init(int,int);
void setupThe_BC_IntFIFO(int);
void setupTheTrigger(BT_INT cardnum);

API_INT_FIFO  sIntFIFO1;          // Thread FIFO structure for BC

//******************************************************************************
//*                                                                            *
//******************************************************************************
int main(int argc, char **argv)                     
{
   BT_UINT status, cardnum;
   BT_UINT mode=1;
   BT_INT device = 0;                     // Default device ID
   char c;

   if(argc == 2)
      device = atoi(argv[1]);             // Read command line device input

   mode = API_B_MODE | API_SW_INTERRUPT;  // 1553B protocol, use SW interrupts.
   status = BusTools_API_OpenChannel( &cardnum, mode, device, CHANNEL_1);
   printf("BusTools_API_OpenChannel BC cardnum=%d  status=%d (%s)\n", cardnum, status, BusTools_StatusGetString(status));

   // Set the Bus to External for BC
   status = BusTools_SetInternalBus(cardnum,EXTERNAL_BUS);  // Set up External bus
   printf("BusTools_SetInternalBus BC status = %d (%s)\n",status, BusTools_StatusGetString(status));

   // Set the time mode for both channels
   status = BusTools_TimeTagMode(cardnum, API_TTD_IRIG, API_TTI_IRIG, API_TTM_FREE, NULL, 0, 0, 0 );
   printf("BusTools_TimeTagMode BC status = %d (%s)\n",status, BusTools_StatusGetString(status));

   // Set up the BC for Trigger-User mode
   status = BusController_Init(cardnum,0);

   // Setting up a callback function to show the BC data
   setupThe_BC_IntFIFO(cardnum); //Set up BC interrupts using BusTools_RegisterFunction

   printf("Do you want to generate the trigger input from on the board? Y/N: ");
   c=getchar();
   fflush(stdin);

   if(c=='y' || c=='Y')
   {
      // Set up and internally wrapped trigger controlled by the user
      setupTheTrigger(cardnum);
      printf("\nUsing discrete 7 and 8 for triggering.  You must connect these two discretes\n");
      printf("Note:  Not all boards have these discretes available.  Check you board's configuration\n\n"); 
   }

   printf("Hit Enter to start.\nHit t for internal trigger\nHit d to dump board's memory\nHit q to stop\n");
   getchar();
   fflush(stdin);

   // Start the BC. Wait until trigger input to start each user-defined set of frames.
   status = BusTools_BC_StartStop(cardnum,BC_START);
   printf("BusTools_BC_StartStop status = %d\n",status);

   printf("...waiting for TRIGGER...\n");
   {
      char ichar;
      do
      {
         ichar = getchar();
         switch(ichar)
         {
            case 't':
            case 'T':
               BusTools_ExtTriggerOut(cardnum,100);
               break;
            case 'd':
            case 'D':
               BusTools_DumpMemory(cardnum,DUMP_ALL,"bc_trigger_dump.txt","exmaple_bc_trigger-user Dumping Memory");
               break;
         }
      }while (ichar != 'q' && ichar != 'Q');
   }

   printf("main: closing API...\n");

   // Stop the bus controller
   status = BusTools_BC_StartStop(cardnum,BC_STOP); 
   printf("BusTools_BC_StartStop status = %d\n",status);

   // Unregister callback function
   status = BusTools_RegisterFunction(cardnum,&sIntFIFO1,UNREGISTER_FUNCTION);
   printf("unregistering via BusTools_RegisterFunction status = %d\n",status);

   // Close the channel
   status = BusTools_API_Close(cardnum);
   printf("BusTools_API_Close BC status = %d\n",status);
}

void setupTheTrigger(BT_INT cardnum)
{
   BT_INT status;

   if(BusTools_BoardIsV6(cardnum))
   {
      status = BusTools_SetV6TrigOut(cardnum, DISCRETE, 7);            //Enable Trigger out on Discrete 7
      printf("BusTools_DiscreteSetIO status = %d\n",status);

      status = BusTools_SetV6TrigIn(cardnum, DISCRETE, 8);             //Enable Trigger in on Discrete 8
      printf("BusTools_DiscreteSetIO status = %d\n",status);

      status = BusTools_DiscreteWrite(cardnum,7,0);                    //Clear Discrete 7
      printf("BusTools_DiscreteSetIO status = %d\n",status);
   }
   else
   {
      status = BusTools_DiscreteSetIO(cardnum,0x155,0x3ff);
      printf("BusTools_DiscreteSetIO status = %d\n",status);

      status = BusTools_DiscreteTriggerOut(cardnum,TRIGGER_OUT_DIS_7);
      printf("BusTools_DiscreteTriggerOut status = %d\n",status);

      status = BusTools_DiscreteTriggerIn(cardnum,TRIGGER_IN_DIS_8);
      printf("BusTools_DiscreteTriggerOut status = %d\n",status);
   }

}

//******************************************************************************
//* Initialize the Bus Controller and setup the messages in the frames         *
//******************************************************************************
int BusController_Init(int cardnum,int sa_index)
{
   BT_INT status, messno;
   API_BC_MBUF bcmessage;

   // Initialize the BC. 
   // Frame time does not come into affect since the frame is stop on competion
   status = BusTools_BC_Init( cardnum, 0, BT1553_INT_END_OF_MESS, 0, 14, 12, 1000000, 1 );
   printf("BusTools_BC_Init status = %d\n",status);

   // Set bus control trigger mode. Each block of frame is started by a trigger.
   // BC_CONTROL_LAST marks the end of each block of frames that run soff a trigger input
   status = BusTools_BC_Trigger( cardnum, BC_TRIGGER_USER );
   printf("BusTools_BC_Trigger status = %d\n",status);

   // Allocate enough BC Buffer
   status = BusTools_BC_MessageAlloc(cardnum,50);
   printf("BusTools_BC_MessageAlloc status = %d\n",status);

   // Create messages for minor frame 1
   messno = 0; // Msg 0
   memset((char*)&bcmessage,0,sizeof(bcmessage));    
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Show as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // Use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_INTERRUPT;
   bcmessage.control |= BC_CONTROL_MFRAME_BEG;
   bcmessage.mess_command1.rtaddr   = 1; 
   bcmessage.mess_command1.subaddr  = 1;
   bcmessage.mess_command1.wcount   = 1;
   bcmessage.mess_command1.tran_rec = 0;
   bcmessage.errorid = 0;                         // Default error injection buffer (no errors)
   bcmessage.gap_time = 100;                      // Inter-message gap in microseconds.
   bcmessage.data[0][0] = 1;
   bcmessage.data[0][1] = 1;

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage);
   printf("BusTools_BC_MessageWrite for messno=%d  status = %d  next=%d\n",messno, status, bcmessage.messno_next);

   messno++;  // Msg 1
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Show as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // Use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_INTERRUPT;
   bcmessage.control |= BC_CONTROL_MFRAME_END;   // End-frame
   bcmessage.mess_command1.rtaddr   = 2;
   bcmessage.mess_command1.subaddr  = 2;
   bcmessage.mess_command1.wcount   = 2;
   bcmessage.mess_command1.tran_rec = 0;
   bcmessage.errorid = 0;                        // Default error injection buffer (no errors)
   bcmessage.gap_time = 100;                     // Inter-message gap in microseconds.

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage);
   printf("BusTools_BC_MessageWrite for messno=%d  status = %d\n",messno, status);

   messno++; // Msg 2
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_LAST;          // stop the BC
   bcmessage.control |= BC_CONTROL_MFRAME_END;   // End-frame
   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite for messno=%d  status = %d\n",messno, status);

   //==========================================================================
   // Create messages for minor frame 2

   messno++;  // Msg 3
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Show as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // Use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_INTERRUPT;
   bcmessage.control |= BC_CONTROL_MFRAME_BEG;
   bcmessage.mess_command1.rtaddr   = 10;
   bcmessage.mess_command1.tran_rec = 0;
   bcmessage.mess_command1.subaddr  = 10;
   bcmessage.mess_command1.wcount   = 10;
   bcmessage.errorid = 0;                         // Default error injection buffer (no errors)
   bcmessage.gap_time = 100;                      // Inter-message gap in microseconds.

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite for messno=%d  status = %d  next=%d\n",messno, status, bcmessage.messno_next);

   messno++;  // Msg 4
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Show as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // Use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_INTERRUPT;
   bcmessage.mess_command1.rtaddr   = 11;
   bcmessage.mess_command1.tran_rec = 1;
   bcmessage.mess_command1.subaddr  = 11;
   bcmessage.mess_command1.wcount   = 11;
   bcmessage.errorid = 0;                         // Default error injection buffer (no errors)
   bcmessage.gap_time = 100;                      // Inter-message gap in microseconds.

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite for messno=%d  status = %d  next=%d\n",messno, status, bcmessage.messno_next);

   messno++; // Msg 5
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_LAST;  // stop the BC
   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite for messno=%d  status = %d\n",messno, status);


   //==========================================================================
   // Create messages for minor frame 3

   messno++;  // Msg 6
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Show as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // Use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_INTERRUPT;
   bcmessage.control |= BC_CONTROL_MFRAME_BEG;
   bcmessage.mess_command1.rtaddr   = 20;
   bcmessage.mess_command1.tran_rec = 1;
   bcmessage.mess_command1.subaddr  = 20;
   bcmessage.mess_command1.wcount   = 20;
   bcmessage.errorid = 0;                         // Default error injection buffer (no errors)
   bcmessage.gap_time = 100;                      // Inter-message gap in microseconds.

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite for messno=%d  status = %d  next=%d\n",messno, status, bcmessage.messno_next);

   messno++;  // Msg 7
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Show as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // Use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_INTERRUPT;
   bcmessage.mess_command1.rtaddr   = 21;
   bcmessage.mess_command1.tran_rec = 0;
   bcmessage.mess_command1.subaddr  = 21;
   bcmessage.mess_command1.wcount   = 21;
   bcmessage.errorid = 0;                         // Default error injection buffer (no errors)
   bcmessage.gap_time = 100;                      // Inter-message gap in microseconds.

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite for messno=%d  status = %d  next=%d\n",messno, status, bcmessage.messno_next);

   messno++; // Msg 8
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_LAST;  // stop the BC
   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite for messno=%d  status = %d\n",messno, status);

   //==========================================================================
   // Ccreate messages for minor frame 4
   messno++;  // Msg 9
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Show as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // Use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_INTERRUPT;
   bcmessage.control |= BC_CONTROL_MFRAME_BEG;
   bcmessage.mess_command1.rtaddr   = 29;
   bcmessage.mess_command1.tran_rec = 0;
   bcmessage.mess_command1.subaddr  = 29;
   bcmessage.mess_command1.wcount   = 29;
   bcmessage.errorid = 0;                         // Default error injection buffer (no errors)
   bcmessage.gap_time = 100;                      // Inter-message gap in microseconds.

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite for messno=%d  status = %d  next=%d\n",messno, status, bcmessage.messno_next);

   messno++;  // Msg 10
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Show as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // Use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_INTERRUPT;
   bcmessage.mess_command1.rtaddr   = 30;
   bcmessage.mess_command1.tran_rec = 0;
   bcmessage.mess_command1.subaddr  = 30;
   bcmessage.mess_command1.wcount   = 30;
   bcmessage.errorid = 0;                        // Default error injection buffer (no errors)
   bcmessage.gap_time = 1000;                    // Inter-message gap in microseconds.

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite for messno=%d  status = %d  next=%d\n",messno, status, bcmessage.messno_next);

   messno++; // Msg 3
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1); 
   bcmessage.control = BC_CONTROL_LAST;             // stop the BC
   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite for messno=%d  status = %d\n",messno, status);

   //==========================================================================
   // Create messages for minor frame 5
   messno++;  // msg 11
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Show as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // Use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_INTERRUPT;
   bcmessage.control |= BC_CONTROL_MFRAME_BEG;
   bcmessage.mess_command1.rtaddr   = 15;
   bcmessage.mess_command1.tran_rec = 0;
   bcmessage.mess_command1.subaddr  = 15;
   bcmessage.mess_command1.wcount   = 15;
   bcmessage.errorid = 0;                        // Default error injection buffer (no errors)
   bcmessage.gap_time = 100;                     // Inter-message gap in microseconds.

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite for messno=%d  status = %d  next=%d\n",messno, status, bcmessage.messno_next);

   messno++;  // Msg 12
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Show as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // Use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_INTERRUPT;
   bcmessage.control |= BC_CONTROL_MFRAME_END;
   bcmessage.mess_command1.rtaddr   = 16;
   bcmessage.mess_command1.tran_rec = 0;
   bcmessage.mess_command1.subaddr  = 16;
   bcmessage.mess_command1.wcount   = 16;
   bcmessage.errorid = 0;                         // Default error injection buffer (no errors)
   bcmessage.gap_time = 100;                      // Inter-message gap in microseconds.

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite for messno=%d  status = %d  next=%d\n",messno, status, bcmessage.messno_next);

   //==========================================================================
   // Create messages for minor frame 6
   messno++;  //10
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Show as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // Use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_INTERRUPT;
   bcmessage.control |= BC_CONTROL_MFRAME_BEG;
   bcmessage.mess_command1.rtaddr   = 25;
   bcmessage.mess_command1.tran_rec = 0;
   bcmessage.mess_command1.subaddr  = 25;
   bcmessage.mess_command1.wcount   = 25;
   bcmessage.errorid = 0;                        // Default error injection buffer (no errors)
   bcmessage.gap_time = 100;                     // Inter-message gap in microseconds.

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite for messno=%d  status = %d  next=%d\n",messno, status, bcmessage.messno_next);

   messno++;  // Msg 13
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Show as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // Use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on Bus A
   bcmessage.control |= BC_CONTROL_INTERRUPT;
   bcmessage.mess_command1.rtaddr   = 26;
   bcmessage.mess_command1.tran_rec = 0;
   bcmessage.mess_command1.subaddr  = 26;
   bcmessage.mess_command1.wcount   = 26;
   bcmessage.errorid = 0;     // Default error injection buffer (no errors)
   bcmessage.gap_time = 100;    // Inter-message gap in microseconds.

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite for messno=%d  status = %d  next=%d\n",messno, status, bcmessage.messno_next);

   messno++; // Msg 14
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(0);
   bcmessage.control = BC_CONTROL_LAST;  // stop the BC
   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite for messno=%d  status = %d\n",messno, status);

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite for messno=%d  status = %d\n",messno, status);

   return status;
} 

/****************************************************************************
*
*  Function:  bc_intFunction
*
*  Description:
*     This routine is the user callback function invoke when the Board detects
*     a end-of-message condition. This function read the data in the BC message
*     and print message data to the screen.
*
****************************************************************************/
BT_INT  _stdcall bc_intFunction(BT_UINT cardnum, struct api_int_fifo *sIntFIFO)
{   
   API_BC_MBUF bcmessage;
   static int sa_index;
   int       i,j;
   int       status;
   int       wcount;
   BT_INT tail;
   BT_UINT messno;
   static int nFlag = 1;

   char outbuf[80];

   tail = sIntFIFO->tail_index;
   while(tail != sIntFIFO->head_index)
   {
      messno = sIntFIFO->fifo[tail].bufferID;
      status = BusTools_BC_MessageRead(cardnum,messno,&bcmessage);    
      if (status)
      {
         printf("JUNK!!!!\n");
         return status;
      }

      printf("%d - %x\n",messno,bcmessage.status);

      if(BusTools_BoardIsV6(cardnum))
         BusTools_TimeGetFmtString(API_TTD_IRIG_NS,&bcmessage.time_tag, outbuf);
      else
         BusTools_TimeGetFmtString(API_TTD_IRIG,&bcmessage.time_tag, outbuf);

      printf("tag time = %s\n",outbuf);
       
      if (bcmessage.mess_command1.tran_rec)
         printf("BC**RT->BC (%d)  ", messno);
      else
         printf("BC->RT (%d)  ", messno);
 
      printf("BC**RT-%d  \n",bcmessage.mess_command1.rtaddr);
      printf("BC**SA-%d  \n",bcmessage.mess_command1.subaddr);
      printf("BC**WC-%d  \n",bcmessage.mess_command1.wcount);
      printf("BC**status-1 0x%04x\n",*((WORD*)(&bcmessage.mess_status1)));
      printf("BC**int_stat-0x%08lx\n",*((DWORD*)(&bcmessage.status)));
      wcount = bcmessage.mess_command1.wcount;
      if (wcount == 0)
         wcount = 32;           // Zero word count means 32 words.

      j = wcount;               // See if this msg shorted than previous
    
      for ( i = 0; i < j; i++ ) // Display all of the data words
      {
         if ( i >= wcount )
         {
            printf("    ");     // Data does not apply to this msg
         }
         printf("BC**%04x ",bcmessage.data[0][i]);
      }
      printf("\n");
               
      tail++;
      tail &= sIntFIFO->mask_index;
      sIntFIFO->tail_index = tail;
   }

   return 0;
} 

/****************************************************************************
*
*  Function:  setupThe_BC_IntFIFO
*
*  Description:
*     This routine setup the interrupt FIFO with the value needed for a 
*     interrupt on a BC message.  The interrupt is set for only the end-of
*     message.  
****************************************************************************/
void setupThe_BC_IntFIFO(int cardnum)
{
   int rt,tr,sa,status;
   // Setup the FIFO structure for this board.
   memset(&sIntFIFO1, 0, sizeof(sIntFIFO1));
   sIntFIFO1.iNotification = 0;
   sIntFIFO1.function     = bc_intFunction;
   sIntFIFO1.iPriority    = THREAD_PRIORITY_ABOVE_NORMAL;
   sIntFIFO1.dwMilliseconds = INFINITE;
   sIntFIFO1.bForceStartup  = 0;
   sIntFIFO1.FilterType     =  EVENT_BC_MESSAGE;

   for ( rt=0; rt < 32; rt++ )
      for (tr = 0; tr < 2; tr++ )
         for (sa = 0; sa < 32; sa++ )
            sIntFIFO1.FilterMask[rt][tr][sa] =  0xffffffff;  // Enable all Word Counts on all RT/SA/TR combos

   // Call the register function to register and start the thread.
   status = BusTools_RegisterFunction(cardnum, &sIntFIFO1, REGISTER_FUNCTION);
   printf("BusTools_RegisterFunction status = %d\n",status);
}


