/*============================================================================*
 * FILE:                 E X A M P L E _ B M _ T R G O U T . C
 *============================================================================*
 *
 *      COPYRIGHT (C) 1998-2016 BY ABACO SYSTEMS, INC. 
 *      ALL RIGHTS RESERVED.
 *
 *      THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND
 *      COPIED ONLY IN ACCORDANCE WITH THE TERMS OF SUCH LICENSE AND WITH
 *      THE INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY
 *      OTHER COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE
 *      AVAILABLE TO ANY OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF THE
 *      SOFTWARE IS HEREBY TRANSFERRED.
 *
 *      THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT
 *      NOTICE AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY ABACO SYSTEMS.
 *
 *============================================================================*
 *
 * FUNCTION:  This example shows how to setup the Bus Monitor to generate 
 * an external trigger out when it records a specific message. In this example 
 * the Bus Monitor and optionally the BC and RT are configured. The Bus Monitor 
 * will generate an external trigger (trigger-out on discrete 7) when it records 
 * a BC message to RT 8. This example also routes the output trigger back to the 
 * input trigger (set up on discrete 8) and sets up an interrupt on external 
 * trigger.  This setup generates an output trigger and captures it as an input 
 * trigger for display.  In order for this set up to work you need to physically 
 * connect the discrete7 to discrete 8. Discrete and triggers vary between boards 
 * and this setup may not run on your board variant.  Please refer to the 
 * MIL-STD-1553 Hardware Installation Guide for details about triggers and 
 * discrete channels available on the different 1553 products.
 *===========================================================================*/

/* $Revision:  8.12 Release $
   Date        Revision
  ----------   ---------------------------------------------------------------
  6/11/2001    initial release
  12/18/2007   Update for BM trigger out/in example
*/

/*---------------------------------------------------------------------------*
 *                     INCLUDES, DEFINES and GLOBALS
 *---------------------------------------------------------------------------*/
#define DISCRETE_TRIGGER

#include <stdio.h>
#include <time.h>
#include "busapi.h"

int BusController_Init(int);
int BusMonitor_Init(int);
int RemoteTerminal_Init(int);
void setupThe_Trig_IntFIFO(int cardnum);

API_INT_FIFO  sIntFIFO;     // FIFO structure for Trigger

void main(int argc, char **argv)                     
{
   BT_INT status,cardnum,device;
   BT_INT bc_rt_running = 0;
   BT_INT loop_exit = 0;
   BT_UINT pwFlag;
   char c;

   pwFlag = API_B_MODE | API_SW_INTERRUPT; // Use software interrupts 

   if(argc==2)                             // command line entry for device
      device = atoi(argv[1]);
   else
      device = 0;                          // Default device

   printf("Starting Test.  Hit Enter to stop\n\n");
   printf("Initializing Test\n");

   status = BusTools_API_OpenChannel(&cardnum,pwFlag,device,CHANNEL_1);  // Initialize the channel
   printf("BusTools_API_Init status = %d\n",status);

   status = BusTools_SetInternalBus(cardnum,EXTERNAL_BUS);               // Set up external bus
   printf("BusTools_SetInternalBus status = %d\n",status);

   status = BusTools_ExtTrigIntEnable(cardnum, EXT_TRIG_ENABLE);         // Enable interrupt on external trigger
   printf("BusTools_ExtTrigIntEnable status = %d\n",status);
 
   status = BusMonitor_Init(cardnum);

   /* Optional setup for the Bus Controller and Remote Terminal */
   printf("Do you want to run the Bus Controller and Remote Terminal?  Y/N: ");
   c = getc(stdin);

   if(c == 'y' || c == 'Y')
   {
      status = BusController_Init(cardnum);
      status = RemoteTerminal_Init(cardnum);
      bc_rt_running = 1;
   }

   /* Set an interrupt callback function for the input trigger */
   setupThe_Trig_IntFIFO(cardnum);

   status = BusTools_BM_StartStop(cardnum,BM_START);
   printf("BusTools_BM_StartStop status = %d\n",status);

   if(bc_rt_running)
   {
      status = BusTools_RT_StartStop(cardnum,RT_START);
      printf("BusTools_RT_StartStop status = %d\n",status);

      status = BusTools_BC_StartStop(cardnum,BC_START);
      printf("BusTools_BC_StartStop status = %d\n",status);
   }

   printf("Enter q/Q to Quit\n      d/D for memory dump\n\n");
   do
   {
      c = getc(stdin);
      switch(c)
      {
         case 'D':
         case 'd':
            // Dump memory to a file.
            BusTools_DumpMemory(cardnum,0xffffff,"bm_trig_dump.txt","dumping");
            break;
         case 'q':
         case 'Q':
            loop_exit=1;
            break;
         default:
            continue;
      }
   }while(!loop_exit);

   printf("main: closing API\n");

   if(bc_rt_running)
   {
      status = BusTools_BC_StartStop(cardnum,BC_STOP);
      printf("BusTools_BC_StartStop status = %d\n",status);

      status = BusTools_RT_StartStop(cardnum,RT_STOP);
      printf("BusTools_RT_StartStop status = %d\n",status);
   }

   status = BusTools_BM_StartStop(cardnum,BM_STOP);
   printf("BusTools_BM_StartStop status = %d\n",status);

   status = BusTools_RegisterFunction(cardnum,&sIntFIFO,UNREGISTER_FUNCTION);
   printf("BusTools_RegisterFunction status = %d\n",status);

   status = BusTools_API_Close(cardnum);
   printf("BusTools_API_Close status = %d\n",status);
}

int BusController_Init(int cardnum)
{
   /* Setup a frame with 6 BC messages */
   BT_INT status, messno, i;
   API_BC_MBUF bcmessage;

   status = BusTools_BC_Init(cardnum,0,BT1553_INT_END_OF_MESS,0,16,14,1000000,1);
   printf("BusTools_BC_Init status = %d\n",status);

   status = BusTools_BC_MessageAlloc(cardnum,50);
   printf("BusTools_BC_MessageAlloc status = %d\n",status); 

   messno = 0;
   memset((char*)&bcmessage,0,sizeof(bcmessage));    
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // BC message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // Use buffer A
   bcmessage.control |= BC_CONTROL_CHANNELA;      // Transmit on Channel A
   bcmessage.control |= BC_CONTROL_MFRAME_BEG;
   bcmessage.mess_command1.rtaddr   = 8;
   bcmessage.mess_command1.subaddr  = 8;
   bcmessage.mess_command1.wcount   = 8;
   bcmessage.mess_command1.tran_rec = 0;
   bcmessage.errorid = 0;                        // Default error injection buffer (no errors)
   bcmessage.gap_time = 20;                      // 20 microsecond inter-message gap.
   for(i=0;i<8;i++)
      bcmessage.data[0][i] = 0xA000 + i;         // Add some data 

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage);
   printf("BusTools_BC_MessageWrite status = %d\n",status);
    
   messno++;                                              
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // show as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // use buffer A
   bcmessage.control |= BC_CONTROL_CHANNELA;      // Transmit on Channel A
   bcmessage.mess_command1.rtaddr   = 3;
   bcmessage.mess_command1.subaddr  = 3;
   bcmessage.mess_command1.wcount   = 3;
   bcmessage.mess_command1.tran_rec = 1;
   bcmessage.errorid = 0;                         // Default error injection buffer (no errors)
   bcmessage.gap_time = 20;                      // 20 microsecond inter-message gap.
 
   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage);
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   messno++;                                              
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // show as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // use buffer A
   bcmessage.control |= BC_CONTROL_CHANNELA;      // Transmit on Channel A
   bcmessage.mess_command1.rtaddr   = 4;
   bcmessage.mess_command1.subaddr  = 4;
   bcmessage.mess_command1.wcount   = 4;
   bcmessage.mess_command1.tran_rec = 1;
   bcmessage.errorid = 0;                        // Default error injection buffer (no errors)
   bcmessage.gap_time = 20;                      // 20 microsecond inter-message gap.

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   messno++;                                              
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // show as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // use buffer A
   bcmessage.control |= BC_CONTROL_CHANNELA;      // Transmit on Channel A
   bcmessage.mess_command1.rtaddr   = 5;
   bcmessage.mess_command1.subaddr  = 5;
   bcmessage.mess_command1.wcount   = 5;
   bcmessage.mess_command1.tran_rec = 1;
   bcmessage.errorid = 0;                        // Default error injection buffer (no errors)
   bcmessage.gap_time = 20;                      // 20 microsecond inter-message gap..

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   messno++;                                              
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // show as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // use buffer A
   bcmessage.control |= BC_CONTROL_CHANNELA;      // Transmit on Channel A
   bcmessage.mess_command1.rtaddr   = 3;
   bcmessage.mess_command1.subaddr  = 8;
   bcmessage.mess_command1.wcount   = 8;
   bcmessage.mess_command1.tran_rec = 1;
   bcmessage.errorid = 0;                        // Default error injection buffer (no errors)
   bcmessage.gap_time = 20;                      // 20 microsecond inter-message gap.

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   messno++;                                              
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(0);
   bcmessage.control = BC_CONTROL_MESSAGE;        // show as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // use buffer A
   bcmessage.control |= BC_CONTROL_CHANNELA;      // Transmit on Channel A
   bcmessage.control |= BC_CONTROL_MFRAME_END;
   bcmessage.mess_command1.rtaddr   = 16;
   bcmessage.mess_command1.subaddr  = 12;
   bcmessage.mess_command1.wcount   = 12;
   bcmessage.mess_command1.tran_rec = 1;
   bcmessage.errorid = 0;                         // Default error injection buffer (no errors)
   bcmessage.gap_time = 20;                       // 20 microsecond inter-message gap.

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   return status; 
} 

int RemoteTerminal_Init(int cardnum)
{
   int status;
   int RT_MBUF_COUNT, NUM_RT;
   API_RT_CBUF cbuf;       // RT Control Buffer Legal Word Count Mask (2 words)
   API_RT_MBUF_WRITE mbuf;
   API_RT_ABUF abuf;
   int         tr;
   int         subaddress;
   int i,j,ii;
   int rt_addr[] = {1,2,3,4,5,8,16,22};

   NUM_RT = 8;
   RT_MBUF_COUNT = 1;

   status = BusTools_RT_Init(cardnum,0);
   printf("BusTools_RT_Init status = %d\n",status);

   /**********************************************************************
   *  Output RT control buffers
   *   Enable or disable all word counts and mode codes as specified by
   *    the caller's arguements.
   **********************************************************************/

   cbuf.legal_wordcount = 0xffffffff;  // legailize all word counts
   for ( i = 0; i < NUM_RT; i++)
   {
      for ( tr = 0; tr < 2; tr++ )
      {
         for ( subaddress = 0; subaddress < 32; subaddress++ )
         {
            status = BusTools_RT_CbufWrite(cardnum,rt_addr[i],subaddress,tr,
			                            RT_MBUF_COUNT,&cbuf);
         }
      }
   }

   /*******************************************************************
   *  Fill in message buffers for receive and transmit sub-units.
   *******************************************************************/

   mbuf.enable = BT1553_INT_END_OF_MESS;   // Interrupt enables on Broadcast Message
   mbuf.error_inj_id = 0;

   // For all subaddresses...
   for (ii = 0; ii<NUM_RT; ii++)
   {  
      for (i=0; i<32; i++)
      {
         // Create data buffer data
         for (j=0; j<32; j++)
            mbuf.mess_data[j] = (BT_U16BIT)(0xF000 + i);
         // For each data buffer...
         for (j=0; j<RT_MBUF_COUNT; j++)
         {
            status = BusTools_RT_MessageWrite(cardnum,rt_addr[ii],i,0,j,&mbuf);
            status = BusTools_RT_MessageWrite(cardnum,rt_addr[ii],i,1,j,&mbuf);
         }
      }
   }

   /************************************************
   *  Setup RT address control block
   ************************************************/

   abuf.enable_a          = 1;    // Enable Bus A...Note that the API bits are
   abuf.enable_b          = 1;    // Enable Bus B... reversed from the HW bits.
   abuf.inhibit_term_flag = 0;
   abuf.status            = 0x0000;
   abuf.command           = 0;
   abuf.bit_word          = 0;

   for(i=0;i<NUM_RT;i++)
   {
      status = BusTools_RT_AbufWrite(cardnum,rt_addr[i],&abuf);
      printf("BusTools_RT_AbufWrite status = %d\n",status);
   }
   
   return status;
}

int BusMonitor_Init(int cardnum)
{
   int status;
   BT_UINT actual;
   API_BM_TBUF trig_buf;

   status = BusTools_BM_Init(cardnum,1,1);
   printf("BusTools_BM_Init status = %d\n",status);

   status = BusTools_BM_MessageAlloc(cardnum, 300, &actual, BT1553_INT_END_OF_MESS); //interrupt on END_OF_MESS
   printf("BusTools_BM_MessageAlloc status = %d - actual =%d\n",status,actual);

   /* Set the trigger to output on a certain message status *
   ***********************************************************************
   *  BM Trigger Buffer definition
   *
   *     3 - Start Trigger Events A - B - C
   *     3 - Stop Trigger Events  E - F - G
   *
   *     control: dialog box entries for order of events:
   *                         START SIDE                  STOP SIDE
   *                    0 -> unconditionally             never
   *                    1 -> if A                        if E
   *                    2 -> if A AND B                  if E AND F
   *                    3 -> if A OR B                   if E OR F
   *                    4 -> if A with B (same message)  if E with F (same message) 
   *                    5 -> if B armed by A             if F armed by E
   *                    6 -> if A with B and C           if E with F and G
   *                    7 -> if A with B or C            if E with F or G
   *                    8 -> if C armed by A with B      if G armed by E with F
   *                    9 -> if A with B armed by C      if E with F armed by G
   *                   10 -> if A and B and C            if E and F and G
   *                   11 -> if A or B or C              if E or E or G
   *     type:    trigger type:
   *                    0 -> none
   *                    1 -> command
   *                    2 -> status
   *                    3 -> data
   *                    4 -> Not used
   *                    5 -> interrupt status lsb
   *                    6 -> interrupt status msb
   *
   *     mask:    16 bit mask (applied prior to compare)
   *     value:   16 bit value to compare
   *     word:    word number (data trigger - 3) or offset (user offset trigger - 5)
   *     count:   number of times event must occur before event_occured bit is set
   *
   **********************************************************************/
 
   memset(&trig_buf,0,sizeof(API_BM_TBUF));            
   trig_buf.trig_ext_output = 2; // Repetitive external output on trigger.
   trig_buf.control1 = 1;        // Trigger on Event A below

   // START Event A:
   trig_buf.capture[0].type  = 1;       // Type = 1, command word
   trig_buf.capture[0].mask  = 0xf800;  // Mask to only look at RT address in command word
   trig_buf.capture[0].value = 0x4000;  // Look for RT address 8
   trig_buf.capture[0].count = 1;       // Count of 1

   status = BusTools_BM_TriggerWrite(cardnum,&trig_buf);
   printf("BusTools_BM_TriggerWrite status = %d\n",status);

   /* Note: Discrete and triggers vary between board types and board options. You should  *
    * check the trigger and discrete options on your board before running this example.   *
    * you may need to change the discrete channels.
    * */
 
   if(BusTools_BoardIsV6(cardnum))
   {
      status = BusTools_SetV6TrigOut(cardnum, DISCRETE, 7);            //Enable Trigger out on Discrete 7
      printf("BusTools_DiscreteSetIO status = %d\n",status);

      status = BusTools_SetV6TrigIn(cardnum, DISCRETE, 8);             //Enable Trigger in on Discrete 8
      printf("BusTools_DiscreteSetIO status = %d\n",status);

      status = BusTools_DiscreteWrite(cardnum,7,0);                    //Clear Discrete 7
      printf("BusTools_DiscreteSetIO status = %d\n",status);

   }
   else
   {
      status = BusTools_DiscreteSetIO(cardnum,0x155,0x3ff);
      printf("BusTools_DiscreteSetIO status = %d\n",status);

      status = BusTools_DiscreteTriggerOut(cardnum,TRIGGER_OUT_DIS_7);
      printf("BusTools_DiscreteTriggerOut status = %d\n",status);

      status = BusTools_DiscreteTriggerIn(cardnum,TRIGGER_IN_DIS_8);
      printf("BusTools_DiscreteTriggerOut status = %d\n",status);
   }

   return status;
}

BT_INT _stdcall trigIntFunction(BT_UINT cardnum, struct api_int_fifo *sIntFIFO)
{
   BT_INT tail;

   tail = sIntFIFO->tail_index;
   while(tail != sIntFIFO->head_index)
   {
      printf("----------------> EXT Trigger interrupt\n");	

      tail++;
      tail &= sIntFIFO->mask_index;
      sIntFIFO->tail_index = tail;
   }
   return 0;
}

/****************************************************************************
*
*  Function:  setupThe_Trig_IntFIFO 
*
*  Description:
*     This routine setup the interrupt FIFO with the value needed for a 
*     interrupt on a BM message.  The interrupt is set for only the end-of
*     message.  
*
****************************************************************************/
void setupThe_Trig_IntFIFO(int cardnum)
{
   BT_INT status;

   // Setup the FIFO structure for trigger interrupt event.
   memset(&sIntFIFO, 0, sizeof(API_INT_FIFO));
   sIntFIFO.function       = trigIntFunction; // This is the interrupt function the API calls
   sIntFIFO.iPriority      = THREAD_PRIORITY_ABOVE_NORMAL;
   sIntFIFO.dwMilliseconds = INFINITE;
   sIntFIFO.iNotification  = 0;
   sIntFIFO.FilterType     = EVENT_EXT_TRIG;  //Trigger Event
 
   // Call the register function to register and start the thread.
   status = BusTools_RegisterFunction(cardnum, &sIntFIFO, REGISTER_FUNCTION);
   printf("BusTools_RegisterFunction status = %d\n",status);
}
