/*================================================================================*
 * FILE:         E X A M P L E _ R T _ B U F F E R _ S W I T C H . C
 *================================================================================*
 *
 *      COPYRIGHT (C) 1998-2016 BY ABACO SYSTEMS, INC. 
 *      ALL RIGHTS RESERVED.
 *
 *      THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND
 *      COPIED ONLY IN ACCORDANCE WITH THE TERMS OF SUCH LICENSE AND WITH
 *      THE INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY
 *      OTHER COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE
 *      AVAILABLE TO ANY OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF THE
 *      SOFTWARE IS HEREBY TRANSFERRED.
 *
 *      THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT
 *      NOTICE AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY ABACO SYSTEMS.
 *
 *=================================================================================*
 *
 * FUNCTION:    Demonstrates how to setup the Remote Terminal to manually switch 
 *              between two RT message buffers.  The RT is initialized
 *              with two message buffer per RT/SA/TX/RX combination.  Then,
 *              using BusTools_RT_MessageGetaddr and BusTools_MemoryWrite2 the 
 *              RT message buffer linked list is altered to have each buffer point 
 *              to its address instead of the next buffer in the list. Instead of 
 *              looping through the message buffer the RT uses only a single buffer.  
 *              The user must write the address of the message they want to run into
 *              the RT control Buffer.
 *          
 *              This example sets up a RT3 SA3 TX with 2 buffers.  Then manually
 *              changes the message buffer linking so only a single buffer
 *              is used.  By entering 0 or 1 at the command prompt the user can
 *              switch between buffer 0 (data = 0x1111) and buffer 1 (data = 0x2222)
 *
 *              This example shows this process for both the V6 and V4/5 firmware 
 *              designs.  The high level API hides the underlying design differences
 *              in the firmware.  When directly accessing memory the user
 *              needs to understand the memory layout for the firmware they are using.
 *              Refer to the "MIL-STD-1553 Universal Core Architecture Reference Manual" 
 *              for the V4/5 firmware and the "MIL-STD-1553 Enhanced Universal Core 
 *              Architecture (UCA32) Local Processing Unit (LPU) Reference Manual 
 *              for V6 firmware.  Also keep in mind that V6 firmware uses 32-bit
 *              addressing while V4/5 uses 16-bit addressing.
 *
 *              When running this example it you a dump the board's memory by typing
 *              'd' or 'D'.  Reviewing the memory dump will help follow the memory 
 *              manipulations in this example.  
 *===================================================================================*/

/* $Revision:  8.12 Release $
   Date        Revision
  ----------   ---------------------------------------------------------------
  6/11/2001    initial release
*/

/*---------------------------------------------------------------------------*
 *                     INCLUDES, DEFINES and GLOBALS
 *---------------------------------------------------------------------------*/
#include <stdio.h>
#include "busapi.h"

int BusController_Init(int,int);
int RemoteTerminal_Init(int);
BT_U32BIT	buff[2], tmpbuf;
BT_U32BIT rt3sa3tx_cbuf_addr, rt2sa2rx_cbuf_addr;
void setupThe_RT_IntFIFO(int cardnum);

API_INT_FIFO  rtIntFIFO;     // Thread FIFO structure for all RTs
static unsigned bm_tcnt=0,bc_tcnt=0,rt_tcnt=0;
BT_U32BIT rt_msg_ptr;

int main(int argc, char **argv)                     
{
   int status,cardnum;
   unsigned device,mode;
   int i=0;
   char ichar;

   mode = API_B_MODE | API_SW_INTERRUPT;     // MIL-STD-1553B with S/W interrupts enabled

   if(argc == 2)
      device = atoi(argv[1]);
   else
      device = 0;                            // Default device 0

   status = BusTools_API_OpenChannel(&cardnum,mode,device,CHANNEL_1);
   printf("BusTools_API_OpenChannel status = %d\n",status);

   status = BusTools_SetInternalBus(cardnum,EXTERNAL_BUS);
   printf("BusTools_SetInternalBus status = %d\n",status);

   status = BusController_Init(cardnum,50); // Set up the Bus Controller for this example
   status = RemoteTerminal_Init(cardnum);   // Set up the RT and manual alter the buffers  

   setupThe_RT_IntFIFO(cardnum);            // Set up interrupts on RT commands

   printf("Press Enter to start, 0 for Buffer 0; 1 for Buffer 1, and q to quit\n");
   getchar(); 

   status = BusTools_RT_StartStop(cardnum,RT_START);
   printf("BusTools_RT_StartStop status = %d\n",status);

   status = BusTools_BC_StartStop(cardnum,BC_START);
   printf("BusTools_BC_StartStop status = %d\n",status);

   do
   {
      ichar = getchar();

      switch(ichar)
      {
         case 'd':
         case 'D':
            BusTools_DumpMemory(cardnum,DUMP_ALL,"rt_buffer_switch.txt","Buffer Switch");
            break;
         case '0':
            // Write the selected message buffer address into the control buffer.
            if(BusTools_BoardIsV6(cardnum))
               status = BusTools_MemoryWrite2(cardnum,RAM32, rt3sa3tx_cbuf_addr,4,&buff[0]);
            else
               status = BusTools_MemoryWrite2(cardnum,RAM, rt3sa3tx_cbuf_addr,2,&buff[0]);
            break;
         case '1':
            // Write the selected message buffer address into the control buffer.
            if(BusTools_BoardIsV6(cardnum))
               status = BusTools_MemoryWrite2(cardnum,RAM32, rt3sa3tx_cbuf_addr,4,&buff[1]);
            else
               status = BusTools_MemoryWrite2(cardnum,RAM, rt3sa3tx_cbuf_addr,2,&buff[1]);         
      }
      fflush(stdin);
   }while(ichar != 'q' && ichar != 'Q');

   status = BusTools_BC_StartStop(cardnum,BC_STOP); 
   printf("BusTools_BC_StartStop status = %d\n",status);

   status = BusTools_RT_StartStop(cardnum,RT_STOP);
   printf("BusTools_RT_StartStop status = %d\n",status);

   status = BusTools_API_Close(cardnum);
   printf("BusTools_API_Close status = %d\n",status);
}

int BusController_Init(int cardnum,int num_buffers)
{
   int status, messno,i;
   API_BC_MBUF bcmessage;

   //initialize the BC with interrupt on end-of-message 16 us no response 14 us late response, 1sec frame (1000000 usec).     
   status = BusTools_BC_Init(cardnum,0,BT1553_INT_END_OF_MESS,0,16,14,1000000,1);
   printf("BusTools_BC_Init status = %d\n",status);

   status = BusTools_BC_MessageAlloc(cardnum,50);//num_buffers);
   printf("BusTools_BC_MessageAlloc status = %d\n",status);

   messno = 0;
   memset((char*)&bcmessage,0,sizeof(bcmessage));       
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Create as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // Use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on bus A
   bcmessage.control |= BC_CONTROL_MFRAME_BEG;    // Start of a minor frame
   bcmessage.mess_command1.rtaddr   = 2;          
   bcmessage.mess_command1.subaddr  = 2;
   bcmessage.mess_command1.wcount   = 3;
   bcmessage.mess_command1.tran_rec = 0;          // BC->RT
   bcmessage.errorid = 0;                         // Default error injection buffer (no errors)
   bcmessage.gap_time = 10;                       // 10 microsecond inter-message gap.

   for(i=0;i<3;i++)
       bcmessage.data[0][i] = 0xA000+i;

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage);
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   messno++;                                              
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)(messno + 1);
   bcmessage.control = BC_CONTROL_MESSAGE;        // Creat as a message
   bcmessage.control |= BC_CONTROL_BUFFERA;       // Use buffer A
   bcmessage.control |= BC_CONTROL_BUSA;          // Transmit on Bus A
   bcmessage.mess_command1.rtaddr   = 3;
   bcmessage.mess_command1.subaddr  = 3;
   bcmessage.mess_command1.wcount   = 3;
   bcmessage.mess_command1.tran_rec = 1;          // RT->BC
   bcmessage.errorid = 0;                         // Default error injection buffer (no errors)
   bcmessage.gap_time = 15;                       // 15 microsecond inter-message gap
 
   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage);
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   messno++;
   memset((char*)&bcmessage,0,sizeof(bcmessage));
   bcmessage.messno = messno;
   bcmessage.messno_next = (BT_U16BIT)0;          // Repeat starting at message 0
   bcmessage.control = BC_CONTROL_NOP;            // 
   bcmessage.control |= BC_CONTROL_MFRAME_END;    // End-of-Frame

   status = BusTools_BC_MessageWrite(cardnum,messno,&bcmessage); 
   printf("BusTools_BC_MessageWrite status = %d\n",status);

   return status; 
} 

int RemoteTerminal_Init(int cardnum)
{
   BT_INT status;
   BT_INT RT_MBUF_COUNT;
   API_RT_CBUF cbuf;       // RT Control Buffer Legal Word Count Mask (2 words)
   API_RT_MBUF_WRITE mbuf;
   API_RT_ABUF abuf;

   BT_INT i,j,jj;
   BT_INT rt_cnt = 2;
   BT_INT rt_addr[]    = {2,3};
   BT_INT rt_subaddr[] = {2,3};  
   BT_U32BIT fbstart,fbend;
   BT_U32BIT	msgaddr;
   RT_MBUF_COUNT = 3;

   status = BusTools_RT_Init(cardnum,0);
   printf("BusTools_RT_Init status = %d\n",status);

   /**********************************************************************
   *  Output RT control buffers
   *   Enable or disable all word counts and mode codes as specified by
   *    the caller's arguements.
   **********************************************************************/

   cbuf.legal_wordcount = 0xffffffff;  // Legailize all word counts

   status = BusTools_RT_CbufWrite(cardnum,2,2,0,1,&cbuf);

   status = BusTools_RT_CbufWrite(cardnum,3,3,1,RT_MBUF_COUNT,&cbuf);

   /*******************************************************************
   *  Fill in message buffers for receive and transmit sub-units.
   *******************************************************************/
   mbuf.error_inj_id = 0;   // No error injection
   mbuf.enable = BT1553_INT_END_OF_MESS;   // Interrupt enable at end-of-message

   status = BusTools_RT_MessageWrite(cardnum,rt_addr[0],rt_subaddr[0],RECEIVE,0,&mbuf);

   for (j=0; j<RT_MBUF_COUNT; j++)
   {   
      for(jj=0;jj<33;jj++)
          mbuf.mess_data[jj] = (BT_U16BIT)(0x1111 + 0x1111 * j); //fill in the TX Message data 0x1111 or 0x2222
      status = BusTools_RT_MessageWrite(cardnum,rt_addr[1],rt_subaddr[1],TRANSMIT,j,&mbuf);
    }

   /************************************************
   *  Setup RT address control block
   ************************************************/

   abuf.enable_a          = 1;      // Enable Bus A
   abuf.enable_b          = 1;      // Enable Bus B
   abuf.inhibit_term_flag = 0;
   abuf.status            = 0x0000; // filled in by the F/W
   abuf.command           = 0;
   abuf.bit_word          = 0;

   for(i=0;i<rt_cnt;i++)
   {
      status = BusTools_RT_AbufWrite(cardnum,rt_addr[i],&abuf);
      printf("BusTools_RT_AbufWrite status = %d\n",status);
   }

   /*******************************************************************
   *  Change the Message Next Pointer to point back back to itself.
   *******************************************************************/

   // Get the address of the RT SA3 Transmit command buffer 0
   status = BusTools_RT_MessageGetaddr(cardnum,rt_addr[1],rt_subaddr[1],TRANSMIT,0,&msgaddr);
   printf("BusTools_RT_MessageGetaddr status = %d\n",status);
   printf("RT3 SA3 TX Buffer 0 addr: %08x\n", msgaddr);  // Display the address of buffer 0

   if(BusTools_BoardIsV6(cardnum))
   {
      // For V6 firmware convert to a absolute RAM address and write that address into the
      // RT message buffer
      buff[0] = BusTools_RamAddr(cardnum,msgaddr);
      status = BusTools_MemoryWrite2(cardnum,RAM32,msgaddr,4,&buff[0]);
   }
   else
   {
      // For V4/5 firmware shift the address by 4 and write that to the message buffer
      buff[0] = msgaddr>>4; 
      status = BusTools_MemoryWrite2(cardnum,RAM,msgaddr,2,&(BT_U16BIT)buff[0]);
   }
///////////////////////////////////////////////////////////////////////////////////////////////////

   // Get the address of the RT SA3 Transmit command buffer 1
   status = BusTools_RT_MessageGetaddr(cardnum,rt_addr[1],rt_subaddr[1],TRANSMIT,1,&msgaddr);
   printf("BusTools_RT_MessageGetaddr status = %d\n",status);
   printf("RT3 SA3 TX Buffer 1 addr: %08x\n", msgaddr);

   if(BusTools_BoardIsV6(cardnum))
   {
      // For V6 firmware convert to a absolute RAM address and write that address into the
      // RT message buffer
      buff[1] = BusTools_RamAddr(cardnum,msgaddr);
      status = BusTools_MemoryWrite2(cardnum,RAM32,msgaddr,4,&buff[1]);
   }
   else
   {
      // For V4/5 firmware shift the address by 4 and write that to the message buffer
      buff[1] = msgaddr>>4; 
      status = BusTools_MemoryWrite2(cardnum,RAM,msgaddr,2,&(BT_U16BIT)buff[1]);
   }


   /***************************************************************************
   *  Now get the filter buffer address for RT3 SA3 TX. It contains the control
   *  buffer address that must be updated to switch RT buffers 
   ****************************************************************************/

   BusTools_GetAddr(cardnum,GETADDR_RTFILTER,&fbstart,&fbend);
   printf("filter buffer start = %x filter buffer end = %x\n",fbstart,fbend);

   if(BusTools_BoardIsV6(cardnum))
   {
      // Read the filter buffer to get cbuf message address
      // For V6 index by a DWORD
      //                                       RT            TX               SA 
      rt_msg_ptr = fbstart + ((BT_U32BIT)(rt_addr[1]<<8) + (TRANSMIT<<7) + (rt_subaddr[1]<<2));
      status = BusTools_MemoryRead2(cardnum,RAM32,rt_msg_ptr,4,&tmpbuf);
      // Convert the RAM address to a relative address
      rt3sa3tx_cbuf_addr = BusTools_RelAddr(cardnum,tmpbuf); 
   }
   else
   {
      // Read the filter buffer to get cbuf message address
      // For V4/5 index by WORD
      //                                        RT            TX               SA 
      rt_msg_ptr = fbstart + ((BT_U16BIT)(rt_addr[1]<<7) + (TRANSMIT<<6) + (rt_subaddr[1]<<1));
      status = BusTools_MemoryRead2(cardnum,RAM,rt_msg_ptr,2,&tmpbuf);
      // Convert the WORD address to a BYTE address and add the offset to the message buffer address
      rt3sa3tx_cbuf_addr = (tmpbuf*2)+4;
   }

   return status;
}

/****************************************************************************
*
*  Function:  rt_intFunction
*
*  Description:
*     This routine is the user callback function invoke when the Board detects
*     a end-of-message condition. This function read the data in the RT message
*     and print message data to the screen.
*
****************************************************************************/
BT_INT _stdcall rt_intFunction(BT_UINT cardnum, struct api_int_fifo *sIntFIFO)
{
   BT_INT tail,i;
   BT_UINT messno,rt_addr,sub_addr,trans;
   API_RT_MBUF_READ rt_mbuf;

   tail = sIntFIFO->tail_index;
   while(tail != sIntFIFO->head_index)
   {

      messno   = sIntFIFO->fifo[tail].bufferID;
      rt_addr  = sIntFIFO->fifo[tail].rtaddress;        // RT address
      trans    = sIntFIFO->fifo[tail].transrec;         // Transmit/Receive
      sub_addr = sIntFIFO->fifo[tail].subaddress;       // Subaddress number

      if(trans    = sIntFIFO->fifo[tail].transrec)
      {
         BusTools_RT_MessageRead(cardnum,rt_addr,sub_addr,trans,messno,&rt_mbuf);
         printf("RT = %d SA = %d T/R = %d buffer = %d\n",rt_addr,sub_addr,trans,messno);
         printf("status = %08x\n",rt_mbuf.status);
         for ( i = 0; i < sIntFIFO->fifo[tail].wordcount; i++ ) // Display all of the data words
         {  
            printf("RT**%04x ",rt_mbuf.mess_data[i]);
         }
         printf("\n");
      }

      tail++;
      tail &= sIntFIFO->mask_index;
      sIntFIFO->tail_index = tail;
   }
   return 0;
}


/****************************************************************************
*
*  Function:  setupThe_RT_IntFIFO
*
*  Description:
*     This routine setup the interrupt FIFO with the value needed for a 
*     interrupt on a RT message.  The interrupt is set for only the end-of
*     message.  
*
*     Any errors detected are reported to the user.  The user
*     has the option of terminating the test by striking any key.
*
****************************************************************************/
void setupThe_RT_IntFIFO(int cardnum)
{
   int rt,tr,sa,status;
   // Setup the FIFO structure for this board.
   memset(&rtIntFIFO, 0, sizeof(API_INT_FIFO));
   rtIntFIFO.function     = rt_intFunction;
   rtIntFIFO.iPriority    = THREAD_PRIORITY_ABOVE_NORMAL;
   rtIntFIFO.dwMilliseconds = INFINITE;
   rtIntFIFO.iNotification  = 0;
   rtIntFIFO.FilterType     = EVENT_RT_MESSAGE;
 
   for (rt = 0;rt<32;rt++)
      for (tr = 0; tr < 2; tr++ )
         for (sa = 0; sa < 32; sa++ )
            rtIntFIFO.FilterMask[rt][tr][sa] =  0xffffffff;  // Enable all Word Counts on all RT/SA/TR combos
  
   // Call the register function to register and start the thread.
   status = BusTools_RegisterFunction(cardnum, &rtIntFIFO, REGISTER_FUNCTION);
   printf("BusTools_RegisterFunction status = %d\n",status);
}
