/*============================================================================*
 * FILE:                 E X A M P L E _ R T _ S E T _ S T A T U S . C
 *============================================================================*
 *
 *      COPYRIGHT (C) 1998-2016 BY ABACO SYSTEMS, INC. 
 *      ALL RIGHTS RESERVED.
 *
 *      THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND
 *      COPIED ONLY IN ACCORDANCE WITH THE TERMS OF SUCH LICENSE AND WITH
 *      THE INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY
 *      OTHER COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE
 *      AVAILABLE TO ANY OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF THE
 *      SOFTWARE IS HEREBY TRANSFERRED.
 *
 *      THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT
 *      NOTICE AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY ABACO SYSTEMS.
 *
 *===========================================================================*
 *
 * FUNCTION:   EXAMPLE PROGRAM
 *             This example program that sets up RT1 with two subaddresses,
 *             SA1 RECEIVE and SA2 TRANSMIT. This example program also
 *             demostrates:
 *             - DISABLING and ENABLING an RT to respond on the 1553 Bus
 *             - toggling of the BUSY, SERVICE REQUEST, and TERMINAL FLAG
 *               bits in the RT Status Word
 *             - ILLEGALIZING of commands to an RTSA
 *
 *             An external BC device is required and configured to send a
 *             BC->RT command to RT1 SA1 and an RT->BC command to RT1 SA2,
 *             at a minimum. Recommended BC setup:
 *             - BC->RT Command RT1, SA1, RECEIVE, 32 Data Words, Bus A
 *             - RT->BC Command RT1, SA1, TRANSMIT, 32 Data Words, Bus A
 *             - BC->RT Command RT1, SA1, RECEIVE, 32 Data Words, Bus B
 *             - RT->BC Command RT1, SA1, TRANSMIT, 32 Data Words, Bus B
 *
 *             After DISABLING the RT, verify that the RT does not respond
 *             with Status to the BC Command on Bus A and Bus B. After
 *             ENABLING the RT, verify that the RT responds with Status
 *             to the BC Command on Bus A and Bus B.
 *
 *             Verify the RT Status Word Response using the BC device as the
 *             BUSY, SERVICE REQUEST, and TERMINAL FLAG bits are set and
 *             cleared in the RT Status Word. The BC List should execute
 *             at a rate to allow for monitoring of the changes to the RT
 *             Status Word.
 *
 *             When ILLEGALIZING commands to RT1-SA1-RX and RT1-SA2-TX, the
 *             RT will respond with the MESSAGE ERROR bit set in the status
 *             word. And, for the TRANSMIT command, the RT will not send any
 *             data words.
 *
 *             Note that alternative coding options are included for setting
 *             and clearing the RT Status Word bits.
 *
 *===========================================================================*/

/* $Revision:  8.12 Release $
   Date        Revision
   ----------  ---------------------------------------------------------------
*/

#include "busapi.h"
#include <stdio.h>

void  setUpTheRT(BT_INT ch_id);

// Main program
int main(int argc, char **argv)
{
   BT_INT            status, ch_id;
   API_RT_ABUF       rtAbuf;              // RT address buffer structure.
   API_RT_CBUF       rtCbuf;              // RT control buffer structure.
   int               exit_loop;
   char              c;
   BT_UINT           device, mode;

   if(argc == 2)
      device = atoi(argv[1]);             // Optional command-line device ID
   else
      device = 0;                         // Default device is 0, configure as needed


   // Open the device and get the channel id
   mode = API_B_MODE | API_SW_INTERRUPT;  // 1553B protocol, use SW interrupts.
   status = BusTools_API_OpenChannel(&ch_id, mode, device, CHANNEL_1);
   printf("BusTools_API_OpenChannel status = %d\n", status);

   // Bus select
   status = BusTools_SetInternalBus(ch_id, EXTERNAL_BUS);   // Select External Bus
   printf("BusTools_SetInternalBus status = %d\n", status); 

   // Setup RT
   setUpTheRT(ch_id);
   
   // Start RT 
   status = BusTools_RT_StartStop(ch_id, RT_START);
   printf("BusTools_RT_StartStop status = %d\n", status);   
   exit_loop=0;

   printf("\nInput: \n Q to stop RT and exit, \n D to Disable RT1 on Bus A & Bus B,");
   printf("\n E to Enable RT1 on Bus A & Bus B, \n I to Illegaize commands to RT1-SA1-RX and RT1-SA2-TX,");
   printf("\n L to Legalize commands to RT1-SA1-RX and RT1-SA2-TX, \n B to set BUSY bit in RT1 STATUS,");
   printf("\n S to set SR bit in RT1 STATUS, \n T to set TF bit in RT1 STATUS,");
   printf("\n C to clear RT1 STATUS bits, \n H to print this Input Menu\n\n");

   do // Run this loop until 'Q'uit is entered
   {

      scanf("%c",&c);

      switch(c)
      {
         case 'd':
         case 'D':
            // Disable RT1 on both Bus A & Bus B
            rtAbuf.enable_a = 0;             // DO NOT Respond on Bus A
            rtAbuf.enable_b = 0;             // DO NOT Respond on Bus B
            rtAbuf.inhibit_term_flag = 0;    // Inhibit terminal flag (check definition of this multi-purpose flag)
            rtAbuf.status   = 0x0000;        // Set RT1 Clear Status
            rtAbuf.bit_word = 0x0000;        // Set BIT word (for mode code 19)
            status = BusTools_RT_AbufWrite(ch_id, 1, &rtAbuf);
            printf("BusTools_RT_AbufWrite status = %d\n\n", status); 
            break;
         case 'e':
         case 'E':
            // Enable RT1 on both Bus A & Bus B
            rtAbuf.enable_a = 1;             // Respond on Bus A
            rtAbuf.enable_b = 1;             // Respond on Bus B
            rtAbuf.inhibit_term_flag = 0;    // Inhibit terminal flag (check definition of this multi-purpose flag)
            rtAbuf.status   = 0x0000;        // Set RT1 Clear Status
            rtAbuf.bit_word = 0x0000;        // Set BIT word (for mode code 19)
            status = BusTools_RT_AbufWrite(ch_id, 1, &rtAbuf);
            printf("BusTools_RT_AbufWrite status = %d\n\n", status); 
            break;
         case 'b':
         case 'B':
            // Set the Busy Bit (BUSY) for RT1 using BusTools_RT_AbufWrite()
            rtAbuf.enable_a = 1;             // Respond on Bus A
            rtAbuf.enable_b = 1;             // Respond on Bus B
            rtAbuf.inhibit_term_flag = 0;    // Inhibit terminal flag (check definition of this multi-purpose flag)
            rtAbuf.status   = 0x0008;        // Set BUSY bit in status word, clearing all other bits except RT Address
            // Alternative Code for line above
            //rtAbuf.status   = 0x0000 | API_1553_STAT_BY; // Set BUSY bit in status word, clearing all other bits except RT Address
            rtAbuf.bit_word = 0x0000;        // Set BIT word (for mode code 19)
            status = BusTools_RT_AbufWrite(ch_id, 1, &rtAbuf);
            printf("BusTools_RT_AbufWrite status = %d\n\n", status); 
            break;

            // Alternative Code for case 'B' not using BusTools_RT_AbufWrite()
            //// Set the Busy Bit (BUSY) for RT1 using BusTools_RT_MessageWriteStatusWord()
            //status = BusTools_RT_MessageWriteStatusWord(ch_id, 1, 1, 0, 0, 0x0800 | API_1553_STAT_BY, 1);
            //printf("BusTools_RT_MessageWriteStatusWord status = %d\n\n", status); 
            //break;
         case 's':
         case 'S':
            // Set the Service Request Bit (SRQ) for RT1 using BusTools_RT_AbufWrite()
            rtAbuf.enable_a = 1;             // Respond on Bus A
            rtAbuf.enable_b = 1;             // Respond on Bus B
            rtAbuf.inhibit_term_flag = 0;    // Inhibit terminal flag (check definition of this multi-purpose flag)
            rtAbuf.status   = 0x0100;        // Set SR bit in status word, clearing all other bits except RT Address
            // Alternative Code for line above
            //rtAbuf.status   = 0x0800 | API_1553_STAT_SR;        // Set SR bit in status word, clearing all other bits except RT Address
            rtAbuf.bit_word = 0x0000;        // Set BIT word (for mode code 19)
            status = BusTools_RT_AbufWrite(ch_id, 1, &rtAbuf);
            printf("BusTools_RT_AbufWrite status = %d\n\n", status); 
            break;

            // Alternative Code for case 'S' not using BusTools_RT_AbufWrite()
            //// Set the Service Request Bit (SRQ) for RT1 using BusTools_RT_MessageWriteStatusWord()
            //status = BusTools_RT_MessageWriteStatusWord(ch_id, 1, 1, 0, 0, 0x0800 | API_1553_STAT_SR, 1);
            //printf("BusTools_RT_MessageWriteStatusWord status = %d\n\n", status); 
            //break;
         case 't':
         case 'T':
            // set the Terminal Flag Bit (TF) for RT1 using BusTools_RT_AbufWrite()
            rtAbuf.enable_a = 1;             // Respond on Bus A
            rtAbuf.enable_b = 1;             // Respond on Bus B
            rtAbuf.inhibit_term_flag = 0;    // Inhibit terminal flag (check definition of this multi-purpose flag)
            rtAbuf.status   = 0x0001;        // Set TF bit in status word, clearing all other bits except RT Address
            // Alternative Code for line above
            //rtAbuf.status   = 0x0000 | API_1553_STAT_TF;  // Set TF bit in status word, clearing all other bits except RT Address
            rtAbuf.bit_word = 0x0000;        // Set BIT word (for mode code 19)
            status = BusTools_RT_AbufWrite(ch_id, 1, &rtAbuf);
            printf("BusTools_RT_AbufWrite status = %d\n\n", status);   
            break;

            // Alternative Code for case 'T' not using BusTools_RT_AbufWrite()
            //// Set the Terminal Flag Bit (TF) for RT1 using BusTools_RT_MessageWriteStatusWord()
            //status = BusTools_RT_MessageWriteStatusWord(ch_id, 1, 1, 0, 0, 0x0800 | API_1553_STAT_TF, 1);
            //printf("BusTools_RT_MessageWriteStatusWord status = %d\n\n", status); 
            //break;
         case 'c':
         case 'C':
            // Clear the Status Bits for RT1 using BusTools_RT_AbufWrite()
            rtAbuf.enable_a = 1;             // Respond on Bus A
            rtAbuf.enable_b = 1;             // Respond on Bus B
            rtAbuf.inhibit_term_flag = 0;    // Inhibit terminal flag (check definition of this multi-purpose flag)
            rtAbuf.status   = 0x0000;        // Clear SR and TF bits in status word
            rtAbuf.bit_word = 0x0000;        // Set BIT word (for mode code 19)
            status = BusTools_RT_AbufWrite(ch_id, 1, &rtAbuf);
            printf("BusTools_RT_AbufWrite status = %d\n\n", status);
            break;

            // Alternative Code for case 'C' not using BusTools_RT_AbufWrite()
            //// Clear the Status Bits for RT1 using BusTools_RT_MessageWriteStatusWord()
            //status = BusTools_RT_MessageWriteStatusWord(ch_id, 1, 1, 0, 0, 0x0800, 1);
            //printf("BusTools_RT_MessageWriteStatusWord status = %d\n\n", status); 
            //break;
         case 'i':
         case 'I':
            // Illegalize all commands to RT1 SA1 Receive and RT1 SA2 Transmit
            // For RT1, SA1 and SA2, set any Word Count Legal.
            // Set the Legal Wordcount for the RT Control Buffers.
            // Only legal word count can be changed using cbufwrite durinn system run time.
            rtCbuf.legal_wordcount = 0x00000000;   // any word count is legal.

            // Setup a control buffer - RT1, SA1, Receive, 1 buffer.
            status = BusTools_RT_CbufWrite(ch_id, 1, 1, 0, 1, &rtCbuf);

            // Setup a control buffer - RT1, SA2, Transmit, 1 buffer.
            status |= BusTools_RT_CbufWrite(ch_id, 1, 2, 1, 1, &rtCbuf);
            printf("BusTools_RT_CbufWrite status = %d\n", status);   
            break;
         case 'l':
         case 'L':
            // Legalize all commands to RT1 SA1 Receive and RT1 SA2 Transmit
            // For RT1, SA1 and SA2, set any Word Count Legal.
            // Set the Legal Wordcount for the RT Control Buffers.
            //Only legal word count can be changed using cbufwrite durinn system run time.
            rtCbuf.legal_wordcount = 0xFFFFFFFF;   // any word count is legal.

            // Setup a control buffer - RT1, SA1, Receive, 1 buffer.
            status = BusTools_RT_CbufWrite(ch_id, 1, 1, 0, 1, &rtCbuf);

            // Setup a control buffer - RT1, SA2, Transmit, 1 buffer.
            status |= BusTools_RT_CbufWrite(ch_id, 1, 2, 1, 1, &rtCbuf);
            printf("BusTools_RT_CbufWrite status = %d\n", status);   
            break;
         case 'h':
         case 'H':
            printf("\nInput: \n Q to stop RT and exit, \n D to Disable RT1 on Bus A & Bus B,");
            printf("\n E to Enable RT1 on Bus A & Bus B, \n I to Illegaize commands to RT1-SA1-RX and RT1-SA2-TX,");
            printf("\n L to Legalize commands to RT1-SA1-RX and RT1-SA2-TX, \n B to set BUSY bit in RT1 STATUS,");
            printf("\n S to set SR bit in RT1 STATUS, \n T to set TF bit in RT1 STATUS,");
            printf("\n C to clear RT1 STATUS bits, \n H to print this Input Menu\n\n");
            break;
         case 'q':
         case 'Q':
            exit_loop = 1;
      }
      fflush(stdin);
   } while(!exit_loop);

   printf("Stopping RT . . . ");
   status = BusTools_RT_StartStop(ch_id, RT_STOP);
   printf("BusTools_RT_StartStop status = %d\n", status);
   printf("RT Stopped.\n");

   // Close API and board
   printf("\nClosing API . . . ");
   status = BusTools_API_Close(ch_id);
   printf("BusTools_API_Close status = %d\n", status);
} // End of main


void setUpTheRT(BT_INT ch_id)
{
   int               i;
   API_RT_ABUF       rtAbuf;           // RT address buffer structure.
   API_RT_CBUF       rtCbuf;           // RT control buffer structure.
   API_RT_MBUF_WRITE msg_buffer_write; // RT message buffer structure.
   BT_INT            status;

   // Setup an RT.
   status = BusTools_RT_Init(ch_id, 0);
   printf("BusTools_RT_Init status = %d\n", status);   

   // Setup RT address buffer for RT (RT1)
   rtAbuf.enable_a = 1;                // Respond on Bus A
   rtAbuf.enable_b = 1;                // Respond on Bus B
   rtAbuf.inhibit_term_flag = 0;       // Inhibit terminal flag (check definition of this multi-purpose flag)
   rtAbuf.status = 0x0800;             // Set status word
   rtAbuf.bit_word = 0x0000;           // Set BIT word (for mode code 19)
   status = BusTools_RT_AbufWrite(ch_id, 1, &rtAbuf);
   printf("BusTools_RT_AbufWrite status = %d\n", status);   

   // Set the Legal Wordcount for the RT Control Buffers.
   rtCbuf.legal_wordcount = 0xFFFFFFFF;   // any word count is legal.
   
   // Setup a control buffer - RT1, SA1, Receive, 1 buffer.
   status = BusTools_RT_CbufWrite(ch_id, 1, 1, 0, 1, &rtCbuf);
   printf("BusTools_RT_CbufWrite status = %d\n", status);
   
   // Setup a control buffer - RT1, SA2, Transmit, 1 buffer.
   status = BusTools_RT_CbufWrite(ch_id, 1, 2, 1, 1, &rtCbuf);
   printf("BusTools_RT_CbufWrite status = %d\n", status);

   // Put data in the transmit and receive buffers
   msg_buffer_write.enable = BT1553_INT_END_OF_MESS;  // Enable interrupt on end of msg
   msg_buffer_write.error_inj_id = 0;                 // No error injection
   for (i=0; i<32; i++)
      msg_buffer_write.mess_data[i] = 0xAB00 + i;

   status = BusTools_RT_MessageWrite(ch_id, 1, 1, 0, 0, &msg_buffer_write);
   printf("BusTools_RT_MessageWrite status = %d\n", status);         

   status = BusTools_RT_MessageWrite(ch_id, 1, 2, 1, 0, &msg_buffer_write);
   printf("BusTools_RT_MessageWrite status = %d\n", status);
} // End of setUpTheRT

