/*============================================================================*
 * FILE:                CEI_WIN32_INTERRUPT_FUNCTIONS.C
 *============================================================================*
 *
 *      COPYRIGHT (C) 1998-2016 BY ABACO SYSTEMS, INC. 
 *      ALL RIGHTS RESERVED.
 *
 *      THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND
 *      COPIED ONLY IN ACCORDANCE WITH THE TERMS OF SUCH LICENSE AND WITH
 *      THE INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY
 *      OTHER COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE
 *      AVAILABLE TO ANY OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF THE
 *      SOFTWARE IS HEREBY TRANSFERRED.
 *
 *      THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT
 *      NOTICE AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY ABACO SYSTEMS.
 *
 *===========================================================================*
 *
 * FUNCTION:   BusTools/1553-API Library:
 *             Low-level Win32 specific interrupt, thread, and timer functons.
 *
 * EXTERNAL ENTRY POINTS:
 *
 * INTERRUPT AND THREAD FUNCTIONS:
 *
 *   CEI_THREAD_CREATE     -  Creates a thread
 *   CEI_THREAD_EXIT       -  Exit thread
 *   CEI_THREAD_DESTROY    -  Destroy the thread
 *   CEI_EVENT_CREATE      -  Create an event
 *   CEI_WAIT_FOR_EVENT    -  Infinite or timed wait for event
 *   CEI_EVENT_SIGNAL      -  Signal event
 *   CEI_EVENT_DESTROY     -  Destroy event
 *   CEI_MUTEX_CREATE      -  Create a critical section
 *   CEI_MUTEX_LOCK        -  Request ownership of a critical section 
 *   CEI_MUTEX_UNLOCK      -  Release ownership of a critical section 
 *   CEI_MUTEX_DESTROY     -  Delete a critical section
 *   CEI_HANDLE_DESTROY    -  Close a handle
 *
 * SYSTEM SPECIFIC TIME FUNCTIONS:
 *   CEI_GET_TIME          -  Gets the current system time in microseconds
 *   PerformanceCounter    -  Higly accurate time
 *
 *===========================================================================*/

/* $Revision:  6.01 Release $
   Date        Revision
  ----------   ---------------------------------------------------------------
 01/31/2006    Creation date.
 12/06/2006    Add return value to CEI_WAIT_FOR_EVENT
 05/06/2008    Update return codes
 07/12/2012    Modified CEI_THREAD_CREATE and CEI_THREAD_DESTROY. added
                CEI_THREAD_EXIT, CEI_MUTEX_LOCK, CEI_MUTEX_UNLOCK, and
                CEI_HANDLE_DESTROY. added debugging functionality. bch

*/

#include <stdio.h>
#include <Windows.h>
#include <process.h>
#include "target_defines.h"
#include "lowlevel.h"

extern void displayError(const CHAR *sErr,...);

// enable the low-level debugging option
#if defined(_DEBUG)
 #if defined(_MSC_VER) && _MSC_VER >= 1500
  #define LL_DEBUG_BT1553
  extern void _displayDebug(const CHAR *sDgb,...);
  #define displayDebug(...) if(IsDebuggerPresent()) _displayDebug(__VA_ARGS__)
 #endif
#endif


/***************************************************************************
* CEI Windows Thread Functions
***************************************************************************/

/****************************************************************
*
*  PROCEDURE - CEI_THREAD_CREATE
*
*  FUNCTION
*     Create Windows Thread.
*
*  RETURNS
*     BTD_OK
*     BTD_ERR_BADOPEN
****************************************************************/
CEI_INT CEI_THREAD_CREATE(CEI_THREAD *pThread, CEI_INT iPriority, void *func, void *arg) {
  HANDLE hThread;
  unsigned int threadID=0;
  unsigned int val32=0;

 #ifdef LL_DEBUG_BT1553
  displayDebug(">>>: %s", __FUNCTION__);
 #endif

  // prevents the "invalid parameter handler" for _beginthreadex
  if(func == NULL)
    return -1;

  *pThread = NULL;

  // when linked to libcmt.lib use endthreadex instead of ExitThread to reclaim allocated resources
  // NOTE: no security so thread cannot be inherited by a child process
  if((hThread = (HANDLE)_beginthreadex(NULL, 0, func, arg, CREATE_SUSPENDED, &threadID)) == 0) {
    displayError("CEI_THREAD_CREATE: _beginthreadex failed (err: %d)", GetLastError());
    return BTD_THRD_CREATE_FAIL;
  }
  *pThread = hThread;

  if(iPriority) {
    if(SetThreadPriority(hThread, iPriority) == 0) {
      displayError("CEI_THREAD_CREATE: SetThreadPriority failed (thread: %d, err: %d)", hThread, GetLastError());
      return BTD_SET_PRIORITY_ERR;
    }
  }

  if(ResumeThread(hThread) == -1) {
    displayError("CEI_THREAD_CREATE: ResumeThread failed (thread: %d, err: %d)", hThread, GetLastError());
    return BTD_ERR_BADOPEN;
  }

  // verify that the thread is active
  if(GetExitCodeThread(hThread, &val32)) {
    if(val32 != STILL_ACTIVE) {
      displayError("CEI_THREAD_CREATE: GetExitCodeThread (thread: %d, status: %d)", hThread, val32);
      CloseHandle(hThread);
      return BTD_THRD_CREATE_FAIL;
    }
  }  
 
 #ifdef LL_DEBUG_BT1553
  displayDebug("<<<: %s, thread: 0x%x/0x%x", __FUNCTION__, hThread, threadID);
 #endif

  return BTD_OK;
}

/****************************************************************
*
*  PROCEDURE - CEI_THREAD_DESTROY
*
*  FUNCTION
*     Destroys the thread and close thread handle.
*
*  RETURNS
*     BTD_OK
****************************************************************/
CEI_INT CEI_THREAD_DESTROY(CEI_THREAD *pThread) {  
 #ifdef LL_DEBUG_BT1553
  displayDebug(">>>: %s, thread: 0x%x", __FUNCTION__, *pThread);
 #endif

  // have the thread return an exit code of "-1" to indicate that the thread was terminated for troubleshooting
  if(TerminateThread(*pThread, -1) == 0) 
    displayError("CEI_THREAD_DESTROY: TerminateThread failed (thread: %d, err: %d)", *pThread, GetLastError());
  if(CloseHandle(*pThread) == 0)
    displayError("CEI_THREAD_DESTROY: CloseHandle failed (thread: %d, err: %d)", *pThread, GetLastError());
  
 #ifdef LL_DEBUG_BT1553
  displayDebug("<<<: %s", __FUNCTION__);
 #endif

  return BTD_OK;
}

/****************************************************************
*
*  PROCEDURE - CEI_THREAD_EXIT
*
*  FUNCTION
*     Terminates the thread returning the value provided. This 
*     will not close the thread's handle.
*
*  RETURNS
*     BTD_OK
****************************************************************/
CEI_INT CEI_THREAD_EXIT(CEI_VOID* pStatus, CEI_VOID* pVal) {
  CEI_INT exit_code=0;

  exit_code = *((CEI_INT*)pStatus);
  
 #ifdef LL_DEBUG_BT1553
  displayDebug(">>>: %s, thrd: 0x%x, ec: %d", __FUNCTION__, GetCurrentThreadId(), exit_code);
 #endif

  _endthreadex(exit_code);

  return BTD_OK;
}

/****************************************************************
*
*  PROCEDURE - CEI_EVENT_CREATE
*
*  FUNCTION
*     Create Windows Event.
*
*  RETURNS
*     BTD_OK
*     BTD_ERR_BADOPEN
****************************************************************/
CEI_INT CEI_EVENT_CREATE(CEI_EVENT *pEvent) {
  CEI_EVENT evt=NULL;

 #ifdef LL_DEBUG_BT1553
  displayDebug(">>>: %s", __FUNCTION__);
 #endif

  if((evt = CreateEvent(NULL, FALSE, FALSE, NULL)) == NULL) {
    displayError("CEI_EVENT_CREATE: CreateEvent failed (err: %d)", GetLastError());
    return BTD_ERR_BADOPEN;
  }
  *pEvent = evt;

 #ifdef LL_DEBUG_BT1553
  displayDebug("<<<: %s, event: 0x%x", __FUNCTION__, *pEvent);
 #endif

  return BTD_OK;
}

/****************************************************************
*
*  PROCEDURE - CEI_EVENT_DESTROY
*
*  FUNCTION
*     Destroys Windows Event.
*
*  RETURNS
*     BTD_OK
****************************************************************/
CEI_INT CEI_EVENT_DESTROY(CEI_EVENT *pEvent) {
 #ifdef LL_DEBUG_BT1553
  displayDebug(">>>: %s, event: 0x%x", __FUNCTION__, *pEvent);
 #endif

  if(CloseHandle(*pEvent) == 0)
    displayError("CEI_EVENT_DESTROY: CloseHandle failed (event: %d, err: %d)", *pEvent, GetLastError());

 #ifdef LL_DEBUG_BT1553
  displayDebug("<<<: %s", __FUNCTION__);
 #endif

  return BTD_OK;
}


/****************************************************************
*
*  PROCEDURE - CEI_WAIT_FOR_EVENT
*
*  FUNCTION
*     Waits for specified event.
*
*  RETURNS
*     Nothing
****************************************************************/
CEI_INT CEI_WAIT_FOR_EVENT(CEI_EVENT *pEvent, CEI_INT iDelay, CEI_MUTEX *pMutex) {
  DWORD retval=0;

 #ifdef LL_DEBUG_BT1553
  displayDebug(">>>: %s, event: 0x%x", __FUNCTION__, *pEvent);
 #endif

  retval = WaitForSingleObject(*pEvent, iDelay);
 
 #ifdef LL_DEBUG_BT1553
  displayDebug("<<<: %s, event: 0x%x", __FUNCTION__, *pEvent);
 #endif

  switch(retval) {
  case WAIT_FAILED:
    displayError("CEI_WAIT_FOR_EVENT: WaitForSingleObject failed (event: %d, err: %d)", *pEvent, GetLastError());
    return BTD_EVENT_WAIT_FAILED;
  case WAIT_ABANDONED:
    return BTD_EVENT_WAIT_ABANDONED;
  case WAIT_OBJECT_0:
    return BTD_OK;
  case WAIT_TIMEOUT:
    return BTD_EVENT_WAIT_TIMEOUT;
  default:
    return BTD_EVENT_WAIT_UNKNOWN;
  };       
}

/****************************************************************
*
*  PROCEDURE - CEI_EVENT_SIGNAL
*
*  FUNCTION
*     Sets the specified event for all waiting threads.
*
*  RETURNS
*     Nothing
****************************************************************/
CEI_INT CEI_EVENT_SIGNAL(CEI_EVENT *pEvent, CEI_MUTEX *pMutex) {
  CEI_INT status=0;

 #ifdef LL_DEBUG_BT1553
  displayDebug(">>>: %s, event: 0x%x", __FUNCTION__, *pEvent);
 #endif

  if(SetEvent(*pEvent) == 0) {
    displayError("CEI_EVENT_SIGNAL: SetEvent failed (event: %d, err: %d)", *pEvent, GetLastError());
    return BTD_EVENT_SIGNAL_ERR;
  }

 #ifdef LL_DEBUG_BT1553
  displayDebug("<<<: %s, event: 0x%x", __FUNCTION__, *pEvent);
 #endif

  return BTD_OK;
}

/****************************************************************
*
*  PROCEDURE - CEI_MUTEX_CREATE
*
*  FUNCTION
*     Create a Windows critical section
*
*  RETURNS
*     BTD_OK
****************************************************************/
CEI_INT CEI_MUTEX_CREATE(CEI_MUTEX *pMutex) {
 #ifdef LL_DEBUG_BT1553
  displayDebug(">>>: %s", __FUNCTION__);
 #endif

  InitializeCriticalSection(pMutex);
  // CreateMutex
 
 #ifdef LL_DEBUG_BT1553
  displayDebug("<<<: %s, mutex: 0x%x", __FUNCTION__, pMutex);
 #endif

  return BTD_OK;
}

/****************************************************************
*
*  PROCEDURE - CEI_MUTEX_LOCK
*
*  FUNCTION
*     Request ownership of a Windows critical section
*
*  RETURNS
*     BTD_OK
****************************************************************/
CEI_INT CEI_MUTEX_LOCK(CEI_MUTEX *pMutex) {
 #ifdef LL_DEBUG_BT1553
  displayDebug(">>>: %s, mutex: 0x%x", __FUNCTION__, pMutex);
 #endif

  EnterCriticalSection(pMutex);
  // WaitForSingleObject
 
 #ifdef LL_DEBUG_BT1553
  displayDebug("<<<: %s, mutex: 0x%x", __FUNCTION__, pMutex);
 #endif

  return BTD_OK;
}

/****************************************************************
*
*  PROCEDURE - CEI_MUTEX_UNLOCK
*
*  FUNCTION
*     Release ownership of a Windows critical section
*
*  RETURNS
*     BTD_OK
****************************************************************/
CEI_INT CEI_MUTEX_UNLOCK(CEI_MUTEX *pMutex) {
 #ifdef LL_DEBUG_BT1553
  displayDebug(">>>: %s, mutex: 0x%x", __FUNCTION__, pMutex);
 #endif

  LeaveCriticalSection(pMutex);
  // ReleaseMutex
 
 #ifdef LL_DEBUG_BT1553
  displayDebug("<<<: %s, mutex: 0x%x", __FUNCTION__, pMutex);
 #endif

  return BTD_OK;
}

/****************************************************************
*
*  PROCEDURE - CEI_MUTEX_DESTROY
*
*  FUNCTION
*     Delete a Windows critical section
*
*  RETURNS
*     BTD_OK
****************************************************************/
CEI_INT CEI_MUTEX_DESTROY(CEI_MUTEX *pMutex) {
 #ifdef LL_DEBUG_BT1553
  displayDebug(">>>: %s, mutex: 0x%x", __FUNCTION__, pMutex);
 #endif

  DeleteCriticalSection(pMutex);
  // CloseHandle

 #ifdef LL_DEBUG_BT1553
  displayDebug("<<<: %s", __FUNCTION__);
 #endif

  return BTD_OK;
}


/****************************************************************
*
*  PROCEDURE - CEI_HANDLE_DESTROY
*
*  FUNCTION
*     Destroys a Windows Handle.
*
*  RETURNS
*     BTD_OK
****************************************************************/
CEI_INT CEI_HANDLE_DESTROY(CEI_HANDLE *pHandle) {
  if(pHandle == NULL)
    return BTD_ERR_PARAM;

 #ifdef LL_DEBUG_BT1553
  displayDebug(">>>: %s, handle: 0x%x", __FUNCTION__, *pHandle);
 #endif

  if(CloseHandle(*pHandle) == 0)
    displayError("CEI_HANDLE_DESTROY: CloseHandle failed (handle: %d, err: %d)", *pHandle, GetLastError());

 #ifdef LL_DEBUG_BT1553
  displayDebug("<<<: %s", __FUNCTION__);
 #endif

  return BTD_OK;
}


/****************************************************************
*
*  PROCEDURE - CEI_GET_TIME
*
*  FUNCTION
*     Returns the system time, in milliseconds.
*
*  RETURNS
*     current time
****************************************************************/
unsigned long CEI_GET_TIME(void) {
  return(timeGetTime());
}

/****************************************************************************
*
*  PROCEDURE NAME - PerformanceCounter()
*
*  FUNCTION:    Microsoft Windows QuerryPerformanceCounter() interface.
*
*  DESCRIPTION: This function is used to read the performance counter in
*               32-bit Windows, to determine timing.
*
*  This function will return:
*              64-bit timer value in microseconds since first call
****************************************************************************/
CEI_UINT64 PerformanceCounter(void) {
  CEI_UINT64 liPerf, current_time;
  static CEI_UINT64 initial_time;
  static int first_pass = 1;
  static int time_correction=0;
  CEI_UINT64 liFreq;     // Frequency of the performance counter.

  // If command is to initialize the counter, read and save it.
  if(first_pass) {
    first_pass = 0;
    // 1,193,182 or the CPU clock frequency (448,790,000 for 450 MHz)
    QueryPerformanceFrequency((LARGE_INTEGER *)&liFreq);
    QueryPerformanceCounter((LARGE_INTEGER *)&initial_time);
    // Round the time correction factor:
    time_correction = (long)((liFreq+49999L)/100000L);
    return 0;
  }

  // Otherwise return the time from the command == 0 call to now.
  QueryPerformanceCounter((LARGE_INTEGER *)&liPerf);
  current_time = (liPerf - initial_time) * 10;
  current_time = current_time / time_correction;

  return current_time;
}
