/*============================================================================*
 * FILE:                      I R Q S E T U P . C 
 *============================================================================*
 *
 *      COPYRIGHT (C) 1998-2016 BY ABACO SYSTEMS, INC. 
 *      ALL RIGHTS RESERVED.
 *
 *      THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND
 *      COPIED ONLY IN ACCORDANCE WITH THE TERMS OF SUCH LICENSE AND WITH
 *      THE INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY
 *      OTHER COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE
 *      AVAILABLE TO ANY OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF THE
 *      SOFTWARE IS HEREBY TRANSFERRED.
 *
 *      THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT
 *      NOTICE AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY ABACO SYSTEMS.
 *
 *===========================================================================*
 *
 * FUNCTION:   BusTools/1553-API Library:
 *             Hardware and software interrupt component.
 *             This file is used for 32-bit WINDOWS             
 *
 * DESCRIPTION:
 *
 * API ENTRY POINTS: (only used by BUSAPI.DLL, BUSAPI32.DLL and BUSAPIDx.LIB):
 *    vbtInterruptSetup       Initializes hardware and/or software interrupts
 *                            for a specified adapter offset.
 *    vbtInterruptClose       Shuts down hardware and/or software interrupts
 *                            for a specified adapter.  Closes all interrupt
 *                            functions if this is the final board being closed.
 *
 * EXTERNAL NON-USER ENTRY POINTS:
 *
 * INTERNAL ROUTINES:
 *    intTimerCallback        Windows multimedia timer callback procedure.
 *
 *===========================================================================*/

/* $Revision:  8.14 Release $
   Date        Revision
  ----------   ---------------------------------------------------------------
  03/31/1999   Completed hardware interrupt support.V3.03.ajh
  11/05/1999   Activated hardware interrupt support.V3.30.ajh
  01/11/2000   Added error reporting thread.V3.30.ajh
  01/18/2000   Added support for the ISA-1553, PMC-1553, and IP PROM V5.V4.00.ajh
  09/07/2000   Modified the way the Error Reporting thread works to help insure
               that it would exit quickly.  Removed the 500 ms polling and
               replaced it with an event object.V4.13.ajh
  10/17/2000   Enable HW interrupts for PCI/PMC-1553 Rev 3 prom boards.V4.18.ajh
  10/25/2000   Modified the way the Error Reporting thread is killed.V4.19.ajh
  11/14/2000   Modified setting HW interrupts for Rev 3 PROM so if another
               process had enabled interrupts we would not fail.V4.22.ajh
  01/02/2000   Fixed vbtInterruptClose problem that closed interrupt processing
               for all open boards when a single board was closed.V4.30.ajh
  01/07/2002   Added supprt for Quad-PMC and Dual Channel IP V4.46 rhc
  08/02/2004   Add interrupt support for the QCP-1553
  12/30/2005   BusTool_SetPolling now returns an error if polling is not on
  12/30/2005   Update for portiblity.
  12/03/2010   Hardware interrupt support removed for IP-D1553. bch
  11/05/2014   Add user allocated buffer in the dump file.
 */

/*---------------------------------------------------------------------------*
 *                    INCLUDES, DEFINES and GLOBALS
 *---------------------------------------------------------------------------*/

#include <stdio.h>
#include <windows.h>
#include "busapi.h"
#include "apiint.h"
#include "globals.h"
#include "btdrv.h"
#include <mmsystem.h>

static CEI_MUTEX error_mutex;
static char   szErrorMsg[800];       // Error message text to display
static char   szErrorTitle[100];     // Error message title to display
static int    iErrorNumber;          // Error number being displayed

// Timer, counts from 0 up to BM_RECORD_TIME then calls the BM recorder
static long bmRecordTime[MAX_BTA] = {0,};
static UINT timerId = 0;          // Timer ID used to close the timer callback.

static CEI_THREAD hIntThrd[MAX_BTA];  // Interrupt thread handle, if it exists.
static CEI_EVENT hIntEvent[MAX_BTA]; // Interrupt signal event, if it exists.
DWORD WINAPI threadStub(LPVOID pFIFOLocal);

//#define TIMER_INCREMENT_TEST

/*---------------------------------------------------------------------------*
 *          LOCAL ROUTINES, NOT REFERENCED OUTSIDE OF THIS MODULE
 *---------------------------------------------------------------------------*/

/*===========================================================================*
 * ENTRY POINT:           i n t T i m e r C a l l b a c k
 *===========================================================================*
 *
 * FUNCTION:    Windows multimedia timer callback procedure.
 *              See Multimedia function timeSetEvent() for details.
 *
 * DESCRIPTION: Count up the time to the BM Recording event.  Call vbt Notify
 *              for each board which is open, so it can process interrupts.
 *
 *      It will return:
 *              Nothing.
 *===========================================================================*/

NOMANGLE void CCONV CALLBACK intTimerCallback(
   UINT id,
   UINT msg,
   DWORD dwUser,
   DWORD dw1,
   DWORD dw2)
{
   /*******************************************************************
   *  Local variables
   *******************************************************************/
   static  int inTimerCallBack = 0;
   int     cardnum;    // Board counter.
   BT_U32BIT beat;
   static bcnt=0;
   

   inTimerCallBack++;

   if ( inTimerCallBack > 1 )  // If the previous timer interrupt has not
      return;                  //  completed just return.

   for ( cardnum = 0; cardnum < MAX_BTA; cardnum++ )
   {
      // Only call cards which have been initialized.
      if ( bt_inited[cardnum] )
      {
#ifdef TIMER_INCREMENT_TEST
         {
            static CEI_UINT64 total_value=0, last_micro=1;
            static BT_UINT tcnt=1;
            static CEI_UINT64 timer_increment;
            BT1553_TIME iTime;
            int status;

            tcnt++;
            status = BusTools_TimeTagRead(cardnum,&iTime);

            total_value += iTime.microseconds - last_micro;
            if((tcnt % 10) == 0)
            {
               timer_increment = total_value/10;
               printf("\n\n***********************************************************************\n");
               printf("*- - - - -  - - - - - - - - - - - - - > Timer Increment = %ld\n",timer_increment/1000);
               printf("***********************************************************************\n\n");
               total_value = 0;
            }
            last_micro = iTime.microseconds;
        
         }
#endif //TIMER_INCREMENT_TEST

         if(hw_int_enable[cardnum]==API_MANUAL_INT)
            return;

         if(++bcnt > 10)
         {
            if(board_is_v5_uca[cardnum])
               beat = vbtGetRegister16(cardnum,HWREG_HEART_BEAT);
            else
               beat = vbtGetRegister[cardnum](cardnum,HWREG_HEART_BEATV6);
  
            if(beat == heartbeat[cardnum]) //
               channel_status[cardnum].wcs_pulse=1;
            heartbeat[cardnum]=beat;
            bcnt=0;
         }
         // Count the time to next BM recording event.
         bmRecordTime[cardnum] += api_polling_interval;
#ifdef DEMO_CODE
         if ( bt_inuse[cardnum] < 0 )        // -1=SW version of the API must
            vbtHardwareSimulation(cardnum);  //      first call the simulator.
#endif
         // Call vbtNotify() at least every timer tick to process messages.
         // If hardware interrupts are enabled just signal the HW interrupt
         //  event.  This prevents re-entering the vbt Notify() function.
         if ( hIntEvent[cardnum] != 0x0 )
            SetEvent(hIntEvent[cardnum]); // This calls vbtNotify
         else
         {
            vbtNotify[cardnum](cardnum, &bmRecordTime[cardnum]);
         }
      } 
   }
   inTimerCallBack = 0;
}

/*===========================================================================*
 * ENTRY POINT:       i n t e r r u p t T h r e a d F u n c t i o n
 *===========================================================================*
 *
 * FUNCTION:    This is a glue function which is called by vbtHookInterrupt
 *              when a HW interrupt occurs.  It passes the call to vbt Notify().
 *
 *      It will return:
 *              Nothing.
 *===========================================================================*/
static void _stdcall interruptThreadFunction(int cardnum)
{
   BT_U32BIT curTime;
   static BT_U32BIT lastTime=0;
   // All boards have their interrupts cleared in the ISR.
   // Process the HW or timer interrupt.
   if(procBmOnInt[cardnum])
   {
      curTime = timeGetTime();
      if(lastTime != 0)
         bmRecordTime[cardnum] += (curTime - lastTime);
      else
         bmRecordTime[cardnum] = 1;

      lastTime = curTime;
   }
   vbtNotify[cardnum](cardnum, &bmRecordTime[cardnum]);
}

/*===========================================================================*
 * ENTRY POINT:           v b t I n t e r r u p t S e t u p
 *===========================================================================*
 *
 * FUNCTION:    Setup specified adapter for interrupt processing.
 *
 * DESCRIPTION: Setup the hardware interrupt if hardware interrupts are
 *              enabled, and setup the polling interrupt if not setup already.
 *
 *      It will return:
 *              Nothing.
 *===========================================================================*/

BT_INT  vbtInterruptSetup(BT_UINT cardnum,      // (i) card number
                       BT_INT hw_int_enable, // (i) enable HW interrupts flag
                       BT_INT RegDevNum)     // (i) hardware interrupt number
                                             //     or device number in WIN32
{
   
   /*******************************************************************
   *  Local variables
   *******************************************************************/
   int    status;            // Function return status

   /*******************************************************************
   *  Setup the hardware interrupt function if interrupts enabled,
   *   but only if HW interrupt is defined and if this is not the
   *   software version of the driver.
   *******************************************************************/
   hIntThrd[cardnum]  = NULL;
   hIntEvent[cardnum] = NULL;

   if ( hw_int_enable > 1 && (bt_inuse[cardnum] > 0) )
   {
      if(CurrentCardType[cardnum] == PCCD1553 || CurrentCardType[cardnum] == R15USB)
      {
         return API_NO_INTERRUPT_SUPPORT;
      }
    
      if(board_is_v5_uca[cardnum])
         vbtSetHWRegister(cardnum, HWREG_WRITE_INTERRUPT, 0);	  // Clear interrupt by writing a zero to WRITE_INTERRUPT_BIT (Bit 9)
      else
         api_clearhwcbits(cardnum,CR1_HOST_INTERP);
	  
	  if (CurrentCardType[cardnum] == QPCI1553)
      {
         BT_U16BIT * plx_base;
		 plx_base = (BT_U16BIT *)bt_iobase[cardnum];
         plx_base[38] |= 0x48;
      }
	  if (CurrentCardType[cardnum] == QCP1553 || CurrentCardType[cardnum] == QPCX1553)
      {
         BT_U16BIT * plx_base;
		 plx_base = (BT_U16BIT *)bt_iobase[cardnum];         
         plx_base[52] |= 0x900;
      }

      /*******************************************************************
      * Setup the interrupt ISR, DPC and processing thread.
      *******************************************************************/
      status = vbtHookInterrupt(api_device[cardnum],
                                cardnum,
                                interruptThreadFunction,
                                THREAD_PRIORITY_TIME_CRITICAL,
                                &hIntThrd[cardnum],
                                &hIntEvent[cardnum],
                                CurrentCardType[cardnum],
                                CurrentCardSlot[cardnum],
                                CurrentCarrier[cardnum]);

      // If we enabled interrupts for the V3 PCI1553 prom problem, and the
      //  error code from vbtHookInterrupt is 0xAA, ignore the error.

      if ( (status == 0xAA) && (hw_int_enable >= 2) )    // V4.22.ajh
      {
      }
      else
      {
         if ( (hIntThrd[cardnum] == NULL) || (status != BTD_OK) )
		 {  // Internal LowLevel error setting up the interrupt thread.
            return status;
		 }

         // Initialize the hardware so interrupts are enabled...

         // Set CPU_INTERRUPT_ENABLE (bit 14) of 1553_CONTROL_REG
         //  (HWREG_CONTROL1) to enable interrupts.  CPU_INTERRUPT
         //  (bit 9) will be set when a HW interrupt occurs.
         // Clear interrupt by writing a zero to WRITE_INTERRUPT_BIT (Bit 9)
         //  of WRITE_INTERRUPT_REG(reg 0x16-0x17), or set it by writing a one.
         if(board_is_v5_uca[cardnum])
            api_writehwreg_or(cardnum, HWREG_CONTROL1, CR1_INT_ENABLE);
         else   
            api_sethwcbits(cardnum, CR1_INT_ENABLE);
      }
   }

   /*******************************************************************
   *  Setup the timer callback routine so we can poll. 
   *******************************************************************/
   if(hw_int_enable <= 2)
      vbtSetPolling(api_polling_interval,TIMER_START);
   return BTD_OK;
}

/*===========================================================================*
 * ENTRY POINT:           v b t S e t P o l l i n g
 *===========================================================================*
 *
 * FUNCTION:    The function sets the polling interval for the application
 *
 * DESCRIPTION: The function stop the timer set the timing interval and
 *              restarts the timer
 *
 *      It will return: status
 *             
 *===========================================================================*/
BT_INT vbtSetPolling(BT_UINT polling, // (i) polling interval
                     BT_UINT tflag)   // (i) timer option 0 - start; 1 - restart
{
   if(tflag==TIMER_RESTART)
   {   
      if ( timerId )// if timer is running stop it
      {
         api_polling_interval = polling;
         if ( api_polling_interval > MAX_POLLING )
           api_polling_interval = MAX_POLLING;
         if ( api_polling_interval < MIN_POLLING )
            api_polling_interval = MIN_POLLING;

         timeKillEvent(timerId);              // Destroy the WINDOWS callback timer.
         timeEndPeriod(api_polling_interval); // Matched with timeBeginPeriod()
         timerId = 0;                         // The timer has been destroyed.
      }
      else
         return API_NO_POLLING;
   }

   if(timerId==TIMER_START)
   {

      // We only use one timer, it is set to only one period.  We assume
      // Close with timeEndPeriod(api_polling_interval);
      if((timeBeginPeriod(api_polling_interval))== TIMERR_NOERROR)
      {
         // Setup the timed callback.  Parameters are delay(ms), resolution,
         //  function to call, call-back data, type of timer event.
         // Matched with a call to timeKillEvent();
 
        timerId = timeSetEvent(api_polling_interval, 0,
                               (LPTIMECALLBACK)intTimerCallback, (DWORD)0L, TIME_PERIODIC);
      }
      else
         return BTD_TIMER_FAIL;
   }
   return BTD_OK;
}

/*===========================================================================*
 * ENTRY POINT:           v b t I n t e r r u p t C l o s e
 *===========================================================================*
 *
 * FUNCTION:    Disables interrupts from the specified adapter.
 *
 * DESCRIPTION: The hardware interrupt is shutdown and the timer callback
 *              is distroyed if this is the last operational adapter.
 *
 *      It will return:
 *              Nothing.
 *===========================================================================*/

void vbtInterruptClose(BT_UINT cardnum)        // (i) card number
{
   /*******************************************************************
   *  Local variables
   *******************************************************************/
   int     i;

   if ( hIntThrd[cardnum] != NULL )     // Kill the interrupt thread?
   {
      // Disable the hardware interrupts from the board...
      // Set/clear CPU_INTERRUPT_ENABLE (bit 14) of 1553_CONTROL_REG
      //  (HWREG_CONTROL1) to enable/disable interrupts.  CPU_INTERRUPT
      //  (bit 9) will be set when a HW interrupt occurs.
      // Clear interrupt by writing a zero to WRITE_INTERRUPT_BIT (Bit 9)
      //  of WRITE_INTERRUPT_REG(reg 0x16-0x17), or set it by writing a one.
      if(board_is_v5_uca[cardnum])
      {
         api_writehwreg_and(cardnum, HWREG_CONTROL1, (BT_U16BIT)~CR1_INT_ENABLE);
         api_writehwreg(cardnum, HWREG_WRITE_INTERRUPT, 0);
      }
      {
         api_clearhwcbits(cardnum,CR1_INT_ENABLE);
         api_clearhwcbits(cardnum,CR1_HOST_INTERP);
      } 
      vbtCloseInterrupt(api_device[cardnum],cardnum);       // Kill the interrupt thread.
      hIntThrd[cardnum] = NULL;         // Remember that it is gone.
      hIntEvent[cardnum] = NULL;
   }

   /*******************************************************************
   *  If any boards are still running, return.  Else destroy the timer.
   *******************************************************************/
   bt_inuse[cardnum] = 0;               // This board is closed.
   for ( i = 0; i < MAX_BTA; i++ )
      if ( bt_inuse[i] )                // If any other boards are still
         return;                        //   open leave the timer running (V4.30).

   if ( timerId )
   {
      timeKillEvent(timerId);           // Distroy the WINDOWS callback timer.
      timeEndPeriod(api_polling_interval); // Matched with timeBeginPeriod()
      timerId = 0;                      // The timer has been distroyed.
   }

}
