/*============================================================================*
 * FILE:                       D M A . C
 *============================================================================*
 *
 *      COPYRIGHT (C) 1998-2016 BY ABACO SYSTEMS, INC. 
 *      ALL RIGHTS RESERVED.
 *
 *      THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND
 *      COPIED ONLY IN ACCORDANCE WITH THE TERMS OF SUCH LICENSE AND WITH
 *      THE INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY
 *      OTHER COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE
 *      AVAILABLE TO ANY OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF THE
 *      SOFTWARE IS HEREBY TRANSFERRED.
 *
 *      THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT
 *      NOTICE AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY ABACO SYSTEMS.
 *
 *===========================================================================*
 *
 * FUNCTION:   This file contains all function related to DMA.
 *
 * USER ENTRY POINTS: 
 *     BusTools_DMA_Setup        - configure PLX 9056 for Block DMA
 *     BusTools_DMA_Start        - initiate a DMA transfer
 *     BusTools_DMA_Setup2       - configure a DMA channel returning a host address
 *     BusTools_DMA_Transfer     - initiate a transfer on a DMA channel using a host address
 *     BusTools_DMA_Init         - configure a DMA channel
 *     BusTools_DMA_Read         - initiate a transfer on a DMA channel
 *     BusTools_DMA_Close        - disables a DMA channel
 *
 *
 * EXTERNAL NON-USER ENTRY POINTS:
 *
 * INTERNAL ROUTINES:
 *     set_plx_reg               - set the PLX register to the provided value
 *     get_plx_reg               - get the value from the PLX register
 *     or_plx_Reg                - OR the PLX register with the provided value
 *
 *===========================================================================*/

/* $Revision:  6.44 Release $
   Date        Revision
  ----------   ---------------------------------------------------------------
  12/03/2010   Initial release. rhc
  12/03/2010   Added BusTools_DMA_Setup2, BusTools_DMA_Transfer,
                BusTools_DMA_Init, BusTools_DMA_Read, BusTools_DMA_Close. bch
  06/22/2012   replaced vbtSetPLXRegister32 and vbtSetPLXRegister8 with
                set_plx_reg. bch
  04/07/2014   added delay to BusTools_DMA_Transfer and removed a local buffer
                in BusTools_DMA_Read. bch		
  02/26/2015   replaced vbtGetPLXRegister with get_plx_reg. bch
  08/01/2016   modified BusTools_DMA_Setup2, BusTools_DMA_Init, 
                BusTools_DMA_Read, and BusTools_DMA_Close to pass the device
                channel instead of the DMA channel to lowlevel. added
                or_plx_Reg to BusTools_DMA_Setup2 and BusTools_DMA_Init. bch
*/

#include <stdio.h>
#include "busapi.h"
#include "apiint.h" 
#include "btdrv.h"
#include "globals.h"


#define set_plx_Reg(_cn, _rn, _v) (*(BT_U32BIT*)(bt_iobase[_cn] + _rn)) = flips(_v)
#define or_plx_Reg(_cn, _rn, _v) (*(BT_U32BIT*)(bt_iobase[_cn] + _rn)) |= flips(_v)
#define get_plx_Reg(_cn, _rn) flips(*((BT_U32BIT*)(bt_iobase[_cn] + _rn)))

/****************************************************************************
*
*  PROCEDURE NAME -   BusTools_DMA_Setup
*
*  FUNCTION
*       This function sets up a block DMA transfer for device with PLX 9056
*   
*
*  RETURNS
*       API_SUCCESS (0)   -> success
*       API_HARDWARE_NOSUPPORT -> illegal address type specified
*       API_BUSTOOLS_BADCARDNUM -> bad card number specified
*       API_BUSTOOLS_NOTINITED  -> Channel not initialized
*
****************************************************************************/
NOMANGLE BT_INT CCONV BusTools_DMA_Setup(
  BT_UINT cardnum,             /* (i) card number (0 - 3)                                */
  BT_UINT dma_channel,         /* (i) 0 = channel 0 1 or > = channel 1                   */
  BT_U32BIT host_addr,         /* (i) Host address for dma read/write (physical address) */
  BT_U32BIT board_addr,        /* (i) board address for dma read/write                   */
  BT_U32BIT byte_count,        /* (i) Number of bytes to transfer                        */
  BT_U32BIT dma_flag)          /* (i) direction of transfer 0 = board to host            */
                               /*                           1 or > = host to board       */
{
  /************************************************
   *  Error checking
   ************************************************/
  if(cardnum >= MAX_BTA)
    return API_BUSTOOLS_BADCARDNUM;

  if(bt_inited[cardnum] == 0)
    return API_BUSTOOLS_NOTINITED;

  if(!board_has_plx_dma[cardnum])
    return API_HARDWARE_NOSUPPORT;

  if(dma_channel == 0) {
    set_plx_Reg(cardnum,PLX_DMA_HOST_ADDR0, host_addr);
    set_plx_Reg(cardnum,PLX_DMA_BOARD_ADDR0, board_addr);
    set_plx_Reg(cardnum,PLX_DMA_BYTE_COUNT0, byte_count);
    if(dma_flag)
      set_plx_Reg(cardnum,PLX_DMA_FLAG0, 0x0);
    else
      set_plx_Reg(cardnum,PLX_DMA_FLAG0, 0x8);
  }
  else {
    set_plx_Reg(cardnum,PLX_DMA_HOST_ADDR1, host_addr);
    set_plx_Reg(cardnum,PLX_DMA_BOARD_ADDR1, board_addr);
    set_plx_Reg(cardnum,PLX_DMA_BYTE_COUNT1, byte_count);
    if(dma_flag)
      set_plx_Reg(cardnum,PLX_DMA_FLAG1, 0x0);
    else
      set_plx_Reg(cardnum,PLX_DMA_FLAG1, 0x8);      
  }

  set_plx_Reg(cardnum,PLX_DMA_THRESHOLD,0x0000f000);

  return API_SUCCESS;
}


/****************************************************************************
*
*  PROCEDURE NAME -   BusTools_DMA_Start
*
*  FUNCTION
*       This function initates the DMA transfer
*
*  RETURNS
*       API_SUCCESS (0)   -> success
*       API_HARDWARE_NOSUPPORT -> illegal address type specified
*       API_BUSTOOLS_BADCARDNUM -> bad card number specified
*       API_BUSTOOLS_NOTINITED  -> Channel not initialized
*
****************************************************************************/
NOMANGLE BT_INT CCONV BusTools_DMA_Start(
  BT_UINT cardnum,             /* (i) card number (0 - 3) */
  BT_U32BIT dma_channel)       /* (i)                     */
{
  /************************************************
   *  Error checking
   ************************************************/
  if(cardnum >= MAX_BTA)
    return API_BUSTOOLS_BADCARDNUM;

  if(bt_inited[cardnum] == 0)
    return API_BUSTOOLS_NOTINITED;

  if(!board_has_plx_dma[cardnum])
    return API_HARDWARE_NOSUPPORT;

  set_plx_Reg(cardnum, PLX_DMA_STATUS0, 0x3);

  return API_SUCCESS;
}


/* 
   The following are new API functions for performing block DMA transfers on
   a device. There are two methods to do a DMA transfer. One is using the
   functions BusTools_DMA_Setup2 and BusTools_DMA_Transfer which uses a host
   address which is mapped to the board address. The second method uses the
   functions BusTools_DMA_Init and BusTools_DMA_Read, which does not require
   a mapped host address.
*/
/****************************************************************************
*
*  PROCEDURE NAME -   BusTools_DMA_Setup2
*
*  FUNCTION
*       This function configures the device for block DMA transfer. User 
*       specifies the DMA channel, the board address, number of bytes to be
*       transfered, and the direction. The returned host address is the
*       mapped user address to be used for read/write to the device. Used
*       in conjuction with BusTools_DMA_Transfer.
*
*  RETURNS
*       API_SUCCESS (0) -> success
*       API_HARDWARE_NOSUPPORT -> illegal address type specified
*       API_BUSTOOLS_BADCARDNUM -> bad card number specified
*       API_BUSTOOLS_NOTINITED  -> channel not initialized
*       BTD_ERR_PARAM -> invalid parameter (device or DMA channel)
*       BTD_ERR_INUSE -> DMA channel already in use
*       BTD_ERR_NOWINRT -> an invalid handle for the device
*       BTD_ERR_LOAD_CEIINST -> invalid pointer to CEI_Install DLL
*       BTD_ERR_NOACCESS -> device does not support DMA
*       BTD_ERR_BADEXTMEM -> invalid DMA address or size 
*
****************************************************************************/
NOMANGLE BT_INT CCONV BusTools_DMA_Setup2(
  BT_UINT cardnum,      /* (i) card number                             */
  BT_UINT dma_channel,  /* (i) 0 = channel 0, 1 = channel 1            */
  CEI_VOID *host_addr,  /* (o) host address for DMA read/write         */
  BT_U32BIT board_addr, /* (i) board address for DMA read/write        */
  BT_U32BIT byte_count, /* (i) number of bytes to transfer             */
  BT_U32BIT dma_flag)   /* (i) direction of transfer 0 = board to host */
                        /*                           1 = host to board */
{
  BT_INT status=0;
 #ifdef WIN64
  CEI_UINT64 dma_usr_addr=0;
  CEI_UINT64 dma_phy_addr=0;
 #else
  CEI_UINT32 dma_usr_addr=0;
  CEI_UINT32 dma_phy_addr=0;
 #endif
   
  if(cardnum >= MAX_BTA)
    return API_BUSTOOLS_BADCARDNUM;
    
  if(bt_inited[cardnum] == 0)
    return API_BUSTOOLS_NOTINITED;
    
  if(!board_has_plx_dma[cardnum])
    return API_HARDWARE_NOSUPPORT;
    
  /* configure a DMA channel for the device */
  status = vbtDMASetup(api_device[cardnum], CurrentCardSlot[cardnum], byte_count, dma_flag, &dma_phy_addr, &dma_usr_addr); 
  if(status != BTD_OK)
    return status; 
 #ifdef WIN64
  *((BT_U64BIT*)host_addr) = (BT_U64BIT)dma_usr_addr;
 #else
  *((BT_U32BIT*)host_addr) = (BT_U32BIT)dma_usr_addr;
 #endif
    
  /* set the offset into the channel's RAM */
  board_addr += (CurrentCardSlot[cardnum]*0x200000) + 0x100000;
    
  /* Mode: 0x00000043 (default) */
  /* program the PCI memory space, board offset and size to the PLX registers */
  if(dma_channel == 0) {
    set_plx_Reg(cardnum, PLX_DMA_HOST_ADDR0, (CEI_UINT32)dma_phy_addr); 
    set_plx_Reg(cardnum, PLX_DMA_BOARD_ADDR0, board_addr);
    set_plx_Reg(cardnum, PLX_DMA_BYTE_COUNT0, byte_count);
    if(dma_flag) {  /* PCI bus to Local bus */
      set_plx_Reg(cardnum, PLX_DMA_FLAG0, 0x0);
      /* requesting 32 full entries in FIFO before transfer */
      or_plx_Reg(cardnum, PLX_DMA_THRESHOLD, 0x0000000F);
    }
    else {  /* Local bus to PCI bus */
      set_plx_Reg(cardnum, PLX_DMA_FLAG0, 0x8);
      /* requesting 32 full entries in FIFO before transfer */
      or_plx_Reg(cardnum, PLX_DMA_THRESHOLD, 0x00000F00);
    }
  }
  else {
    set_plx_Reg(cardnum, PLX_DMA_HOST_ADDR1, (CEI_UINT32)dma_phy_addr);
    set_plx_Reg(cardnum,PLX_DMA_BOARD_ADDR1, board_addr);
    set_plx_Reg(cardnum,PLX_DMA_BYTE_COUNT1, byte_count);
    if(dma_flag) {  /* PCI bus to Local bus */
      set_plx_Reg(cardnum, PLX_DMA_FLAG1, 0x0);
      /* requesting 32 full entries in FIFO before transfer */
      or_plx_Reg(cardnum, PLX_DMA_THRESHOLD, 0x000F0000);
    }
    else {  /* Local bus to PCI bus */
      set_plx_Reg(cardnum, PLX_DMA_FLAG1, 0x8);      
      /* requesting 32 full entries in FIFO before transfer */
      or_plx_Reg(cardnum, PLX_DMA_THRESHOLD, 0x0F000000);
    }
  }
    
  return API_SUCCESS;
}


/****************************************************************************
*
*  PROCEDURE NAME -   BusTools_DMA_Transfer
*
*  FUNCTION
*       This function initiates a block DMA transfer on the specified DMA
*       channel that was configured using BusTools_DMA_Setup2. It waits until
*       the transfer is complete. 
*
*  RETURNS
*       API_SUCCESS (0) -> success
*       API_BUSTOOLS_BADCARDNUM -> bad card number specified
*       API_BAD_PARAM -> invalid DMA channel
*
****************************************************************************/
NOMANGLE BT_INT CCONV BusTools_DMA_Transfer(
  BT_UINT cardnum,     /* (i) card number                  */
  BT_UINT dma_channel) /* (i) 0 = channel 0, 1 = channel 1 */
{
  BT_U32BIT offset=0;
    
  if(cardnum >= MAX_BTA)
    return API_BUSTOOLS_BADCARDNUM;

  if(dma_channel > 1)
    return API_BAD_PARAM;
    
  if(dma_channel == 0)
    offset = PLX_DMA_STATUS0;
  else 
    offset = PLX_DMA_STATUS1;
    
  /* enable channel to transfer, start transfer */
  set_plx_Reg(cardnum, offset, 0x3);
  /* wait till channel done bit is "1", indicating that the transfer is complete */
  while((get_plx_Reg(cardnum, offset) & 0x10) == 0) 
    MSDELAY(1);
  
  /* disable DMA channel from transfering */
  set_plx_Reg(cardnum, offset, 0);
    
  return API_SUCCESS;
}


/****************************************************************************
*
*  PROCEDURE NAME -   BusTools_DMA_Init
*
*  FUNCTION
*       This function configures the device for block DMA transfer. User 
*       specifies the DMA channel, number of bytes to be transfered, and
*       the direction. Used in conjuction with BusTools_DMA_Read.
*
*  RETURNS
*       API_SUCCESS (0) -> success
*       API_HARDWARE_NOSUPPORT -> illegal address type specified
*       API_BUSTOOLS_BADCARDNUM -> bad card number specified
*       API_BUSTOOLS_NOTINITED  -> Channel not initialized
*       API_BAD_PARAM -> invalid DMA channel
*       BTD_ERR_PARAM -> invalid parameter (device or DMA channel)
*       BTD_ERR_INUSE -> DMA channel already in use
*       BTD_ERR_NOWINRT -> an invalid handle for the device
*       BTD_ERR_LOAD_CEIINST -> invalid pointer to CEI_Install DLL
*       BTD_ERR_NOACCESS -> device does not support DMA
*       BTD_ERR_BADEXTMEM -> invalid DMA address or size 
*
****************************************************************************/
NOMANGLE BT_INT CCONV BusTools_DMA_Init(
  BT_UINT cardnum,     /* (i) card number                             */
  BT_UINT dma_channel, /* (i) 0 = channel 0, 1 = channel 1            */
  BT_U32BIT size,      /* (i) number of bytes to transfer             */
  BT_U32BIT flag)      /* (i) direction of transfer 0 = board to host */
                       /*                           1 = host to board */
{
  BT_INT status=0;
 #ifdef WIN64
  CEI_UINT64 dma_usr_addr=0;
  CEI_UINT64 dma_phy_addr=0;
 #else
  CEI_UINT32 dma_usr_addr=0;
  CEI_UINT32 dma_phy_addr=0;
 #endif
    
  if(cardnum >= MAX_BTA)
    return API_BUSTOOLS_BADCARDNUM;
    
  if(bt_inited[cardnum] == 0)
    return API_BUSTOOLS_NOTINITED;
   
  if(!board_has_plx_dma[cardnum])
    return API_HARDWARE_NOSUPPORT;
    
  if(dma_channel > 1)
    return API_BAD_PARAM;
    
  /*  configure a DMA channel for the device  */
  if((status = vbtDMASetup(api_device[cardnum], CurrentCardSlot[cardnum], size, flag, &dma_phy_addr, &dma_usr_addr)) != BTD_OK)
    return status; 
    
  /*  Mode: 0x00000043 (default)  */
  if(dma_channel == 0) {
    /*  PCI Memory space where DMA transfers start  */
    set_plx_Reg(cardnum, PLX_DMA_HOST_ADDR0, (CEI_UINT32)dma_phy_addr); 
    /*  PCI bus to Local bus  */
    if(flag) {
      set_plx_Reg(cardnum, PLX_DMA_FLAG0, 0x0);
      /*  requesting 32 full entries in FIFO before transfer  */
      or_plx_Reg(cardnum, PLX_DMA_THRESHOLD, 0x0000000F);
    }
    else { // Local bus to PCI bus
      set_plx_Reg(cardnum, PLX_DMA_FLAG0, 0x8);
      /*  requesting 32 full entries in FIFO before transfer  */
      or_plx_Reg(cardnum, PLX_DMA_THRESHOLD, 0x00000F00);
    }
  }
  else {
    set_plx_Reg(cardnum, PLX_DMA_HOST_ADDR1, (CEI_UINT32)dma_phy_addr);
    if(flag) {
      set_plx_Reg(cardnum, PLX_DMA_FLAG1, 0x0);
      /*  requesting 32 full entries in FIFO before transfer  */
      or_plx_Reg(cardnum, PLX_DMA_THRESHOLD,0x000F0000);
    }
    else {
      set_plx_Reg(cardnum, PLX_DMA_FLAG1, 0x8);      
      /*  requesting 32 full entries in FIFO before transfer  */
      or_plx_Reg(cardnum, PLX_DMA_THRESHOLD, 0x0F000000);
    }
  }
    
  return API_SUCCESS;
}


/****************************************************************************
*
*  PROCEDURE NAME -   BusTools_DMA_Read
*
*  FUNCTION
*       This function initiates a block DMA transfer on a DMA channel that 
*       was configured using BusTools_DMA_Init. Specifiy the DMA channel,
*       the board address and the number of bytes to be transfered.
*
*  RETURNS
*       API_SUCCESS (0) -> success
*       API_HARDWARE_NOSUPPORT -> illegal address type specified
*       API_BUSTOOLS_BADCARDNUM -> bad card number specified
*       API_BUSTOOLS_NOTINITED  -> Channel not initialized
*       API_BAD_PARAM -> invalid DMA channel
*       API_MEM_ALLOC_ERR -> system failed to allocate memory
*       BTD_ERR_PARAM -> invalid parameter (device or DMA channel)
*       BTD_ERR_NOWINRT -> an invalid handle for the device
*       BTD_ERR_LOAD_CEIINST -> invalid pointer to CEI_Install DLL
*       BTD_ERR_NOACCESS -> device does not support DMA
*
****************************************************************************/
NOMANGLE BT_INT CCONV BusTools_DMA_Read(
  BT_UINT cardnum,     /* (i) card number                      */
  BT_UINT dma_channel, /* (i) 0 = channel 0, 1 = channel 1     */
  BT_UINT board_addr,  /* (i) board address for dma read/write */
  BT_UINT byte_count,  /* (i) number of bytes to transfer      */
  BT_UINT *buf)        /* (o) user buffer                      */
{
  BT_INT status=0;
  BT_UINT addr_offset=0;
  CEI_UINT flag=0;
    
  if(cardnum >= MAX_BTA)
    return API_BUSTOOLS_BADCARDNUM;
    
  if(bt_inited[cardnum] == 0)
    return API_BUSTOOLS_NOTINITED;
    
  if(!board_has_plx_dma[cardnum])
    return API_HARDWARE_NOSUPPORT;
    
  if(dma_channel > 1)
    return API_BAD_PARAM;
    
   /*  check that the count will not exceed the channel's RAM range  */
  if(byte_count > (0x000FFFFF - byte_count))
    return API_BAD_PARAM;
    
  /*  check for a valid address offset into the channel's RAM (1 MB)  */
  if(board_addr > 0x000FFFFF)
    return API_BAD_PARAM;
    
  /*  set the offset into the channel's RAM  */
  addr_offset = board_addr + (CurrentCardSlot[cardnum]*0x200000) + 0x100000;
    
  /*  program the board offset and size to the PLX registers  */
  if(dma_channel == 0) {
    set_plx_Reg(cardnum, PLX_DMA_BOARD_ADDR0, addr_offset);
    set_plx_Reg(cardnum,PLX_DMA_BYTE_COUNT0, byte_count);
  }
  if(dma_channel == 1) {
    set_plx_Reg(cardnum, PLX_DMA_BOARD_ADDR1, addr_offset);
    set_plx_Reg(cardnum,PLX_DMA_BYTE_COUNT1, byte_count);
    flag = 0x2; // set to DMA channel 1
  }

  if((status = vbtDMARead(api_device[cardnum], CurrentCardSlot[cardnum], addr_offset, byte_count, flag, buf)) != BTD_OK)
    return status; 
     
  return API_SUCCESS;
}


/****************************************************************************
*
*  PROCEDURE NAME -   BusTools_DMA_Close
*
*  FUNCTION
*       This function disables the specified DMA channel. 
*
*  RETURNS
*       API_SUCCESS (0)   -> success
*       API_HARDWARE_NOSUPPORT -> illegal address type specified
*       API_BUSTOOLS_BADCARDNUM -> bad card number specified
*       API_BUSTOOLS_NOTINITED  -> Channel not initialized
*       API_BAD_PARAM -> invalid DMA channel
*       BTD_ERR_PARAM -> invalid parameter (device or DMA channel)
*       BTD_ERR_NOWINRT -> an invalid handle for the device
*       BTD_ERR_LOAD_CEIINST -> invalid pointer to CEI_Install DLL
*       BTD_ERR_NOACCESS -> device does not support DMA
*
****************************************************************************/
NOMANGLE BT_INT CCONV BusTools_DMA_Close(
  BT_UINT cardnum,     /* (i) card number                   */
  BT_UINT dma_channel) /* (i) 0 = channel 0, 1 = channel 1 */
{
  BT_INT status=0;
    
  if(cardnum >= MAX_BTA)
    return API_BUSTOOLS_BADCARDNUM;
    
  if(bt_inited[cardnum] == 0)
    return API_BUSTOOLS_NOTINITED;
    
  if(!board_has_plx_dma[cardnum])
    return API_HARDWARE_NOSUPPORT;
    
  if(dma_channel > 1)
    return API_BAD_PARAM;
    
  if((status = vbtDMAClose(api_device[cardnum], CurrentCardSlot[cardnum])) != BTD_OK)
    return status; 
    
  return API_SUCCESS;
}
