/*============================================================================*
 * FILE:                        M E M _ V M I C . C
 *============================================================================*
 *
 *      COPYRIGHT (C) 1998-2016 BY ABACO SYSTEMS, INC. 
 *      ALL RIGHTS RESERVED.
 *
 *      THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND
 *      COPIED ONLY IN ACCORDANCE WITH THE TERMS OF SUCH LICENSE AND WITH
 *      THE INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY
 *      OTHER COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE
 *      AVAILABLE TO ANY OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF THE
 *      SOFTWARE IS HEREBY TRANSFERRED.
 *
 *      THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT
 *      NOTICE AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY ABACO SYSTEMS.
 *
 *===========================================================================*
 *
 * FUNCTION:   BusTools/1553-API Library:
 *             Low-level VMIC/WIN32 board memory mapping component.
 *             This module contains the routines which create the memory-mapped
 *             address pointer to the BusTools board.  Also included are the
 *             routines for freeing this pointer.
 *
 *             These functions only support 32-bit Windows.
 *
 *             All of the functions in this module are very Operating System
 *             and platform specific.  If you need support for a new OS
 *             or platform, start with BTDRV.C and work your way into this
 *             module.  Most of the functions in this module will need to be
 *             re-written from scratch.
 *
 * USER ENTRY POINTS:
 *    none.
 *
 * EXTERNAL NON-USER ENTRY POINTS: (only used by btdrv.c):
 *    vbtMapVMICAddress  - Map address under VMIC VME Manager Windows NT
 *    vbtFreeVMICAddress - Free address under VMIC
 *
 * INTERNAL ROUTINES:
 *    None.
 *
 *===========================================================================*/

/* $Revision:  4.58 Release $
   Date        Revision
  ----------   ---------------------------------------------------------------
  10/20/1999   Split off the 32-bit Windows functions from MEMMAP.C and placed
               them in this file.V3.22.ajh
  11/22/1999   Further split VMIC functions only into this file.V3.30.ajh
  01/27/2003   Add support for VME-1553 on a VMIC SBC.
 */

/*---------------------------------------------------------------------------*
 *                         INCLUDES, DEFINES and GLOBALS
 *---------------------------------------------------------------------------*/

#include <stdio.h>
#include "busapi.h"
#include "apiint.h"
#include "globals.h"
#include "btdrv.h"

#ifdef INCLUDE_VMIC 

#include <dos.h>
#include <stdlib.h>
#include <stdio.h>


/*---------------------------------------------------------------------------*
 *          LOCAL ROUTINES, NOT REFERENCED OUTSIDE OF THIS MODULE
 *---------------------------------------------------------------------------*/

/****************************************************************
*
*  PROCEDURE - vbtMapVMICAddress
*
*  FUNCTION
*     Setup the page access data elements for the specified IP1553
*     on the Greenspring IP carrier for VME bus (VIPC616).
*     We only support this carrier in A32 space using the VMIC
*     VME Manager Windows NT memory mapping calls
*
* DESCRIPTION
*     bt_PageAddr[cardnum][0] maps the board dual-port memory, and
*     bt_PageAddr[cardnum][1] points to the IO register.
*     bt_PageAddr[cardnum][2] points to the ID PROM space.
*     bt_PageAddr[cardnum][3] points to Interrupt Vector reg (not used)
*
*  RETURNS
*     0 if successful, non-zero upon error.
****************************************************************/

// Following are constants from the VMIC VME manager library "vmemgr.h" we use.
#define V_BIG_ENDIAN     123
#define V_DATA16           2
#define V_LEVEL_BR3        3
#define V_A16SD          0x2D     // 16 bit address as supervisor to data memory
#define V_A32SD          0x0D     // 32 bit address as supervisor to data memory
#define V_A32UD          0x09
#define V_A32UP          0x3A

static HINSTANCE hVMIC    = NULL;                 // Handle to vmemgr.dll
static int (WINAPI *vmeInit)(int,int) = NULL; // Pointer to vmeInit() func.
static int (WINAPI *vmeLockVmeWindow)(int,int,int,
        DWORD*,int,int,int,int,int,int) = NULL; // Ptr to vmeLockVmeWindow().
static int (WINAPI *vmeGetVmeWindowAdr)
                   (DWORD,DWORD,void *) = NULL; // Ptr to vmeGetVmeWindowAdr().
static int (WINAPI *vmeFreeVmeWindowAdr)
                    (DWORD,DWORD,void*) = NULL; // Ptr to vmeFreeVmeWindowAdr().
static int (WINAPI *vmeUnlockVmeWindow)
                          (DWORD,DWORD) = NULL; // Ptr to vmeUnlockVmeWindow().
static int (WINAPI *vmeTerm)(int)   = NULL; // Ptr to vmeTerm().
static HANDLE hDevHandle[MAX_BTA][2];  // Handles to VMIC devices.

BT_INT vbtMapVMICAddress(BT_UINT cardnum, BT_U32BIT baseaddr,
                         BT_INT  ioaddress)
{
   /*******************************************************************
   *  Local variables
   *******************************************************************/
   DWORD      status;         // Function return status.
   DWORD      WinHandle;      // Handle to mapping functions.
   DWORD      NumElements;
   
   if ( ((CurrentCarrier[cardnum] == VIPC616_618) && (CurrentCardType[cardnum] == IPD1553))||
	    ((CurrentCardType[cardnum] == VME1553)) || 
        ((CurrentCardType[cardnum] ==	QVME1553)))
	  status = 0;
   else
      return BTD_NON_SUPPORT; // Bus/Carrier/OS combination not supported

   /*******************************************************************
   *  Setup the VME base address of the dual-port memory based on the
   *   slot.  We only support a 32 Mb mapping in A32 space.
   *******************************************************************/
   if ( hVMIC == NULL )
   {
       hVMIC = LoadLibrary("vmemgr");  // Search standard path for DLL
       if ( hVMIC == NULL )
       {
          wsprintf(szMsg, "Cannot load VMEMGR.DLL due to system error"
                   " %X ", GetLastError());
          MessageBox(NULL, szMsg, "Abaco", MB_OK | MB_SYSTEMMODAL);
          return BTD_ERR_NOACCESS;
       }

       vmeInit = (void *)GetProcAddress(hVMIC, "vmeInit");
       if ( vmeInit == NULL )
       {
          wsprintf(szMsg, "Cannot locate vmeInit() in VMEMGR.DLL due to error"
                  " %X ", GetLastError());
          MessageBox(NULL, szMsg, "Abaco", MB_OK | MB_SYSTEMMODAL);
          return BTD_ERR_NOACCESS;
       }

       vmeLockVmeWindow    = (void *)GetProcAddress(hVMIC, "vmeLockVmeWindow");
       vmeGetVmeWindowAdr  = (void *)GetProcAddress(hVMIC, "vmeGetVmeWindowAdr");
       vmeFreeVmeWindowAdr = (void *)GetProcAddress(hVMIC, "vmeFreeVmeWindowAdr");
       vmeUnlockVmeWindow  = (void *)GetProcAddress(hVMIC, "vmeUnlockVmeWindow");
       vmeTerm             = (void *)GetProcAddress(hVMIC, "vmeTerm");
   }

   // Initialize the VMIC VME library

   status = vmeInit(0,0);
   if ( status )
   {
      wsprintf(szMsg, "vmeInit call returned %lX as the OS or VME Manager"
               " error code\n", status);
      MessageBox(NULL, szMsg, "vbtPageAccessSetupVME_IP", MB_OK | MB_SYSTEMMODAL);
      return status;
   }

   // Lock the VME memory-mapped memory range into local space.
   status = vmeLockVmeWindow(0,               // DWORD Reserved,
                             V_DATA16,        // DWORD Size = 2 bytes,
                             baseaddr,        // DWORD Adr,
                             &WinHandle,      // DWORD *Handle,
                             0x400000,        // DWORD NumElements(4M*2 = 8 Mb),
                             V_A32UD,         // DWORD AccessMode,
                             V_BIG_ENDIAN,    // DWORD Endian,
                             V_LEVEL_BR3,     // DWORD RequestLevel,
                             100,             // DWORD Timeout
                             0                // DWORD Flags);
                            );

   if ( status )
   {
      if ( status == ERROR_CALL_NOT_IMPLEMENTED )
      {
         wsprintf(szMsg, "vmeLockVmeWindow is not implemented on this"
                  " hardware platform\n");
         MessageBox(NULL, szMsg, "vbtPageAccessSetupVME_IP", MB_OK | MB_SYSTEMMODAL);
      }
      else
      {
         wsprintf(szMsg, "vmeLockVmeWindow call returned %lX as the"
                  " OS or VME Manager error code\n", status);
         MessageBox(NULL, szMsg, "vbtPageAccessSetup_VME_IP", MB_OK | MB_SYSTEMMODAL);
      }
      return status;
   }

   // Get the local address of the mapped VME space
   hDevHandle[cardnum][0] = (HANDLE)WinHandle;
   status = vmeGetVmeWindowAdr(0,                       // DWORD Reserved,
                               WinHandle,               // DWORD WinHandle,
                               (void *)&bt_PageAddr[cardnum][0]  // void  **Adr
                              );
   if ( status )
   {
       wsprintf(szMsg, "vmeGetVmeWindowAdr call returned %lX as the"
                " OS or VME Manager error code\n", status);
       MessageBox(NULL, szMsg, "vbtPageAccessSetupVME_IP", MB_OK | MB_SYSTEMMODAL);
       return status;
   }

   /*******************************************************************
   *  Setup the VME address of the IO register, based on the slot
   *  number. Lock the VME IO-mapped memory range into local space.
   *******************************************************************/

   NumElements = 32;

   status =
   vmeLockVmeWindow(0,               // DWORD Reserved,
                    V_DATA16,        // DWORD Size,
                    ioaddress,       // DWORD Adr,
                    &WinHandle,      // DWORD *Handle,
                    NumElements,     // DWORD NumElements (512*2 = 1024 bytes),
                    V_A16SD,         // DWORD AccessMode,
                    V_BIG_ENDIAN,    // DWORD Endian,
                    V_LEVEL_BR3,     // DWORD RequestLevel,
                    100,             // DWORD Timeout
                    0);              // DWORD Flags);
   if ( status )
   {
       if ( status == ERROR_CALL_NOT_IMPLEMENTED )
       {
           wsprintf(szMsg, "vmeLockVmeWindow is not implemented on this"
                     " hardware platform\n");
           MessageBox(NULL, szMsg, "vbtPageAccessSetupVME_IP", MB_OK | MB_SYSTEMMODAL);
       }
       else
       {
           wsprintf(szMsg, "vmeLockVmeWindow call returned %lX as the"
                     " OS or VME Manager error code\n", status);
           MessageBox(NULL, szMsg, "vbtPageAccessSetupVME_IP", MB_OK | MB_SYSTEMMODAL);
       }
       return status;
   }

   // Get the local address of the mapped VME space
   hDevHandle[cardnum][1] = (HANDLE)WinHandle;
   status = vmeGetVmeWindowAdr(0,                        // DWORD Reserved,
                               WinHandle,                // DWORD WinHandle,
                               (void *)&bt_PageAddr[cardnum][1]); // void  **Adr
   if ( status )
   {
      wsprintf(szMsg, "vmeGetVmeWindowAdr call returned %lX as the"
               " OS or VME Manager error code\n", status);
      MessageBox(NULL, szMsg, "vbtPageAccessSetupVME_IP", MB_OK | MB_SYSTEMMODAL);
      return status;
   }

   return BTD_OK;
}

/****************************************************************
*
*  PROCEDURE - vbtFreeVMICAddress(base_address, map_length)
*
*  FUNCTION
*     This routine frees the VMIC resources which were obtained
*     in the mapping of the phyical board address.
*
*  RETURNS
*     Nothing
****************************************************************/

DWORD vbtFreeVMICAddress(BT_UINT cardnum)
{
   /*******************************************************************
   *  Release the VMIC VME Manager memory mapping resources.
   *******************************************************************/
   DWORD      status;
   int        pagenum;        // page pointers to release.
   
   for ( pagenum = 0; pagenum < 2; pagenum++ )
   {
      // Free the VMIC pointers
      status = vmeFreeVmeWindowAdr(
                    0,                                   // DWORD Reserved,
                    (DWORD)hDevHandle[cardnum][pagenum], // DWORD Handle,
                    bt_PageAddr[cardnum][pagenum]);      // void  **Adr
      if ( status )
      {
         wsprintf(szMsg, "vmeFreeVmeWindowAdr() call returned %lX as"
                         " the OS or VME Manager error code\n", status);
         MessageBox(NULL, szMsg, "vbtShutdown", MB_OK | MB_SYSTEMMODAL);
      }
      // Unlock the hardware window
      status = vmeUnlockVmeWindow(
                    0,                                    // DWORD Reserved,
                    (DWORD)hDevHandle[cardnum][pagenum]); // DWORD Handle,
      if ( status )
      {
         wsprintf(szMsg, "vmeUnlockVmeWindow call returned %lX as the"
                         " OS or VME Manager error code\n", status);
         MessageBox(NULL, szMsg, "vbtShutdown", MB_OK | MB_SYSTEMMODAL);
      }
   }
   // We're done with the library
   // vmeTerm(0);
   return status;
}
#endif
