/*============================================================================*
 * FILE:                        U S R S E T U P . C
 *============================================================================*
 *
 *      COPYRIGHT (C) 1998-2016 BY ABACO SYSTEMS, INC. 
 *      ALL RIGHTS RESERVED.
 *
 *      THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND
 *      COPIED ONLY IN ACCORDANCE WITH THE TERMS OF SUCH LICENSE AND WITH
 *      THE INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY
 *      OTHER COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE
 *      AVAILABLE TO ANY OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF THE
 *      SOFTWARE IS HEREBY TRANSFERRED.
 *
 *      THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT
 *      NOTICE AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY ABACO SYSTEMS.
 *
 *===========================================================================*
 *
 * FUNCTION:   BusTools/1553-API Library:
 *             User board addressing setup and user DLL interface component.
 *
 *             This module performs the setup of the address pointers for all
 *             supported BusTools boards, using user-supplied mapped addresses.
 *             This file is used in environments which use a user-supplied DLL
 *             to compute and return the board mapping address(es).
 *
 *             This module also opens the user interface DLL (if present) and
 *             maps all of the user-defined entry points.  These entry points
 *             are called by the API whenever the associated API entry point
 *             is called.  This gives the user "hooks" into the operation of
 *             the API, without having to directly modify the API source code
 *             and re-compile the busapi32.dll.
 *
 * DESCRIPTION:  The user-specified addressing DLL is first loaded, then the
 *               user entry point "BusTools_API_Init_External" is called to
 *               map the board addresses.  If the call is to unmap addresses,
 *               the DLL must have been already loaded by a previous call.
 *
 *               The names of the user interface DLLs are hard-coded into this
 *               function.  vbtMapUserDLL opens them in sequence; later DLL entry
 *               points replace previously defined entry points.  For any one
 *               function only one user interface DLL function will be called.
 *
 * API ENTRY POINTS:
 *    vbtDisplayWin32Error   Display the text string associated with a WIN32
 *                           error code returned by GetLastError().
 *    vbtMapUserBoardAddress Setup adapter page access data pointers for
 *                           specified BusTools board using the specified
 *                           user addressing DLL.
 *    vbtMapUserDLL          Load the user interface DLL (if present)
 *
 *===========================================================================*/

/* $Revision:  6.20 Release $
   Date        Revision
  ----------   ---------------------------------------------------------------
  08/10/1999   Added user board address setup function interface.V3.20.ajh
  06/21/2000   Added mapping of the user interface DLL.V4.06.ajh
  02/15/2002   Added modular API support v4.48
 */

/*---------------------------------------------------------------------------*
 *                    INCLUDES, DEFINES and GLOBALS
 *---------------------------------------------------------------------------*/

#include <stdio.h>
#include "busapi.h"
#include "apiint.h"
#include "globals.h"
#include "btdrv.h"
#include <stdlib.h>
#include <string.h>

#ifdef _USER_INIT_
 /*---------------------------------------------------------------------------*
 *      Pointers to the user addressing DLL and it's internal functions.
 *---------------------------------------------------------------------------*/

// Handles to user addressing DLLs.
static HINSTANCE hUSER[MAX_BTA] = {NULL,NULL,NULL,NULL};
// Pointer to BusTools_API_InitExternal() function in user DLL.
static int (WINAPI *pBusTools_API_InitExternal[MAX_BTA])(
   BT_UINT   cardnum,       // (i) card number (0 - based)
   BT_UINT   wOpen,         // (i) 1->open device, 0->close device
   BT_U32BIT phys_addr,     // (i) base address of board/carrier memory area
   BT_U32BIT wIOaddress,    // (i) board/carrier io address
   BT_UINT   cardType,      // (i) PCI1553, PMC1553, ISA1553, or .....
   BT_UINT   carrier,       // (i) NATIVE.
   BT_UINT   slot,          // (i) SLOT_A, SLOT_B, SLOT_C, SLOT_D, etc.
   char * PageAddr[4]);     // (o) array of board addresses.

/*===========================================================================*
 * API ENTRY POINT:    v b t D i s p l a y W i n 3 2 E r r o r
 *===========================================================================*
 *
 * FUNCTION:    Call the Win32 function to translate an error number into
 *              a display string.
 *
 * DESCRIPTION: Call the FormatMessage Function.
 *
 *      It will return:
 *         nothing.
 *===========================================================================*/
static void vbtDisplayWin32Error(
   DWORD lastError,         // (i) - The Windows error code to be displayed
   char * szHeading)        // (i) - Pointer to a massage to include in the display
{
   LPVOID lpMsgBuf;

   // Ask the system what the error means.
   FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
                 NULL,
                 lastError,
                 MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
                 (LPTSTR) &lpMsgBuf,
                 0,
                 NULL);

   // Display the string.
   MessageBox(NULL, lpMsgBuf, (LPCTSTR)szHeading, MB_OK | MB_SYSTEMMODAL | MB_ICONHAND);

    // Free the buffer.
    LocalFree(lpMsgBuf);
}

/*===========================================================================*
 * API ENTRY POINT:    v b t M a p U s e r B o a r d A d d r e s s
 *===========================================================================*
 *
 * FUNCTION:    Call the user-supplied memory mapping function
 *              BusTools_API_InitExternal.  First we must load the DLL that
 *              contains this function.  It's name is given in the global
 *              "bt_UserDLLName[cardnum]".
 *              Then call the BusTools_API_InitExternal() function.  Save the
 *              return memory mapping pointers, and return to setup the board.
 *
 * DESCRIPTION: Call BusTools_API_InitExternal() to setup the following board
 *              access pointers, for each board type:
 *              This functions must setup the bt_PageAddr[] access pointers and
 *              return a value of BTD_OK, as listed below for each board type:
 *
 *
 *           Standard board setup:
 *              bt_PageAddr[0] maps the board base address
 *              bt_PageAddr[1] points to the Hardware registers
 *              bt_PageAddr[2] points to the RAM registers
 *              bt_PageAddr[3] points to the Host Interfae Registers
 *
 *           PCCD-1553:
 *              bt_PageAddr[0] maps the board dual-port RAM
 *              bt_PageAddr[1] points to the Hardware registers
 *              bt_PageAddr[2] points to the RAM registers
 *              bt_PageAddr[3] points to the Host Interfae Registers
 *
 *      It will return:
 *              BTD_OK if successful, non-zero upon error.
 *===========================================================================*/
BT_INT vbtMapUserBoardAddress(
   BT_UINT cardnum,          // (i) - Card number to open/close
   BT_INT wOpenFlag,         // (i) - 1-> open specified card, 0-> close card
   BT_U32BIT phys_addr,      // (i) - physical base address from _Init() call
   BT_U32BIT io_addr,        // (i) - physical I/O address from _Init() call
   char * UserDLLName)       // (i) - Pointer to the name of the mapping DLL
{
   /*******************************************************************
   *  Local variables
   *******************************************************************/
   int      status;                       // Function return status.
   static   BT_U32BIT SaveMemAddr[MAX_BTA];
   static   BT_U32BIT SaveIOAddr[MAX_BTA];

   /*******************************************************************
   *  Given the DLL name, load the user-specified DLL, and look up
   *   the addresses of the functions we call.
   *******************************************************************/

   if ( hUSER[cardnum] < (HINSTANCE)HINSTANCE_ERROR )
   {
      // Load the user-specified DLL file.
      hUSER[cardnum] = LoadLibrary((LPCTSTR)UserDLLName);
      if ( hUSER[cardnum] < (HINSTANCE)HINSTANCE_ERROR )
      {
         int    localError = GetLastError();

         vbtDisplayWin32Error(localError,
                              "vbtMapUserBoardAddress()/LoadLibrary()");

         wsprintf(szMsg, "Cannot load user dll \'%s\' due to system error"
                  " %X ", (LPCTSTR)UserDLLName, localError);
         MessageBox(NULL, (LPCTSTR)szMsg, "BusTools/1553-API", MB_OK | MB_SYSTEMMODAL | MB_ICONHAND);
         return BTD_ERR_NOACCESS;
      }
      // Now get the address of the function we need from the DLL.
      pBusTools_API_InitExternal[cardnum] =
           (void *)GetProcAddress(hUSER[cardnum], "BusTools_API_InitExternal");
      if ( pBusTools_API_InitExternal[cardnum] == NULL )
      {
         int    localError = GetLastError();

         vbtDisplayWin32Error(localError,
                              "vbtMapUserBoardAddress()/GetProcAddress()");

         wsprintf(szMsg, "Cannot locate BusTools_API_InitExternal() in \'%s\' due to error"
                  " %X ", (LPCTSTR)UserDLLName, localError);
         MessageBox(NULL, (LPCTSTR)szMsg, "BusTools/1553-API", MB_OK | MB_SYSTEMMODAL | MB_ICONHAND);
         return BTD_ERR_NOACCESS;
      }
   }

   /*********************************************************************
   *  Given the HW base addresses, call the user-supplied function to
   *   map the addresses into host address space.  This function must
   *   perform any other initialization specific to the board or carrier
   *   that needs to be performed.
   *********************************************************************/
   if ( wOpenFlag )
   {
      SaveMemAddr[cardnum] = phys_addr;   // Save for the close
      SaveIOAddr [cardnum] = io_addr;
   }

   status = pBusTools_API_InitExternal[cardnum](
               cardnum,                  // (i) card number (0 - based)
               wOpenFlag,                // (i) 1->open card, 0->close card
               SaveMemAddr[cardnum],     // (i) base address of board/carrier
               SaveIOAddr[cardnum],      // (i) board/carrier io address
               CurrentCardType[cardnum], // (i) PCI1553, PMC1553 or ISA1553
               CurrentCarrier[cardnum],  // (i) As supplied by caller.
               CurrentCardSlot[cardnum], // (i) SLOT_A, SLOT_B, SLOT_C, SLOT_D, etc.
               bt_PageAddr[cardnum]);    // (o) array of board addresses.

   return status;
}
#endif

/*===========================================================================*
 * API ENTRY POINT:    B u s T o o l s _ A P I _ L o a d U s e r D L L
 *===========================================================================*
 *
 * FUNCTION:   This function opens the specified user interface DLL and
 *             maps all of the user-defined entry points.  The API calls
 *             these entry points whenever the associated API entry point
 *             is called.  This gives the user "hooks" into the operation of
 *             the API, without having to directly modify the API source code
 *             and re-compile the busapi32.dll.
 *
 * DESCRIPTION:  The names of the user interface DLLs are hard-coded into this
 *               function.  vbtMapUserDLL opens them in sequence; later DLL
 *               entry points replace previously defined entry points.
 *               For any one API function only one user interface DLL function
 *               will be called.
 *               The setup of the user interface DLLs is only done once; all
 *               cardnum's share the same set of user interface DLLs.
 *
 * It will return:
 *       API_SUCCESS       - User DLL opened successfully.
 *       API_NO_OS_SUPPORT - User DLLs not supported, or other system error
 *       API_CANT_LOAD_USER_DLL - LoadLibrary() function failed
 *       API_BUSTOOLS_NOTINITED - Init will wipe out the user DLL function list
 *       API_BUSTOOLS_BADCARDNUM
 *===========================================================================*/

NOMANGLE BT_INT CCONV BusTools_API_LoadUserDLL(
   BT_UINT cardnum,                // (i) - Card number to open/close
   const char * szDLLName)     // (i) - Pointer to the comma-delimited list of DLL names
{
#ifdef _USER_DLL_
   /************************************************
   *  local variables
   ************************************************/
   static char szWhiteSpace[] = {" ,;\t"};
   HINSTANCE  hUSERdll;       // Handle to user interface DLLs
   void      *pFunction;      // Pointer to the user function
   char  *pDLLName;       // Pointer to the local DLL name string
   char  *p;              // Pointer to the parsed DLL name string

   /************************************************
   *  Check initial conditions
   ************************************************/
   if ( cardnum >= MAX_BTA )
      return API_BUSTOOLS_BADCARDNUM;

   if ( szDLLName == NULL )
      return API_SUCCESS;     // Ignore if DLL name is a null pointer

   if ( strlen(szDLLName) == 0 )
      return API_SUCCESS;     // Ignore if DLL name is null

   /***********************************************************************
   * Allocate and copy the DLL name string so we can parse it.
   ***********************************************************************/
   pDLLName = malloc(strlen(szDLLName));
   if ( pDLLName == NULL )
      return API_NO_OS_SUPPORT;
   strcpy(pDLLName, szDLLName);
   
   /***********************************************************************
   *  Loop through the supplied list of user interface DLLs.  Load each
   *   one and lookup the entry points.
   ***********************************************************************/
   p = strtok(pDLLName, szWhiteSpace);
   while ( p != NULL )
   {
      // Load the user-specified DLL file.
      hUSERdll = LoadLibrary((LPCTSTR)p);
      if ( hUSERdll < (HINSTANCE)HINSTANCE_ERROR )
         return API_CANT_LOAD_USER_DLL;        // Can't load the DLL

      // Now get the address of the functions we need from the DLL.
      // Look up each function and save its address if it exists.
      pFunction = GetProcAddress(hUSERdll, "UsrAPI_Close");
      if ( pFunction ) pUsrAPI_Close[cardnum] = pFunction;

      pFunction = GetProcAddress(hUSERdll, "UsrBC_MessageAlloc");
      if ( pFunction ) pUsrBC_MessageAlloc[cardnum] = pFunction;

      pFunction = GetProcAddress(hUSERdll, "UsrBC_MessageRead");
      if ( pFunction ) pUsrBC_MessageRead[cardnum] = pFunction;

      pFunction = GetProcAddress(hUSERdll, "UsrBC_MessageUpdate");
      if ( pFunction ) pUsrBC_MessageUpdate[cardnum] = pFunction;

      pFunction = GetProcAddress(hUSERdll, "UsrBC_MessageWrite");
      if ( pFunction ) pUsrBC_MessageWrite[cardnum] = pFunction;

      pFunction = GetProcAddress(hUSERdll, "UsrBC_StartStop");
      if ( pFunction ) pUsrBC_StartStop[cardnum] = pFunction;

      pFunction = GetProcAddress(hUSERdll, "UsrBM_MessageAlloc");
      if ( pFunction ) pUsrBM_MessageAlloc[cardnum] = pFunction;

      pFunction = GetProcAddress(hUSERdll, "UsrBM_MessageRead");
      if ( pFunction ) pUsrBM_MessageRead[cardnum] = pFunction;

      pFunction = GetProcAddress(hUSERdll, "UsrBM_StartStop");
      if ( pFunction ) pUsrBM_StartStop[cardnum] = pFunction;

      pFunction = GetProcAddress(hUSERdll, "UsrRT_CbufWrite");
      if ( pFunction ) pUsrRT_CbufWrite[cardnum] = pFunction;

      pFunction = GetProcAddress(hUSERdll, "UsrRT_MessageRead");
      if ( pFunction ) pUsrRT_MessageRead[cardnum] = pFunction;

      pFunction = GetProcAddress(hUSERdll, "UsrRT_StartStop");
      if ( pFunction ) pUsrRT_StartStop[cardnum] = pFunction;

      // Point to the next DLL in the list, if any.
      p = strtok(NULL, szWhiteSpace);
   }
   free(pDLLName);
   return API_SUCCESS;
#else
   return API_NO_BUILD_SUPPORT;
#endif //_USER_DLL_
}


