/*============================================================================*
 * FILE:                        W I N 3 2 B O A R D S E T U P . C
 *============================================================================*
 *
 *      COPYRIGHT (C) 1998-2019 BY ABACO SYSTEMS, INC. 
 *      ALL RIGHTS RESERVED.
 *
 *      THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND
 *      COPIED ONLY IN ACCORDANCE WITH THE TERMS OF SUCH LICENSE AND WITH
 *      THE INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY
 *      OTHER COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE
 *      AVAILABLE TO ANY OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF THE
 *      SOFTWARE IS HEREBY TRANSFERRED.
 *
 *      THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT
 *      NOTICE AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY ABACO SYSTEMS.
 *
 *===========================================================================*
 *
 * FUNCTION:   BusTools/1553-API Library:
 *             This module contains	WIN32 specific intialization, and
 *             memory access functions for the BusTools Application
 *             Programmer's Interface (API).
 *
 *             The first API call must be to BusTools_API_OpenChannel() or
 *             BusTools_API_InitExtended() initialization functions.
 *             These routines call the low-level board access setup
 *             functions that map the specified board and makes it ready for
 *             access
 *
 * USER ENTRY POINTS: 
 *
 *     BusTools_API_OpenChannel  - Initializes the selected device and channel 
 *     BusTools_GetDevInfo       - Reteives device information
 *
 * Internal Routines:
 *
 *     vbtBoardAccessSetup       - Setup the specific board mapping
 *===========================================================================*/

/* $Revision:  8.28 Release $
   Date        Revision
  ----------   ---------------------------------------------------------------
  12/07/2006   Initial version.
  11/15/2007   Moved implementations of vbtOpen1553Channel, BusTools_FindDevice, 
               and BusTools_ListDevices into lowlevel.c. skb
  05/12/2008   Add support for the R15-AMC
  06/29/2009   Add support for the RXMC-1553
  12/10/2010   Add 64-Bit Windows support.
  12/13/2010   Add BusTools_CreateIntFifo, BusTools_UpdateIntFifo,and 
               BusTools_UpdateTailPTR to support C# wrappers.
  01/26/2011   Add BusTools_SetdumpPath to this O/S specific file
  04/11/2011   Add support for LPCIe-1553 and RXMC2-1553. added
                CEI_GET_ENV_VAR and CEI_SET_ENV_VAR. bch
  05/14/2012   Add support for R15-USB 
  09/07/2016   Added support for new hardware MPCIE
  11/16/2017   Changes to address warnings when using VS9 Wp64 compiler option.
  09/29/2019   Changed DEV_INFO to DEVINFO_T in BusTools_GetDevInfo
*/

#include <stdio.h>
#include <time.h>
#include <math.h>
#include <sys\stat.h>
#include "busapi.h"
#include "apiint.h"
#include "globals.h"
#include "btdrv.h"


/****************************************************************************
*
*  PROCEDURE NAME - BusTools_API_OpenChannel
*
*  FUNCTION
*     This routine initializes the channel on the specified device.
*
****************************************************************************/
NOMANGLE BT_INT CCONV BusTools_API_OpenChannel(
  BT_UINT   *chnd,       // (o) card number (0 - 12) (device number in 32-bit land)
  BT_UINT   mode,        // (i) operational mode
  BT_INT    devid,       // (i) linear base address of board/carrier memory area or WinRT device number
  BT_UINT   channel)     // (i) channel 1 - 4.
{
   BT_INT status, cardnum;

   //clear the assigned cardnum table on initial start
   if(assigned_cards[MAX_BTA] != 0xbeef)
   {
       assigned_cards[MAX_BTA] = 0xbeef;
       for(cardnum = 0;cardnum<MAX_BTA;cardnum++)
          assigned_cards[cardnum] = 0;
   }
   
   if(status = vbtOpen1553Channel(chnd,mode,devid,channel))
      return status; 

   return( API_Init(*chnd, devid, 0, &mode));
}


/****************************************************************************
*
*  PROCEDURE NAME - BusTools_API_OpenDeivceChannel
*
*  FUNCTION
*     This routine initializes the channel on the specified device.
*
****************************************************************************/
NOMANGLE BT_INT CCONV BusTools_API_OpenDeviceChannel(
  BT_UINT   *chnd,       // (o) card number (0 - 12) (device number in 32-bit land)
  BT_UINT   mode,        // (i) operational mode
  BT_INT    devid,       // (i) linear base address of board/carrier memory area or WinRT device number
  BT_UINT   channel)     // (i) channel 1 - 4.
{
   BT_INT status, cardnum, btype;
   DEVINFO_T dev_info;
   static UINT QdevID[] = {QPMC1553,0,QPCI1553,0,Q1041553P,0x0,QCP1553,QPMC1553,R15EC,R15AMC,RXMC1553,RPCIe1553,0x0,R15XMC2,LPCIE1553,0,0,RAR15XMCXT,R15PMC,MPCIE1553};

   //clear the assigned cardnum table on initial start
   if(assigned_cards[MAX_BTA] != 0xbeef)
   {
       assigned_cards[MAX_BTA] = 0xbeef;
       for(cardnum = 0;cardnum<MAX_BTA;cardnum++)
          assigned_cards[cardnum] = 0;
   }
   
   for(cardnum = 0; cardnum<MAX_BTA; cardnum++)
   {
      if(assigned_cards[cardnum] == 0)
      {
         assigned_cards[cardnum] = 0xff;
         break;
      }
   }
   if(cardnum == MAX_BTA)
   return API_MAX_CHANNELS_INUSE;
   *chnd = cardnum;

   status = vbtGetCEIDevInfo(devid,     // device to open (0 - 15)
                     &dev_info);        // pointer to device info structure
   if(status)
      return status;

   btype = (dev_info.host_interface & 0x3e)>>1;
   CurrentCardType[cardnum] = QdevID[btype-1];

   return (API_Init(*chnd, devid, 0, &mode));
}


/**************************************************************************
*
*  PROCEDURE NAME - BusTools_GetDevInfo
*
*  FUNCTION
*     This function gets the basic board information without requiring the
*     board to be intialized.
*
**************************************************************************/

NOMANGLE BT_INT CCONV BusTools_GetDevInfo(
   BT_UINT device,  // the device number of the VME A16 address
   DEVICE_INFO *pInfo)
{
   int status=0;

   if(device > MAX_BTA) //This is a VME A16 address
   {
      static int (WINAPI *p_vbtGetCEIVMEDevInfo)(
      BT_UINT,        // (i) card number (0 - based)
      DEVICE_INFO *);     // (o) array of board addresses.
      
      static HMODULE hUSER;
      DEVICE_INFO devinfo;

	  memset(&devinfo, 0, sizeof(DEVICE_INFO));
     
      // Load the required user-specified DLL
      hUSER = LoadLibrary((LPCTSTR)"BTVXIMAP.DLL");
      if ( hUSER < (HINSTANCE)HINSTANCE_ERROR )
         return BTD_ERR_NOACCESS;

      if((p_vbtGetCEIVMEDevInfo = (void *)GetProcAddress(hUSER, "vbtGetCEIVMEDevInfo")) == NULL )
         status = BTD_ERR_NOACCESS;
	  else {
         if((status = p_vbtGetCEIVMEDevInfo(device, &devinfo)) == API_SUCCESS)
	        memcpy(pInfo, &devinfo, sizeof(DEVICE_INFO));
      }

      FreeLibrary(hUSER);
   }
   else
   {
      DEVINFO_T devinfo;

      memset(&devinfo, 0, sizeof(devinfo));
      if((status = vbtGetCEIDevInfo(device, &devinfo)) == API_SUCCESS) {
        pInfo->busType = devinfo.busType;
		pInfo->nchan = devinfo.nchan;
        pInfo->irig = devinfo.irig;
        pInfo->mode = devinfo.mode;
        pInfo->memSections = devinfo.memSections;
		pInfo->VendorID = devinfo.VendorID;
        pInfo->DeviceID = devinfo.DeviceID;
        pInfo->CSCReg = devinfo.host_interface;
	  }
   }

   return status;
}

/*===========================================================================*
 * LOCAL ENTRY POINT:      v b t B o a r d A c c e s s S e t u p
 *===========================================================================*
 *
 * FUNCTION:    Setup specified adapter for read/write access.
 *
 * DESCRIPTION: Initialize the page access data elements and map the board
 *              into the host address space..
 *
 *      It will return:
 *              0 if successful, non-zero upon error.
 *===========================================================================*/

EXTERN BT_INT vbtBoardAccessSetup(
   BT_UINT   cardnum,       // (i) card number (0 based)
   BT_U32BIT phys_addr)
{
   BT_INT     status;
   char      *hostaddr=NULL;        // Mapped address of the board in host space.

   /*********************************************************************
   *  First verify that this is a board type we can handle.
   *********************************************************************/
   
   if ( CurrentPlatform[cardnum] == PLATFORM_PC )
   {
      if(CurrentCardType[cardnum] == R15USB)
         hostaddr = 0x0;
     /******************************************************************
      *  Get addresses for standard 32-bit Environments
      ******************************************************************/
      else if( (status = vbtMapBoardAddresses( api_device[cardnum],&bt_devmap[api_device[cardnum]])) != BTD_OK)
         return status;
         
      if(bt_devmap[api_device[cardnum]].memSections == 1)
      {
         hostaddr = (char *)bt_devmap[api_device[cardnum]].memHostBase[0];
      }
      else if (bt_devmap[api_device[cardnum]].memSections == 2)
      {
         if ( (bt_devmap[api_device[cardnum]].memLengthBytes[0] != 0x80)  &&
              (bt_devmap[api_device[cardnum]].memLengthBytes[0] != 0x200) && 
              (bt_devmap[api_device[cardnum]].memLengthBytes[0] != 0x1000) )
            return BTD_ERR_NOACCESS;

         if ( bt_devmap[api_device[cardnum]].memLengthBytes[1] != 0x00800000 )
            return BTD_ERR_NOACCESS;
         
         hostaddr = (char *)bt_devmap[api_device[cardnum]].memHostBase[1];
         bt_iobase[cardnum] = bt_devmap[api_device[cardnum]].memHostBase[0];
      }
      boardBaseAddress[cardnum] = hostaddr;
   } 
#ifdef _USER_INIT_
   /*********************************************************************
   *  Map user defined 32-Bit Window Environment.
   *********************************************************************/
   else if ( CurrentPlatform[cardnum] == PLATFORM_USER )
   {
      // sanity-check to make sure we can represent a pointer as a BT_U32BIT
      if (sizeof(BT_U32BIT) != sizeof(bt_iobase[cardnum]))
         return API_NO_BUILD_SUPPORT;

      /******************************************************************
      *  Get the addresses for User-Specified environments.  Call the
      *   user supplied function in the user specified DLL.
      ******************************************************************/
      status = vbtMapUserBoardAddress(cardnum, 1, phys_addr, (BT_U32BIT)((CEI_UINT64)bt_iobase[cardnum]), /* extraneous CEI_UINT64 cast for Wp64 warning */ 
                                      bt_UserDLLName[cardnum]);

      if ( status  != BTD_OK )
         return status;  // Cannot map the board.

      hostaddr           = bt_PageAddr[cardnum][0];
      bt_iobase[cardnum] = bt_PageAddr[cardnum][1];
      boardBaseAddress[cardnum] = bt_PageAddr[cardnum][1];
   }
#endif

#ifdef _LVRT_
   else if ( CurrentPlatform[cardnum] == PLATFORM_LVRT )
   {
      /******************************************************************
      *  Here we map a VMIC NT processor (x86 compatible) under NT.
      ******************************************************************/
      if ( (status = vbtMapVISAaddress(cardnum,
                                       api_device[cardnum],
                                       &hostaddr,
                                       &MemoryMapSize[cardnum],
									   CurrentMemMap[cardnum])) != BTD_OK )
         return status;  // Cannot map the board.
      boardBaseAddress[cardnum] = hostaddr;
   }
#endif  // _LVRT_
   else
      return BTD_NO_PLATFORM;  // Platform specified unknown or not supported.
   /****************************************************************************
   *  Setup the access parameters for QPMC-1553, QPM-1553, R15-AMC, RPCIe-1553.
   *****************************************************************************/
   if (CurrentCardType[cardnum] == (unsigned)QPM1553   ||
       CurrentCardType[cardnum] == (unsigned)R15AMC    ||
       CurrentCardType[cardnum] == (unsigned)RPCIe1553)
   {
      return vbtPageAccessSetupQPMC(cardnum, hostaddr);
   }
   /*********************************************************************
   *  Setup the access parameters for the QPCI-1553 and QPCX-1553.
   *********************************************************************/
   if ((CurrentCardType[cardnum] == (unsigned)QPCX1553) || (CurrentCardType[cardnum] == (unsigned)QPCI1553))
   {
      return vbtPageAccessSetupQPCX(cardnum, hostaddr);
   }
   /*********************************************************************
   *  Setup the access parameters for the QCP-1553.
   *********************************************************************/
   if (CurrentCardType[cardnum] == (unsigned)QCP1553)
   {
      return vbtPageAccessSetupQCP(cardnum, hostaddr);
   }
   /*********************************************************************
   *  Setup the access parameters for the Q104-1553P
   *********************************************************************/
   if ( CurrentCardType[cardnum] == Q1041553P )
   {
      return vbtPageAccessSetupQ104(cardnum, hostaddr);
   }
   /*********************************************************************
   *  Setup the access parameters for the R15-EC.
   *********************************************************************/
   if (CurrentCardType[cardnum] == (unsigned)R15EC)
   {
      return vbtPageAccessSetupR15EC(cardnum, hostaddr);
   }
   /*********************************************************************
   *  Setup the access parameters for the RXMC-1553.
   *********************************************************************/
   if (CurrentCardType[cardnum] == (unsigned)RXMC1553)
   {
      return vbtPageAccessSetupRXMC(cardnum, hostaddr);
   }
   /*********************************************************************
   *  Setup the access parameters for the R15-PMC.
   *********************************************************************/
   if (CurrentCardType[cardnum] == (unsigned)R15PMC)
   {
      return vbtPageAccessSetupR15PMC(cardnum, hostaddr);
   }
   /*********************************************************************
   *  Setup the access parameters for the RXMC2-1553.
   *********************************************************************/
   if (CurrentCardType[cardnum] == (unsigned)R15XMC2)
   {
      return vbtPageAccessSetupRXMC2(cardnum, hostaddr);
   }
   /*********************************************************************
   *  Setup the access parameters for the RAR15-XMC-XT.
   *********************************************************************/
   if (CurrentCardType[cardnum] == (unsigned)RAR15XMCXT)
   {
      return vbtPageAccessSetupRAR15XT(cardnum, hostaddr);
   }
   /*********************************************************************
   *  Setup the access parameters for the R15-LPCIE.
   *********************************************************************/
   if (CurrentCardType[cardnum] == (unsigned)LPCIE1553)
   {
      return vbtPageAccessSetupLPCIE(cardnum, hostaddr);
   }
   /*********************************************************************
   *  Setup the access parameters for the R15-MPCIE.
   *********************************************************************/
   if (CurrentCardType[cardnum] == (unsigned)MPCIE1553)
   {
      return vbtPageAccessSetupMPCIE(cardnum, hostaddr);
   }
#ifdef INCLUDE_PCCD
   /*********************************************************************
   *  Setup the access parameters for the PCC-D1553.
   *********************************************************************/
   if ( CurrentCardType[cardnum] == PCCD1553 )
   {
      return vbtPageAccessSetupPCCD(cardnum, hostaddr);
   }
#endif //INCLUDE_PCCD
#ifdef INCLUDE_USB_SUPPORT
   /*********************************************************************
   *  Setup the access parameters for the USB-1553.
   *********************************************************************/
   if (CurrentCardType[cardnum] == (unsigned)R15USB)
   {
      return vbtPageAccessSetupUSB(cardnum);
   }
#endif
#if defined(INCLUDE_VME_VXI_1553)
   /*********************************************************************
   *  Setup the access parameters for the VME-1553.
   *********************************************************************/
   if ( CurrentCardType[cardnum] == QVME1553)
   {
      return vbtPageAccessSetupVME(cardnum, (unsigned)phys_addr, hostaddr);
   }
#endif

   return BTD_NON_SUPPORT;
}

/****************************************************************************
*
*  PROCEDURE - get_64BitHostTimeTag()
*
*  FUNCTION
*     This routine reads the host clock and returns an API-formatted initial
*     time tag value as a 64-bit integer in microseconds.  We wait for the
*     clock to change to make sure we get as accurate a time value as possible.
*
****************************************************************************/
void get_64BitHostTimeTag(BT_INT mode, BT1553_TIME *host_time)
{
	union bigtime
	{
       CEI_UINT64 rtn_time;
       BT1553_TIME rtn_val;
	}bt;
	
   time_t   now;             // Current time, modify to get Time Reference Point.
   struct   tm start;        // Formatted "now" current time, modified for TRP.
   time_t   then=0;            // Time Reference Point, Midnight or Jan 1.

   struct   timeb ts;        // Current time, use to get milliseconds.

   unsigned last_milli;      // This is the time we use to wait for time to change.
   union{
         CEI_UINT64 diff;            // This is the difference time we return in microseconds,
         BT_U32BIT tt[2];
        }utime;                          //  either time since Midnight or time since Jan 1.
   switch (mode)
   {
   case API_TTI_DAY:  // Initial time relative to midnight
   case API_TTI_DAY64:
      time(&now);               // Time in seconds since midnight Jan 1, 1970 GMT
      start = *localtime(&now); // Local time in seconds, min, hour, day, month
      start.tm_sec  = 0;        // Local
      start.tm_min  = 0;        //       time
      start.tm_hour = 0;        //            at midnight
      then = mktime(&start);    // local midnight to time in seconds since Jan 1, 1970
      break;

   case API_TTI_IRIG:   // Initial time relative to Jan 1, this year
   case API_TTI_IRIG64: // Initial time relative to Jan 1, this year
      time(&now);               // Time in seconds since midnight Jan 1, 1970 GMT
      start = *localtime(&now); // Local
      start.tm_sec  = 0;        //   time
      start.tm_min  = 0;        //     at
      start.tm_hour = 0;        //       Jan 1
      start.tm_mday = 0;        //         midnight (This is day 1)
      start.tm_mon  = 0;        //           this
      start.tm_yday = 0;        //             year
      then = mktime(&start);    // local Jan 1 to time in seconds since Jan 1, 1970
                                // We do IRIG time, which makes Jan 1 "day 1" */
      break;
   }

   ftime(&ts);               // Current time
   last_milli = ts.millitm;  //  save milliseconds   

   utime.diff = ts.time-then;       // We only want the time since Midnight or Jan 1
   utime.diff *= 1000000;           // Number of Seconds converted to microseconds
   utime.diff += ts.millitm * 1000; //  plus number of Milliseconds converted to microseconds.

   if(mode == API_TTI_IRIG64 || mode == API_TTI_DAY64)
   {
      utime.diff*=1000; //convert to nanoseconds
      if(mode==API_TTI_IRIG64)
         bt.rtn_time = (utime.diff-86400000000000);        // Pass it back to the caller.
      else
         bt.rtn_time = utime.diff;
   }
   else
   {
      if(mode==API_TTI_IRIG)
         bt.rtn_time = (utime.diff-86400000000);        // Pass it back to the caller.
      else
         bt.rtn_time = utime.diff;
   }
   
   *host_time = *(BT1553_TIME*)&bt.rtn_val; 
   return;
}

/****************************************************************************
*
*  PROCEDURE - get_48BitHostTimeTag()
*
*  FUNCTION
*     This routine reads the host clock and returns an API-formatted initial
*     time tag value as a 64-bit integer in microseconds.  We wait for the
*     clock to change to make sure we get as accurate a time value as possible.
*
****************************************************************************/
void get_48BitHostTimeTag(BT_INT mode, BT1553_TIME *host_time)
{
	union bigtime
	{
       CEI_UINT64 rtn_time;
       BT1553_TIME rtn_val;
	}bt;
	
   time_t   now;             // Current time, modify to get Time Reference Point.
   struct   tm start;        // Formatted "now" current time, modified for TRP.
   time_t   then=0;            // Time Reference Point, Midnight or Jan 1.

   struct   timeb ts;        // Current time, use to get milliseconds.

   unsigned last_milli;      // This is the time we use to wait for time to change.
   union{
         CEI_UINT64 diff;            // This is the difference time we return in microseconds,
         BT_U32BIT tt[2];
        }utime;                          //  either time since Midnight or time since Jan 1.
   switch (mode)
   {
   case API_TTI_DAY:  // Initial time relative to midnight
      time(&now);               // Time in seconds since midnight Jan 1, 1970 GMT
      start = *localtime(&now); // Local time in seconds, min, hour, day, month
      start.tm_sec  = 0;        // Local
      start.tm_min  = 0;        //       time
      start.tm_hour = 0;        //            at midnight
      then = mktime(&start);    // local midnight to time in seconds since Jan 1, 1970
      break;

   case API_TTI_IRIG: // Initial time relative to Jan 1, this year
      time(&now);               // Time in seconds since midnight Jan 1, 1970 GMT
      start = *localtime(&now); // Local
      start.tm_sec  = 0;        //   time
      start.tm_min  = 0;        //     at
      start.tm_hour = 0;        //       Jan 1
      start.tm_mday = 0;        //         midnight (This is day 1)
      start.tm_mon  = 0;        //           this
      start.tm_yday = 0;        //             year
      then = mktime(&start);    // local Jan 1 to time in seconds since Jan 1, 1970
                                // We do IRIG time, which makes Jan 1 "day 1" */
      break;
   }

   ftime(&ts);               // Current time
   last_milli = ts.millitm;  //  save milliseconds   

   utime.diff = ts.time-then;       // We only want the time since Midnight or Jan 1
   utime.diff *= 1000000;          // Number of Seconds converted to microseconds
   utime.diff += ts.millitm * 1000; //  plus number of Milliseconds converted to microseconds.

   if(mode==API_TTI_IRIG)
      bt.rtn_time = (utime.diff-86400000000);        // Pass it back to the caller.
   else
      bt.rtn_time = utime.diff;

   *host_time = bt.rtn_val; 
   return;
}


/****************************************************************************
*
*  PROCEDURE NAME -    BusTools_SetDumpPath()
*
*  FUNCTION
*       This function creates a environment variable CONDOR_HOME used to set
*       the path to where the dumpfiles are output.
*
*  RETURNS
*       API_SUCCESS
*       API_BAD_PARAM - this return code indicate the path is invalid
****************************************************************************/
NOMANGLE BT_INT CCONV BusTools_SetDumpPath(char * dpath)
{
  BT_INT status=0;
  char dump_path[256];

  memset(dump_path, 0, sizeof(dump_path));	

  if((GetFileAttributes(dump_path) & FILE_ATTRIBUTE_DIRECTORY) == 0)
    return API_BAD_PARAM;

  if((status = CEI_SET_ENV_VAR("CONDOR_HOME", dpath)) != BTD_OK)
    return status;

  // verify that the path was set
  if((status = CEI_GET_ENV_VAR("CONDOR_HOME", dump_path, sizeof(dump_path))) != BTD_OK)
    return status;

  if(strcmp(dpath, dump_path) != 0) 
    return API_BAD_PARAM;	 

  return API_SUCCESS;
}

/****************************************************************************
*
* PROCEDURE NAME:  BusTools_UpdateIntFifo
*
* FUNCTION:
*     This function updates the API_INT_FIFO with user data.  For C#
*     compatibility only.
*
*   Parameters
*
*     API_INT_FIFO *uFifo    = API FIFO pointer
*     API_INT_FIFO *nFifo    = USER FIFO pointer
*
****************************************************************************/
NOMANGLE void CCONV BusTools_UpdateIntFifo(LPVOID uFifo, LPVOID nFifo)
{
   memcpy(uFifo,nFifo, sizeof(API_INT_FIFO));
}

/****************************************************************************
*
* PROCEDURE NAME:  BusTools_UpdateTailPTR
*
* FUNCTION:
*     This function updates the API_INT_FIFO tail pointer.    For C#
*     compatibility only.
*
*   Parameters
*
*     API_INT_FIFO *uFifo    = API FIFO pointer
*     BT_INT       tail      = Tail pointer data
*
****************************************************************************/
NOMANGLE void CCONV BusTools_UpdateTailPTR(LPVOID uFifo, BT_INT tail)
{
   ((API_INT_FIFO *)uFifo)->tail_index = tail;
}


/****************************************************************************
*
* PROCEDURE NAME:  BusTools_CreateIntFIFO
*
* FUNCTION:
*     This function mallocs API_INT_FIFO.  For C# compatibility only.
*   Parameters
*
*     BT_UINT cardnum        = card number (0 - 3)
*     API_INT_FIFO *sIntFIFO = Pointer to Thread Interrupt Filter,
*                              Control and FIFO structure.
*   Returns
*     pointer to interrupt fifo structure.
****************************************************************************/
 
NOMANGLE API_INT_FIFO* CCONV BusTools_CreateIntFifo(BT_INT (_stdcall *function)(BT_UINT cardnum, struct api_int_fifo *pFIFO))
{
   API_INT_FIFO * fifo;

   fifo = (API_INT_FIFO *)CEI_MALLOC(sizeof(API_INT_FIFO));
   memset(fifo,0,sizeof(API_INT_FIFO));
   fifo->function = function;

   return fifo; 
}

/****************************************************************************
*
* PROCEDURE NAME:  BusTools_DestroyIntFifo
*
* FUNCTION:
*     This function frees the API_INT_FIFO created by BusTools_CreateIntFifo.  
*     For C# compatibility only.
*
*   Parameters
*
*     API_INT_FIFO *uFifo    = API FIFO pointer
*     API_INT_FIFO *nFifo    = USER FIFO pointer
*
****************************************************************************/
NOMANGLE void BusTools_DestroyIntFifo(API_INT_FIFO * fifo)
{
   free((void *)fifo);
}

/****************************************************************
*
*  PROCEDURE - CEI_GET_ENV_VAR
*
*  FUNCTION
*     Get the value for the specified environment variable
*
*  RETURNS
*     BTD_OK
*     API_BAD_PARAM
*     API_MEM_ALLOC_ERR
****************************************************************/
CEI_INT CEI_GET_ENV_VAR(CEI_CHAR* name, CEI_CHAR* buf, CEI_INT size) {
  CEI_CHAR* tmp_buf=NULL;
  CEI_INT status, tmp_size=512;

  if((name == NULL) || (buf == NULL) || (size == 0)) 
    return API_BAD_PARAM; 
  
  // if user buffer is larger than default then increase the size
  if(size > tmp_size) 
    tmp_size = size;
  if((tmp_buf = calloc(tmp_size, 1)) == NULL) 
    return API_MEM_ALLOC_ERR;		 

  if((status = GetEnvironmentVariable(name, tmp_buf, tmp_size)) == 0) {
    if(GetLastError() == ERROR_ENVVAR_NOT_FOUND)
      return API_BAD_PARAM;
  }

  if(status > tmp_size)  // buffer is too small for characters in env variable
    return API_BAD_PARAM;

  strncpy(buf, tmp_buf, size); 
	
  free(tmp_buf);
   
  return BTD_OK;
}


/****************************************************************
*
*  PROCEDURE - CEI_SET_ENV_VAR
*
*  FUNCTION
*     Set the specified environment variable with the provide value  
*
*  RETURNS
*     BTD_OK
*     API_BAD_PARAM
****************************************************************/
CEI_INT CEI_SET_ENV_VAR(CEI_CHAR* name, CEI_CHAR* buf) {
  if((name == NULL) || (buf == NULL)) 
    return API_BAD_PARAM; 

  if(SetEnvironmentVariable(name, buf) == 0)
    return API_BAD_PARAM;

  return BTD_OK;
}


/****************************************************************
*
*  PROCEDURE - CEI_GET_FILE_PATH
*
*  FUNCTION
*     Checks that the file that is provided has a valid directory
*
*  RETURNS
*     BTD_OK
*     API_BAD_PARAM
****************************************************************/
CEI_INT CEI_GET_FILE_PATH(CEI_CHAR* name, CEI_CHAR* buf) {
  CEI_UINT status=0;
  CEI_CHAR tmp_buf[512]="";
  CEI_CHAR* tok="\\";

  if(name == NULL)
    return API_BAD_PARAM;

  memset(tmp_buf, 0, sizeof(tmp_buf));
  // check if the user supplied file name contains a path, if so verify  
  // that it is a directory
  if(strchr(name, tok[0]) != NULL) {
    strncpy(tmp_buf, name, strlen(name) - strlen(strrchr(name, tok[0]) + 1)); 
    status = GetFileAttributes(tmp_buf);
    if((status == -1) || ((status & FILE_ATTRIBUTE_DIRECTORY) == 0))
     return API_BAD_PARAM;
    sprintf(buf, "%s", name);
  }
  else {
    // check if file_name contains a path fragment, occurs when the user
    // supplies a path without the necessary escape character
    if(strchr(name, ':') != NULL) 
      return API_BUSTOOLS_NO_FILE; 
    // use the path from the "CONDOR_HOME" environment variable
    if(CEI_GET_ENV_VAR("CONDOR_HOME", tmp_buf, sizeof(tmp_buf)) == BTD_OK) {
      // verify the directory
      if((GetFileAttributes(tmp_buf) & FILE_ATTRIBUTE_DIRECTORY) == 0)
        return API_BAD_PARAM;
	    sprintf(buf, "%s%s%s", tmp_buf, tok, name); 
    }
    else
      sprintf(buf, "%s", name);  // set path to local
  }

  return BTD_OK;
}

/*===========================================================================*
 * EXTERNAL API ENTRY POINT:  v b t A c q u i r e F r a m e R e g i s t e r
 *===========================================================================*
 *
 * FUNCTION:    Aquire or release the CriticalSection for the specified card.
 *
 * DESCRIPTION: This function is used by interrupt service to globally lock
 *              access to the frame register for the duration of the interrupt
 *              routine execution.
 *              This speeds execution of the interrupt function (since it
 *              will not be interrupted by forground threads, and because it
 *              will always find the frame register owned by itself) and it
 *              supports the vbtGetPagePtr() function, which assumes that
 *              the contents of the frame register will not change for the
 *              duration of the execution of it's caller.V4.01.ajh
 *
 *      It will return:
 *              Nothing.
 *===========================================================================*/
void vbtAcquireFrameRegister(
   BT_UINT   cardnum,           // (i) card number.
   BT_UINT   flag)              // (i) 1=Acquire Frame Critical Section,
                                //     0=Release Critical Section.
{
   // If the current card does not have a paged architecture return done.

   if ( bt_inuse[cardnum] < 0 )
      return;      // Software versions of the driver have no frame register.

   // Now acquire or release the Critical Section:
   if ( flag )
      CEI_MUTEX_LOCK(&hFrameCritSect[cardnum]);
   else
      CEI_MUTEX_UNLOCK(&hFrameCritSect[cardnum]);
}
